@@ -3,7 +3,7 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2025-12-16
+ *  Updated   : 2026-01-20
  *  Notes     :
  *
  * Copyright (c) 1997-2026 Iain Lea <iain@bricbrac.de>
@@ -465,6 +465,7 @@ extern constext *txt_quick_ks_header_options[];
 extern constext *txt_quote_style_type[];
 extern constext *txt_show_from[];
 extern constext *txt_show_help_mail_sign_options[];
+extern constext *txt_show_article_info[];
 extern constext *txt_sort_a_type[];	/* a=articles */
 extern constext *txt_sort_t_type[];	/* t=threads */
 extern constext *txt_strip_bogus_type[];
@@ -612,7 +613,7 @@ extern constext txt_attrib_file_savefile[];
 extern constext txt_attrib_file_sigfile[];
 extern constext txt_attrib_file_show_author[];
 extern constext txt_attrib_file_show_signatures[];
-extern constext txt_attrib_file_show_art_score[];
+extern constext txt_attrib_file_show_art_info[];
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern constext txt_attrib_file_suppress_soft_hyphens[];
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -972,7 +973,6 @@ extern constext txt_filesystem_full_backup[];
 extern constext txt_filter_comment[];
 #ifdef DEBUG
 	extern constext txt_filter_error_overview_no_servername[];
-	extern constext txt_filter_error_overview_xref[];
 	extern constext txt_filter_error_skipping_xref_filter[];
 #endif /* DEBUG */
 extern constext txt_filter_file[];
@@ -1713,6 +1713,7 @@ extern constext txt_usage_savedir[];
 extern constext txt_usage_save_new_news[];
 extern constext txt_usage_start_if_unread_news[];
 extern constext txt_usage_tin[];
+extern constext txt_usage_tinrc_file[];
 #ifdef HAVE_COLOR
 	extern constext txt_usage_toggle_color[];
 #endif /* HAVE_COLOR */
@@ -2315,7 +2316,7 @@ extern struct opttxt txt_show_help_mail_sign;
 extern struct opttxt txt_show_only_unread_arts;
 extern struct opttxt txt_show_only_unread_groups;
 extern struct opttxt txt_show_signatures;
-extern struct opttxt txt_show_art_score;
+extern struct opttxt txt_show_art_info;
 extern struct opttxt txt_sigdashes;
 extern struct opttxt txt_sigfile;
 extern struct opttxt txt_signature_repost;
@@ -2423,4 +2424,7 @@ extern struct opttxt txt_xpost_quote_format;
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern struct opttxt txt_render_bidi;
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
+#ifdef HAVE_LIBEXTTEXTCAT
+	extern void *textcat_handle;
+#endif /* HAVE_LIBEXTTEXTCAT */
 #endif /* !EXTERN_H */
