@@ -3,10 +3,10 @@
  *  Module    : policy.h
  *  Author    : Ralf Doeblitz <doeblitz@gmx.de>
  *  Created   : 1999-01-12
- *  Updated   : 2025-08-27
+ *  Updated   : 2026-02-17
  *  Notes     : #defines and static data for policy configuration
  *
- * Copyright (c) 1999-2025 Ralf Doeblitz <doeblitz@gmx.de>
+ * Copyright (c) 1999-2026 Ralf Doeblitz <doeblitz@gmx.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -360,7 +360,6 @@ static const char *gnksa_domain_list[] = {
 	"gle",
 	"gmo",
 	"gmx",
-	"goo",
 	"gop",
 	"got",
 	"gov",
@@ -902,7 +901,6 @@ static const char *gnksa_domain_list[] = {
 	"design",
 	"direct",
 	"doctor",
-	"dunlop",
 	"dupont",
 	"durban",
 	"emerck",
@@ -1424,7 +1422,6 @@ static const char *gnksa_domain_list[] = {
 	"xn--qcka1pmc",
 	"international",
 	"lifeinsurance",
-	"wolterskluwer",
 	"xn--cckwcxetd",
 	"xn--eckvdtc9d",
 	"xn--fpcrj9c3d",
