@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2025-07-29
+ *  Updated   : 2026-01-19
  *  Notes     :
  *
- * Copyright (c) 1997-2025 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2026 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,7 +58,7 @@ extern t_bool process_bogus(const char *name);
 extern t_bool need_reread_active_file(void);
 extern t_bool resync_active_file(void);
 extern void create_save_active_file(void);
-extern void load_newnews_info(char *info);
+extern void load_newnews_info(const char *info);
 extern void active_add(struct t_group *ptr, t_artnum count, t_artnum max, t_artnum min, const char *moderated);
 
 /* art.c */
@@ -66,18 +66,18 @@ extern int find_artnum(t_artnum art);
 extern int global_get_multipart_info(int aindex, MultiPartInfo *setme);
 extern int global_get_multiparts(int aindex, MultiPartInfo **malloc_and_setme_info, t_bool tagging);
 extern t_bool global_look_for_multipart(int aindex, char start, char stop);
-extern t_bool index_group(struct t_group *group);
+extern t_bool index_group(struct t_group *group, t_bool flag);
 extern void do_update(t_bool catchup);
 extern void find_base(const struct t_group *group);
 extern void free_mailbox_list(struct t_mailbox *mb);
-extern void make_threads(struct t_group *group, t_bool rethread);
+extern void make_threads(const struct t_group *group, t_bool rethread);
 extern void set_article(struct t_article *art);
 extern void show_art_msg(const char *group);
-extern void sort_arts(unsigned /* int */ sort_art_type);
 extern void write_overview(struct t_group *group);
 
 /* attrib.c */
 extern int add_scope(const char *scope);
+extern t_bool getart_limit_set_from_scope(struct t_group *group);
 extern t_bool read_attributes_file(t_bool global_file);
 extern void assign_attributes_to_groups(void);
 extern void build_news_headers_array(struct t_attribute *scope, t_bool header_to_display);
@@ -101,7 +101,7 @@ extern char *convert_to_printable(char *buf, t_bool keep_tab);
 extern t_bool charset_unsupported(const char *charset);
 extern t_bool charset_compare(const char *cs_a, const char *cs_b);
 extern void convert_iso2asc(char *iso, char **asc_buffer, size_t *max_line_len, int t);
-extern void convert_tex2iso(char *from, char *to);
+extern void convert_tex2iso(const char *from, char *to);
 extern void csguess_convert_str(char **line, const char *def_cs);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern wchar_t *wconvert_to_printable(wchar_t *wbuf, t_bool keep_tab);
@@ -122,12 +122,12 @@ extern void draw_pager_line(const char *str, unsigned int flags, t_bool raw_data
 extern char **ulBuildArgv(const char *cmd, int *new_argc);
 extern char *quote_space_to_dash(const char *str);
 extern const char *print_boolean(t_bool value);
-extern t_bool match_boolean(char *line, const char *pat, t_bool *dst);
+extern t_bool match_boolean(const char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer(const char *line, const char *pat, int *dst, int maxval);
-extern t_bool match_list(char *line, constext *pat, constext *const *table, int *dst);
+extern t_bool match_list(const char *line, constext *pat, constext *const *table, int *dst);
 extern t_bool match_long(const char *line, const char *pat, long *dst);
-extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen);
-extern t_bool match_string_ptr(char *line, const char *pat, char **dst);
+extern t_bool match_string(const char *line, const char *pat, char *dst, size_t dstlen);
+extern t_bool match_string_ptr(const char *line, const char *pat, char **dst);
 extern t_bool read_config_file(char *file, t_bool global_file);
 extern void quote_dash_to_space(char *str);
 extern void read_server_config(unsigned int option_mask);
@@ -200,13 +200,13 @@ extern void word_highlight_string(int row, int col, int size, int color);
 extern void envargs(int *Pargc, char ***Pargv, const char *envstr);
 
 /* feed.c */
-extern int feed_articles(int function, int level, t_function type, struct t_group *group, int respnum);
+extern int feed_articles(int function, int level, t_function type, const struct t_group *group, int respnum);
 
 /* filter.c */
 extern t_bool filter_articles(struct t_group *group);
 extern t_bool filter_menu(t_function type, struct t_group *group, struct t_article *art);
 extern t_bool filter_on_path(struct t_group *group);
-extern t_bool quick_filter(t_function type, struct t_group *group, struct t_article *art);
+extern t_bool quick_filter(t_function type, const struct t_group *group, struct t_article *art);
 extern t_bool quick_filter_select_posted_art(const struct t_group *group, const char *subj, const char *a_message_id);
 extern t_bool read_filter_file(const char *file);
 extern void free_filter_array(struct t_filters *ptr);
@@ -270,7 +270,7 @@ extern const char *get_host_name(void);
 #endif /* !HAVE_HEAPSORT */
 
 /* inews.c */
-extern t_bool submit_news_file(char *name, struct t_group *group, char *a_message_id);
+extern t_bool submit_news_file(char *name, const struct t_group *group, char *a_message_id);
 extern void get_from_name(char *from_name, size_t len, const struct t_group *thisgrp);
 
 /* init.c */
@@ -341,6 +341,7 @@ extern void free_all_arrays(void);
 extern void free_art_array(void);
 extern void free_save_array(void);
 extern void free_scope(int num);
+extern void free_scopes_list(struct t_group *group);
 extern void *my_malloc1(const char *file, int line, size_t size);
 extern void *my_calloc1(const char *file, int line, size_t nmemb, size_t size);
 extern void *my_realloc1(const char *file, int line, void *p, size_t size);
@@ -360,9 +361,9 @@ extern FILE *tin_fopen(const char *pathname, const char *mode);
 extern char *buffer_to_ascii(char *c);
 extern char *escape_shell_meta(const char *source, enum quote_enum quote_area);
 extern char *get_tmpfilename(const char *filename);
-extern char *idna_decode(char *in);
-extern char *quote_wild(char *str);
-extern char *quote_wild_whitespace(char *str);
+extern char *idna_decode(const char *in);
+extern char *quote_wild(const char *str);
+extern char *quote_wild_whitespace(const char *str);
 extern char *random_organization(char *in_org);
 extern char *strip_line(char *line);
 extern char *split_mailbox_list(char *from);
@@ -380,7 +381,7 @@ extern int rndm(void);
 extern int sync_close(int fd);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group, t_bool expand_all);
-extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
+extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, const char *format);
 extern int tin_version_info(FILE *fp, int verb);
 extern int tin_gettime(struct t_tintime *tt);
 extern long file_mtime(const char *file);
@@ -496,7 +497,7 @@ extern void tintls_exit(void);
 /* nrctbl.c */
 extern t_bool get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
 #ifdef NNTP_ABLE
-	extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, char *nick_name);
+	extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, const char *nick_name);
 #endif /* NNTP_ABLE */
 
 /* options_menu.c */
@@ -587,7 +588,7 @@ extern char *tin_fgets(FILE *fp, t_bool header);
 extern char *get_references(struct t_msgid *refptr);
 extern struct t_msgid *find_msgid(const char *msgid);
 extern t_bool valid_msgid(char *msgid);
-extern void build_references(const struct t_group *group);
+extern void build_references(void);
 extern void clear_art_ptrs(void);
 extern void collate_subjects(void);
 extern void free_msgids(void);
@@ -636,9 +637,9 @@ extern char *rfc1522_decode(const char *s);
 extern char *rfc1522_encode(char *s, const char *charset, t_bool ismail);
 extern int mmdecode(const char *what, int encoding, int delimiter, char *where);
 extern int b642str(const char *in, char *out);
-extern void rfc15211522_encode(const char *filename, constext *mime_encoding, struct t_group *group, t_bool allow_8bit_header, t_bool ismail);
-extern void compose_mail_mime_forwarded(const char *filename, FILE *articlefp, t_bool include_text, struct t_group *group);
-extern void compose_mail_text_plain(const char *filename, struct t_group *group);
+extern void rfc15211522_encode(const char *filename, constext *mime_encoding, const struct t_group *group, t_bool allow_8bit_header, t_bool ismail);
+extern void compose_mail_mime_forwarded(const char *filename, FILE *articlefp, t_bool include_text, const struct t_group *group);
+extern void compose_mail_text_plain(const char *filename, const struct t_group *group);
 
 /* save.c */
 extern int check_start_save_any_news(int function, t_bool catchup, int num_cmd_line_groups);
@@ -769,7 +770,7 @@ extern int my_toupper(int);
 #else
 	extern char *abbr_groupname(const char *grpname, size_t len);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-extern char *expand_tab(char *str, size_t tab_width);
+extern char *expand_tab(const char *str, size_t tab_width);
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
@@ -780,6 +781,9 @@ extern char *expand_tab(char *str, size_t tab_width);
 	extern UChar *char2UChar(const char *str);
 	extern char *UChar2char(const UChar *ustr);
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && !NO_LOCALE */
+#if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H) && defined(HAVE_LIBEXTTEXTCAT)
+	extern char *tag2lang(const char *tag);
+#endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H && HAVE_LIBEXTTEXTCAT */
 
 /* tags.c */
 extern int line_is_tagged(int n);
@@ -832,6 +836,6 @@ extern t_bool wildmatpos(const char *text, char *p, t_bool icase, REGEX_SIZE *sr
 extern t_bool overview_xref_support(void);
 extern void NSETRNG0(t_bitmap *bitmap, t_artnum low, t_artnum high);
 extern void NSETRNG1(t_bitmap *bitmap, t_artnum low, t_artnum high);
-extern void art_mark_xref_read(struct t_article *art);
+extern void art_mark_xref_read(const struct t_article *art);
 
 #endif /* !PROTO_H */
