@@ -3,10 +3,10 @@
  *  Module    : rfc2046.h
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-02-18
- *  Updated   : 2025-05-09
+ *  Updated   : 2026-02-01
  *  Notes     : RFC 2046 MIME article definitions
  *
- * Copyright (c) 2000-2025 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2026 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -90,16 +90,19 @@
 #	define MIME_CHARSET_UNSUPPORTED			(1 << 10)
 #	define MIME_TRANSFER_ENCODING_MISSING	(1 << 11)
 #	define MIME_TRANSFER_ENCODING_UNKNOWN	(1 << 12)
+#	define MIME_LANGUAGE_MISSING		(1 << 13)
+#	define MIME_LANGUAGE_GUESSED		(1 << 14)
 #	define MIME_INIT						(MIME_VERSION_MISSING \
 											| MIME_VERSION_UNSUPPORTED \
 											| MIME_TYPE_MISSING \
 											| MIME_SUBTYPE_MISSING \
 											| MIME_CHARSET_MISSING \
-											| MIME_TRANSFER_ENCODING_MISSING)
+											| MIME_TRANSFER_ENCODING_MISSING \
+											| MIME_LANGUAGE_MISSING)
 
 typedef struct hints
 {
-	unsigned flags:13;	/* Result of mime parsing
+	unsigned flags:15;	/* Result of mime parsing
 						define MIME_OK							no hints, to check if (mime_hints) {}
 						define MIME_VERSION_MISSING				mime version missing
 						define MIME_VERSION_UNSUPPORTED			mime version != 1.0
@@ -151,6 +154,7 @@ typedef struct part
 	long offset;			/* offset in article of the text of attachment */
 	unsigned long bytes;	/* part size in bytes */
 	int line_count;			/* # lines in this part */
+	int id;					/* section the current part belongs to */
 	int depth;				/* For multipart within multipart */
 	int yenc_part;			/* current yenc part */
 	int yenc_total;			/* total # of yenc parts */
