@@ -3,7 +3,7 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-11-17
+ *  Updated   : 2026-02-14
  *  Notes     : #include files, #defines & struct's
  *
  * Copyright (c) 1997-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -211,13 +211,11 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	if defined(HAVE_INT_LEAST64_T) && !defined(HAVE_INTTYPES_H)
 #		undef HAVE_INT_LEAST64_T
 #	endif /* HAVE_INT_LEAST64_T && !HAVE_INTTYPES_H */
-#	if defined(HAVE_INT_LEAST64_T) || defined(HAVE_LONG_LONG) || defined(quad_t)
-#		if defined(HAVE_ATOLL) || defined(HAVE_ATOQ)
-#			ifdef HAVE_STRTOLL
-#				define USE_LONG_ARTICLE_NUMBERS 1
-#			endif /* HAVE_STRTOLL */
-#		endif /* HAVE_ATOLL || HAVE_ATOQ */
-#	endif /* HAVE_INT_LEAST64_T || HAVE_LONG_LONG || quad_t */
+#	if defined(HAVE_STRTOLL) && (defined(HAVE_INT_LEAST64_T) || defined(HAVE_LONG_LONG))
+#		define USE_LONG_ARTICLE_NUMBERS 1
+#		define strtoartnum(in, end) strtoll(in, end, 10)
+#		define atoartnum(in) strtoll(in, NULL, 10)
+#	endif /* HAVE_STRTOLL && (HAVE_INT_LEAST64_T || HAVE_LONG_LONG) */
 #endif /* ENABLE_LONG_ARTICLE_NUMBERS && !SMALL_MEMORY_MACHINE */
 #ifdef USE_LONG_ARTICLE_NUMBERS
 #	if defined(HAVE_INT_LEAST64_T) && defined(HAVE_PRIDLEAST64) && defined(HAVE_SCNDLEAST64) && defined(HAVE_INT64_C)
@@ -238,12 +236,6 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #		endif /* CPP_DOES_CONCAT */
 #		define ARTNUM_MAX LLONG_MAX
 #	endif /* HAVE_INT_LEAST64_T && HAVE_PRIDLEAST64 && HAVE_SCNDLEAST64 && HAVE_INT64_C */
-#	ifdef HAVE_ATOLL
-#		define atoartnum atoll
-#	else
-#		define atoartnum atoq
-#	endif /* HAVE_ATOLL */
-#	define strtoartnum strtoll
 #else
 	typedef long t_artnum;
 #	define T_ARTNUM_PFMT "ld"
@@ -254,8 +246,8 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #		define T_ARTNUM_CONST(v) (long)v
 #	endif /* CPP_DOES_CONCAT */
 #	define ARTNUM_MAX LONG_MAX
-#	define atoartnum atol
-#	define strtoartnum strtol
+#	define atoartnum strtol(in, NULL, 10)
+#	define strtoartnum(in, end) strtol(in, end, 10)
 #endif /* USE_LONG_ARTICLE_NUMBERS */
 
 /*
@@ -451,6 +443,9 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #if !defined(INT_MIN)
 #	define INT_MIN (-INT_MAX - 1)
 #endif /* INT_MIN */
+#if !defined(INT32_MAX)
+#	define INT32_MAX 2147483647 /* 2^31-1 */
+#endif /* !INT32_MAX */
 
 /*
  * If OS misses the isascii() function
@@ -1175,6 +1170,12 @@ enum {
  */
 #define MAX_MARK	2
 
+/*
+ * Maximal article info in pager
+ * 0 = nothing, 1 = score, 2 = language, 3 = both
+ */
+#define MAX_ART_INFO	3
+
 /* Line number (starting at 0) of 1st non-header data on the screen */
 /* ie, size of header */
 #define INDEX_TOP	2
@@ -1417,6 +1418,7 @@ enum {
 
 
 /* RFC 9580 6.2.1 / 6.2.3 */
+#define PGP_SIGNED_TAG "-----BEGIN PGP SIGNED MESSAGE-----\n"
 #define PGP_PUBLIC_KEY_TAG "-----BEGIN PGP PUBLIC KEY BLOCK-----\n"
 #define PGP_PUBLIC_KEY_END_TAG "-----END PGP PUBLIC KEY BLOCK-----\n"
 #if 0 /* we do not look for these */
@@ -1668,6 +1670,9 @@ enum {
 #	ifdef HAVE_UNICODE_UBIDI_H
 #		include <unicode/ubidi.h>
 #	endif /* HAVE_UNICODE_UBIDI_H */
+#	ifdef HAVE_UNICODE_UCOL_H
+#		include <unicode/ucol.h>
+#	endif /* HAVE_UNICODE_UCOL_H */
 #endif /* HAVE_LIBICUUC */
 
 #define ACE_PREFIX "xn--"
@@ -1816,6 +1821,7 @@ struct t_newsheader {
 #	define BITS_OF_quick_kill_header	3
 #	define BITS_OF_quick_select_header	3
 #	define BITS_OF_show_author			2
+#	define BITS_OF_show_art_info	2
 #	define BITS_OF_sort_article_type	4
 #	define BITS_OF_sort_threads_type	3
 #	define BITS_OF_thread_articles		3
@@ -1834,6 +1840,7 @@ struct t_newsheader {
 		quick_kill_header   = 3,
 		quick_select_header = 3,
 		show_author         = 2,
+		show_art_info       = 2,
 		sort_article_type   = 4,
 		sort_threads_type   = 3,
 		thread_articles     = 3,
@@ -1947,7 +1954,7 @@ struct t_attribute {
 	IntField(thread_perc);			/* percentage threading threshold */
 	IntField(show_author);			/* 0=none, 1=name, 2=addr, 3=both */
 	BoolField(show_signatures);		/* 0=none, 1=show signatures */
-	BoolField(show_art_score);		/* 0=none, 1=show score */
+	IntField(show_art_info);		/* 0=none, 1=show score, 2=show language, 3=show both */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	BoolField(suppress_soft_hyphens);	/* set TRUE to remove soft hyphens (U+00AD) from articles */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -2046,7 +2053,7 @@ struct t_attribute_state {
 	BoolField(show_author);
 	BoolField(show_only_unread_arts);
 	BoolField(show_signatures);
-	BoolField(show_art_score);
+	IntField(show_art_info);
 	BoolField(sigdashes);
 	BoolField(sigfile);
 	BoolField(signature_repost);
