@@ -3,10 +3,10 @@
  *  Module    : tinrc.h
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-04-13
- *  Updated   : 2025-02-17
+ *  Updated   : 2025-11-17
  *  Notes     :
  *
- * Copyright (c) 1999-2025 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2026 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -80,6 +80,7 @@ struct t_config {
 	char *attachment_format;			/* format string for the attachment level */
 	char *page_mime_format;				/* format string for the mime header in page level */
 	char *page_uue_format;				/* format string for the uue header in page level */
+	char *page_yenc_format;				/* format string for the yenc header in page level */
 	char *date_format;					/* format string for the date display in the page header */
 	char *mail_quote_format;
 	char *mailer_format;				/* mailer + parameters  %M %S %T %F */
@@ -111,6 +112,7 @@ struct t_config {
 	char *underscores_regex;			/* regex used to highlight _underscores_ */
 	char *verbatim_begin_regex;			/* regex used to find the begin of a verbatim block */
 	char *verbatim_end_regex;			/* regex used to find the end of a verbatim block */
+	char *hideline_regex;				/* regex to skip over matching lines */
 	char *savedir;						/* directory to save articles to */
 	char *sigfile;
 	char *spamtrap_warning_addresses;
@@ -165,7 +167,7 @@ struct t_config {
 	int default_save_mode;				/* Append/Overwrite existing file when saving */
 	int getart_limit;					/* number of article to get */
 	int goto_next_unread;				/* jump to next unread article with SPACE|PGDN|TAB */
-	int hide_uue;						/* treatment of uuencoded data in pager */
+	int hide_inline_data;				/* handling of none text in pager */
 	int interactive_mailer;				/* invoke user's mailreader */
 	int kill_level;						/* Define how killed articles are shown */
 	int mono_markdash;					/* attribute for text highlighting with _underdashes_ */
@@ -193,6 +195,7 @@ struct t_config {
 	int score_kill;						/* default score for "kill" filter rules */
 	int score_select;					/* default score for "hot" filter rules */
 	int scroll_lines;					/* # lines to scroll by in pager */
+	int show_art_info;					/* show article info when displaying articles (score, language) */
 	int show_author;					/* show_author value from 'M' menu in tinrc */
 	int show_help_mail_sign;			/* show help/mail sign in level title */
 	int sort_article_type;				/* method used to sort arts[] */
@@ -233,6 +236,7 @@ struct t_config {
 	t_bool info_in_last_line;
 	t_bool inverse_okay;
 	t_bool keep_dead_articles;			/* keep all dead articles in dead.articles */
+	t_bool keep_expired_filters;		/* keep expired filter rules in file */
 	t_bool mail_8bit_header;			/* allow 8bit chars. in header of mail message */
 	t_bool mark_ignore_tags;			/* Ignore tags for GROUP_MARK_THREAD_READ/THREAD_MARK_ARTICLE_READ */
 	t_bool mark_saved_read;				/* mark saved article/thread as read */
@@ -248,7 +252,6 @@ struct t_config {
 	t_bool show_only_unread_arts;		/* show only new/unread arts or all arts */
 	t_bool show_only_unread_groups;		/* set TRUE to see only subscribed groups with new news */
 	t_bool show_signatures;				/* show signatures when displaying articles */
-	t_bool show_art_score;				/* show article score when displaying articles */
 	t_bool sigdashes;					/* set TRUE to prepend every signature with dashes */
 	t_bool signature_repost;			/* set TRUE to add signature when reposting articles */
 #	ifndef USE_CURSES
@@ -290,7 +293,7 @@ struct t_config {
 	t_bool word_highlight;				/* like word_highlight but stored in tinrc */
 	t_bool wrap_on_next_unread;			/* Wrap around threads when searching next unread article */
 	/*
-	 * Chars used to show article status
+	 * Chars used to show article/scope status
 	 */
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		wchar_t art_marked_deleted;
@@ -302,6 +305,10 @@ struct t_config {
 		wchar_t art_marked_read;
 		wchar_t art_marked_killed;
 		wchar_t art_marked_read_selected;
+		wchar_t scope_marked_global;
+		wchar_t scope_marked_global_match;
+		wchar_t scope_marked_local;
+		wchar_t scope_marked_local_match;
 #	else
 		char art_marked_deleted;
 		char art_marked_inrange;		/* misnomer, as it's also used for other ranges */
@@ -312,6 +319,10 @@ struct t_config {
 		char art_marked_read;
 		char art_marked_killed;
 		char art_marked_read_selected;
+		char scope_marked_global;
+		char scope_marked_global_match;
+		char scope_marked_local;
+		char scope_marked_local_match;
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	char *attrib_editor_format;
 	char *attrib_fcc;
@@ -345,6 +356,7 @@ struct t_config {
 		t_bool attrib_undeclared_cs_guess;
 #		endif /* USE_ICU_UCSDET */
 #	endif /* !CHARSET_CONVERSION */
+	int attrib_hide_inline_data;
 	int attrib_trim_article_body;
 	int attrib_verbatim_handling;
 	int attrib_auto_cc_bcc;
@@ -354,10 +366,12 @@ struct t_config {
 	int attrib_post_mime_encoding;
 	int attrib_post_process_type;
 	int attrib_show_author;
+	int attrib_show_art_info;
 	int attrib_sort_article_type;
 	int attrib_sort_threads_type;
 	int attrib_thread_articles;
 	int attrib_thread_perc;
+	int attrib_getart_limit;
 	t_bool attrib_add_posted_to_filter;
 	t_bool attrib_advertising;
 	t_bool attrib_alternative_handling;
@@ -380,7 +394,6 @@ struct t_config {
 	t_bool attrib_prompt_followupto;
 	t_bool attrib_show_only_unread_arts;
 	t_bool attrib_show_signatures;
-	t_bool attrib_show_art_score;
 	t_bool attrib_sigdashes;
 	t_bool attrib_signature_repost;
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -398,6 +411,31 @@ struct t_config {
 	t_bool attrib_quick_kill_expire;
 	t_bool attrib_quick_select_case;
 	t_bool attrib_quick_select_expire;
+	char *serverrc_add_cmd_line_opts;
+	char *serverrc_disabled_nntp_cmds;
+	int serverrc_nntp_pipeline_limit;
+	t_bool serverrc_cache_overview_files;
+#	ifdef USE_ZLIB
+	t_bool serverrc_compress_overview_files;
+#	endif /* USE_ZLIB */
+};
+
+
+#	define SRVRC_MASK_UPDATE_INDEX		(1 << 0)	/* cmd.-line -u vs. serverrc -G */
+#	define SRVRC_MASK_READ_SAVED_NEWS	(1 << 1)	/* cmd.-line -R	vs. serverrc -t */
+struct t_serverrc {
+#	if 0 /* we don't need the outside read_server_config() */
+	char *version;
+	char *last_newnews;
+#	endif /* 0 */
+	char *add_cmd_line_opts;
+	char *disabled_nntp_cmds;
+	long motd_hash;
+	int nntp_pipeline_limit;
+	t_bool cache_overview_files;
+#	ifdef USE_ZLIB
+	t_bool compress_overview_files;
+#	endif /* USE_ZLIB */
 };
 
 #endif /* !TINRC_H */
