@@ -3,7 +3,7 @@
  *  Module    : tnntps.h
  *  Author    : Enrik Berkhan
  *  Created   : 2022-09-10
- *  Updated   : 2022-12-13
+ *  Updated   : 2026-02-14
  *  Notes     : TLS #include files, #defines & struct's
  *
  * Copyright (c) 2022-2026 Enrik Berkhan <Enrik.Berkhan@inka.de>
@@ -60,7 +60,7 @@
 #			include <openssl/x509.h>
 #		endif /* HAVE_OPENSSL_X509_H */
 
-#		if TLS_API < 20200120
+#		if defined(TLS_API) && TLS_API < 20200120
 #			error "Please use LibreSSL TLS_API >= 20200120"
 #		else
 #			define USE_LIBTLS 1
@@ -73,7 +73,7 @@
 #			include <openssl/err.h>
 #			include <openssl/rand.h>
 
-#			if OPENSSL_VERSION_NUMBER < 0x1010100fL
+#			if defined(OPENSSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER < 0x1010100fL
 #				error "Please use OpenSSL >= 1.1.1"
 #			else
 #				define USE_OPENSSL 1
@@ -85,7 +85,7 @@
 #				include <gnutls/gnutls.h>
 #				include <gnutls/x509.h>
 
-#				if GNUTLS_VERSION_NUMBER < 0x030700
+#				if defined(GNUTLS_VERSION_NUMBER) && GNUTLS_VERSION_NUMBER < 0x030700
 #					error "Please use GnuTLS >= 3.7.0"
 #				else
 #					define USE_GNUTLS 1
