@@ -6,7 +6,7 @@
  *  Updated   : 2021-02-01
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,13 +43,13 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"2.6.5"
+#		define VERSION		"2.6.6"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20250919"
-#	define RELEASENAME	"Helmsdale"
+#	define RELEASEDATE	"20260222"
+#	define RELEASENAME	"Glenury"
 /* config-file versions - must be dotted triples in the range 0 to 99 each */
-#	define TINRC_VERSION	"1.3.20"
-#	define ATTRIBUTES_VERSION	"1.0.13"
+#	define TINRC_VERSION	"1.3.21"
+#	define ATTRIBUTES_VERSION	"1.0.14"
 #	define FILTER_VERSION	"1.0.2"
 #	define KEYMAP_VERSION	"1.0.12"
 #	define SERVERCONFIG_VERSION	"1.0.1"
