@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-07-03
+ *  Updated   : 2026-02-06
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,7 +72,7 @@
 /*
  * TODO: fixup to remove CURR_GROUP dependency in all sort funcs
  */
-#define SortBy(func)	tin_sort(arts, (size_t) top_art, sizeof(struct t_article), func);
+#define SortBy(func)	tin_sort(arts, (size_t) top_art, sizeof(struct t_article), func)
 
 int top_art = 0;				/* # of articles in arts[] */
 
@@ -94,7 +94,7 @@ static int last_date_comp_base_desc(t_comptype p1, t_comptype p2);
 static int lines_comp_asc(t_comptype p1, t_comptype p2);
 static int lines_comp_desc(t_comptype p1, t_comptype p2);
 static int read_art_headers(const struct t_group *group, int total, t_artnum top);
-static int read_overview(struct t_group *group, t_artnum min, t_artnum max, t_artnum *top, t_bool local, t_bool *rebuild_cache);
+static int read_overview(struct t_group *group, t_artnum min, t_artnum max, t_artnum *top, t_bool local, t_bool *rebuild_cache, t_bool flag);
 static int score_comp_asc(t_comptype p1, t_comptype p2);
 static int score_comp_desc(t_comptype p1, t_comptype p2);
 static int score_comp_base(t_comptype p1, t_comptype p2);
@@ -107,6 +107,7 @@ static t_bool parse_headers(FILE *fp, struct t_article *h);
 static t_compfunc eval_sort_arts_func(unsigned int sort_art_type);
 static time_t get_last_posting_date(t_artnum n);
 static void build_mailbox_list(struct t_article *art, const char *hdr);
+static void sort_arts(unsigned int sort_art_type);
 static void sort_base(unsigned int sort_threads_type);
 static void thread_by_multipart(void);
 static void thread_by_percentage(unsigned int percentage);
@@ -126,7 +127,9 @@ void
 show_art_msg(
 	const char *group)
 {
-	wait_message(0, _(txt_group), cCOLS - (strwidth(_(txt_group)) > cCOLS ? cCOLS : strwidth(_(txt_group)) + 2 - 3), group);
+	int n = strwidth(_(txt_group));
+
+	wait_message(0, _(txt_group), cCOLS - (n > cCOLS ? cCOLS : n + 2 - 3), group);
 }
 
 
@@ -234,12 +237,12 @@ setup_hard_base(
 #ifdef NNTP_ABLE
 		char buf[NNTP_STRLEN];
 		char line[NNTP_STRLEN];
-		int getart_limit = (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
+		int getart_limit = getart_limit_set_from_scope(group) ? group->attribute->getart_limit : (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
 		FILE *fp;
 		t_artnum last, start, count = T_ARTNUM_CONST(0), j = T_ARTNUM_CONST(0);
 		static t_bool skip_listgroup = FALSE;
 
-		if (newsrc_active && !list_active) /* -n */
+		if (!skip_listgroup && ((newsrc_active && !list_active) || !nntp_caps.listgroup))
 			skip_listgroup = TRUE;
 
 		if (getart_limit > 0)
@@ -250,52 +253,11 @@ setup_hard_base(
 			j = group->xmin;
 
 		/*
-		 * Some nntp servers are broken and need an extra GROUP command
-		 * (reported by reorx@irc.pl). This affects (old?) versions of
-		 * nntpcache, leafnode and SurgeNews. Usually this should not be
-		 * needed.
-		 *
 		 * For getart_limit recheck lowwatermark as at least giganews gives
 		 * very different results for LIST ACTIVE (3 year retention for all)
 		 * and GROUP (based on the clients contract).
 		 * Calculate range and prepare base[] not to lose unread arts.
 		 */
-		if (nntp_caps.broken_listgroup) {
-			snprintf(buf, sizeof(buf), "GROUP %s", group->name);
-			if (nntp_command(buf, OK_GROUP, line, sizeof(line)) == NULL)
-				return -1;
-
-			if (sscanf(line, "%"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT, &count, &start, &last) != 3)
-				return -1;
-
-#	ifdef DEBUG
-			if ((debug & DEBUG_NNTP) && verbose > 1)
-				debug_print_file("NNTP", "setup_hard_base(GROUP %s %d-%d (%d))", buf, start, last, count);
-#	endif /* DEBUG */
-
-			total = count;
-			grpmenu.max = 0;
-
-			if (getart_limit > 0 && group->xmax > getart_limit) {
-				if ((j = find_first_unread(group)) > start) {
-					start = MIN(j, group->xmax - getart_limit);
-					total = getart_limit;
-				}
-			}
-
-			if (getart_limit < 0) {
-				if ((j = (getart_limit + find_first_unread(group))) > start)
-					if (j < group->xmax)
-						start = j;
-			}
-
-			while (start <= last) {
-				if (grpmenu.max >= max_base)
-					expand_base();
-				base[grpmenu.max++] = start++;
-			}
-		}
-
 		if (!skip_listgroup) {
 			if (getart_limit != 0 && nntp_caps.type == CAPABILITIES && nntp_caps.reader) /* RFC 3977 allows ranges in LISTGROUP */
 				snprintf(buf, sizeof(buf), "LISTGROUP %s %"T_ARTNUM_PFMT"-", group->name, j); /* shorthand for j "-" group->xmax */
@@ -332,7 +294,7 @@ setup_hard_base(
 				skip_listgroup = TRUE;
 		}
 
-		if (!nntp_caps.broken_listgroup && (skip_listgroup || (newsrc_active && list_active))) { /* LISTGROUP was skipped or failed or -ln */
+		if (skip_listgroup || (newsrc_active && list_active)) { /* LISTGROUP was skipped or failed or -ln */
 			/*
 			 * Handle the obscure case that the user aborted before the LISTGROUP
 			 * had a chance to respond
@@ -341,7 +303,7 @@ setup_hard_base(
 				return -1;
 
 			snprintf(buf, sizeof(buf), "GROUP %s", group->name);
-			if (nntp_command(buf, OK_GROUP, line, sizeof(line)) == NULL)
+			if (new_nntp_command(buf, OK_GROUP, line, sizeof(line)) != OK_GROUP)
 				return -1;
 
 			if (sscanf(line, "%"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT, &count, &start, &last) != 3)
@@ -425,11 +387,17 @@ setup_hard_base(
  * the index by looking at the articles in the spool directory,
  * and attempt to write a new index if necessary.
  *
+ * a flag indicates if article headers shall be decoded and
+ * articles should be sorted and put in threads (TRUE) or
+ * we're just interested in the raw data (to write the
+ * cache or the like) (FALSE).
+ *
  * Returns FALSE if the user aborted the indexing, otherwise TRUE
  */
 t_bool
 index_group(
-	struct t_group *group)
+	struct t_group *group,
+	t_bool flag)
 {
 	int i;
 	int changed;				/* Count of articles whose overview has changed */
@@ -439,7 +407,7 @@ index_group(
 	t_artnum last_read_article;
 	t_artnum min, new_min, max;
 	t_bool caching_xover;
-	t_bool filtered;
+	t_bool filtered = FALSE;
 	t_bool rebuild_cache = FALSE;
 
 	if (group == NULL)
@@ -474,7 +442,7 @@ index_group(
 	}
 #endif /* DEBUG */
 
-	min = grpmenu.max ? base[0] : group->xmin;
+	min = group->xmin;
 	max = grpmenu.max ? base[grpmenu.max - 1] : min - 1;
 
 	/*
@@ -483,20 +451,18 @@ index_group(
 	if (max < 0)
 		return FALSE;
 
-	getart_limit = (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
+	getart_limit = getart_limit_set_from_scope(group) ? group->attribute->getart_limit : (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
 
 	if (getart_limit > 0) {
 		if (grpmenu.max && (grpmenu.max > getart_limit))
 			min = base[grpmenu.max - getart_limit];
-		else
-			min = group->xmin;
-	} else if (getart_limit < 0) {
-		t_artnum first_unread = find_first_unread(group);
+	} else {
+		if (getart_limit < 0) {
+			t_artnum first_unread = find_first_unread(group);
 
-		if (min - first_unread < getart_limit)
-			min = first_unread + getart_limit;
-		else
-			min = group->xmin;
+			if (min - first_unread <= getart_limit)
+				min = first_unread + getart_limit;
+		}
 	}
 
 	top_art = 0;
@@ -511,7 +477,7 @@ index_group(
 	 * cache (if found) otherwise the private overview cache will be read
 	 */
 	caching_xover = (serverrc.cache_overview_files && nntp_caps.over_cmd && group->type == GROUP_TYPE_NEWS);
-	if ((changed = read_overview(group, min, max, &last_read_article, caching_xover, &rebuild_cache)) == -1)
+	if ((changed = read_overview(group, min, max, &last_read_article, caching_xover, &rebuild_cache, flag)) == -1)
 		return FALSE;	/* user aborted indexing */
 
 	/*
@@ -521,7 +487,7 @@ index_group(
 	if ((last_read_article < max) && caching_xover) {
 		new_min = (last_read_article >= min) ? last_read_article + 1 : min;
 
-		if ((i = read_overview(group, new_min, max, &last_read_article, FALSE, &rebuild_cache)) == -1)
+		if ((i = read_overview(group, new_min, max, &last_read_article, FALSE, &rebuild_cache, flag)) == -1)
 			return FALSE;	/* user aborted indexing */
 		else
 			changed += i;
@@ -539,6 +505,7 @@ index_group(
 		}
 		if (base[i] <= last_read_article)		/* It is vital this test be done last */
 			continue;
+
 		++total;
 	}
 
@@ -596,27 +563,40 @@ index_group(
 	if (changed || caching_xover || rebuild_cache)
 		write_overview(group);
 
+	if (flag) {
+		/*
+		 * The articles are currently unsorted, and are as they
+		 * were put by setup_hard_base()
+		 */
+		BegStopWatch();
+		if (group->attribute && group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
+			sort_arts(group->attribute->sort_article_type);
+		EndStopWatch("sort_arts()");
+	}
+
 	/*
 	 * Create the reference tree. The msgid and ref ptrs will
 	 * be free()d now that the NovFile has been written.
 	 */
 	BegStopWatch();
-	build_references(group);
+	build_references();
 	EndStopWatch("build_references()");
 
-	/*
-	 * Needs access to the reference tree
-	 */
-	BegStopWatch();
-	filtered = filter_articles(group);
-	EndStopWatch("filter_articles()");
+	if (flag) {
+		/*
+		 * Needs access to the reference tree
+		 */
+		BegStopWatch();
+		filtered = filter_articles(group);
+		EndStopWatch("filter_articles()");
 
-	/*
-	 * Thread the group
-	 */
-	BegStopWatch();
-	make_threads(group, FALSE);
-	EndStopWatch("make_threads()");
+		/*
+		 * Thread the group
+		 */
+		BegStopWatch();
+		make_threads(group, FALSE);
+		EndStopWatch("make_threads()");
+	}
 
 	if ((changed > 0 || filtered) && !batch_mode)
 		clear_message();
@@ -661,8 +641,10 @@ open_art_header(
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
-		FILE *fp;
-		static t_bool no_next = FALSE; /* TODO: move to t_capabilities ? */
+		static t_bool no_next = FALSE;
+
+		if (!no_next && strstr(serverrc.disabled_nntp_cmds, "NEXT") != NULL)
+			no_next = TRUE;
 		/*
 		 * Don't bother requesting if we have not got there yet.
 		 * This is a big win if the group has got holes in it (ie. if 000's
@@ -672,18 +654,25 @@ open_art_header(
 			return NULL;
 
 		snprintf(buf, sizeof(buf), "HEAD %"T_ARTNUM_PFMT, art);
-		if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
-			return fp;
+		switch(new_nntp_command(buf, OK_HEAD, NULL, 0)) {
+			case OK_HEAD:
+				return FAKE_NNTP_FP;
+				/* NOTREACHED */
+				break;
 
-		/*
-		 * TODO:
-		 * shall we stop on 5xx?, i.e JamNNTPd/2 1.3 responds with
-		 * "503 Access denied" instead of 480 but NEXT still works,
-		 * so tin loops over all articles without getting useful data
-		 *
-		 * usenet.farm may return ERR_GOODBYE and NEXT then also fails
-		 * we could flag that like no_next ...
-		 */
+			case ERR_FAULT:		/* JamNNTPd/2 1.3 */
+			case ERR_GOODBYE:	/* usenet.farm */
+			case ERR_NOART:		/* usenet.farm */
+				no_next = TRUE;
+				return NULL;
+				/* NOTREACHED */
+				break;
+
+			/* do we need case ERR_NCING: to send GROUP here? */
+
+			default:
+				break;
+		}
 
 		if (!no_next) { /* usenet.farm doesn't do NEXT */
 			/*
@@ -700,16 +689,34 @@ open_art_header(
 				 * we do not have disabled_nntp_cmds=LISTGROUP
 				 */
 				case ERR_NCING:
-					nntp_caps.broken_listgroup = TRUE;
+					nntp_caps.listgroup = FALSE;
 					snprintf(buf, sizeof(buf), "GROUP %s", groupname);
-					if (nntp_command(buf, OK_GROUP, NULL, 0) == NULL)
+					if (new_nntp_command(buf, OK_GROUP, NULL, 0) != OK_GROUP)
 						return NULL;
+
 					snprintf(buf, sizeof(buf), "HEAD %"T_ARTNUM_PFMT, art);
-					if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
-						return fp;
-					if (nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)))
+					switch(new_nntp_command(buf, OK_HEAD, NULL, 0)) {
+						case OK_HEAD:
+							return FAKE_NNTP_FP;
+							/* NOTREACHED */
+							break;
+
+						case ERR_NCING:		/* we did select the group! */
+						case ERR_NOART:		/* we didn't ask for an id! */
+							no_next = TRUE;
+							return NULL;
+							/* NOTREACHED */
+							break;
+
+						default:
+							break;
+					}
+
+					if (new_nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)) == OK_NOTEXT) {
 						*next = atoartnum(buf);
-					break;
+						break;
+					}
+					/* FALLTHROUGH */
 
 				case ERR_COMMAND:	/* TODO: abort loop over all arts */
 					no_next = TRUE;
@@ -732,7 +739,7 @@ open_art_header(
 					 * ERR_NOARTIG may occur on servers where LISTGROUP
 					 * isn't broken...
 					 */
-					nntp_caps.broken_listgroup = TRUE;
+					nntp_caps.listgroup = FALSE;
 					break;
 			}
 		}
@@ -804,9 +811,7 @@ read_art_headers(
 		 * Skip articles that are below the low water mark or are
 		 * already present
 		 */
-		if (valid_artnum(art) >= 0)
-			continue;
-		if (art <= top)
+		if (valid_artnum(art) >= 0 || art <= top)
 			continue;
 
 		/*
@@ -1274,7 +1279,7 @@ thread_by_multipart(
  */
 void
 make_threads(
-	struct t_group *group,
+	const struct t_group *group,
 	t_bool rethread)
 {
 	int i;
@@ -1412,7 +1417,7 @@ eval_sort_arts_func(
 }
 
 
-void
+static void
 sort_arts(
 	unsigned int sort_art_type)
 {
@@ -1607,8 +1612,8 @@ parse_headers(
  * If there are more than MAX_CNT ranges after optimization, fetch all in one
  * big range
  */
-#define MIN_CNT 10
-#define MAX_CNT 50
+#	define MIN_CNT 10
+#	define MAX_CNT 50
 static struct t_article_range *
 build_range_list(
 	t_artnum min,
@@ -1777,7 +1782,7 @@ get_path_header(
 #	endif /* DEBUG */
 
 	if (nntp_caps.type == CAPABILITIES && nntp_caps.list_headers && !*nntp_caps.headers_range && nntp_caps.hdr_cmd[0] != 'X') {
-		switch (new_nntp_command("LIST HEADERS RANGE", 215, cmd, sizeof(cmd))) {
+		switch (new_nntp_command("LIST HEADERS RANGE", 215, NULL, 0)) {
 			case 215:
 				while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #	ifdef DEBUG
@@ -1796,21 +1801,22 @@ get_path_header(
 	}
 
 	/* does HDR return Path? */
-	if (nntp_caps.headers_range && (ptr = strtok(nntp_caps.headers_range, "\n")) != NULL) {
-		do {
-			if ((*ptr == ':' && *(ptr + 1) == '\0') || !strncasecmp(ptr, "Path", 4))
-				found = TRUE;
-		} while (!found && *ptr && (ptr = strtok(NULL, "\n")) != NULL);
-	}
+	if (nntp_caps.type == CAPABILITIES) {
+		if (nntp_caps.headers_range && (ptr = strtok(nntp_caps.headers_range, "\n")) != NULL) {
+			do {
+				if ((*ptr == ':' && *(ptr + 1) == '\0') || !strncasecmp(ptr, "Path", 4))
+					found = TRUE;
+			} while (!found && *ptr && (ptr = strtok(NULL, "\n")) != NULL);
+		}
+	} else
+		found = TRUE;
 
-	if ((nntp_caps.hdr && nntp_caps.hdr_cmd) && found) {
+	if (nntp_caps.hdr && nntp_caps.hdr_cmd && found) {
 		if (min == max)
 			snprintf(cmd, sizeof(cmd), "%s Path %"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min);
 		else
 			snprintf(cmd, sizeof(cmd), "%s Path %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min, MAX(min, max));
 		fp = nntp_command(cmd, nntp_caps.hdr_cmd[0] == 'X' ? OK_XHDR : OK_HDR, NULL, 0);
-		if (!nntp_caps.hdr && fp)
-			nntp_caps.hdr = TRUE;
 	} else if (nntp_caps.xpat) {
 		if (min == max)
 			snprintf(cmd, sizeof(cmd), "XPAT Path %"T_ARTNUM_PFMT" *", min);
@@ -1820,8 +1826,8 @@ get_path_header(
 	}
 
 	if (fp) {
-		t_artnum artnum, i, j = T_ARTNUM_CONST(0);
 		char *prep_msg = fmt_string(_(txt_prep_for_filter_on_path), cur, cnt);
+		t_artnum artnum, i, j = T_ARTNUM_CONST(0);
 
 		while ((buf = tin_fgets(fp, FALSE)) != NULL && buf[0] != '.') {
 #	ifdef DEBUG
@@ -1843,11 +1849,11 @@ get_path_header(
 			if (++artnum % MODULO_COUNT_NUM == 0)
 				show_progress(prep_msg, artnum - min, max - min);
 		}
-#	ifdef DEBUG
+#ifdef DEBUG
 		/* log end of multiline response to get timing data */
 		if ((debug & DEBUG_NNTP) && !verbose)
 			debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
-#	endif /* DEBUG */
+#endif /* DEBUG */
 		free(prep_msg);
 		return supported;
 	}
@@ -1970,7 +1976,8 @@ read_overview(
 	t_artnum max,
 	t_artnum *top,
 	t_bool local,
-	t_bool *rebuild_cache)
+	t_bool *rebuild_cache,
+	t_bool flag)
 {
 	FILE *fp;
 	char *ptr;
@@ -2063,15 +2070,11 @@ read_overview(
 		over_fields = 7;
 	}
 
-#ifdef USE_ZLIB
 	while (
-		gzfp ? gzgets(gzfp, buf, 32768) != Z_NULL /* FIXME: don't use fixed size */
-			: (buf = tin_fgets(fp, FALSE)) != NULL
-	)
-#else
-	while ((buf = tin_fgets(fp, FALSE)) != NULL)
+#ifdef USE_ZLIB
+		gzfp ? gzgets(gzfp, buf, 32768) != Z_NULL : /* FIXME: don't use fixed size */
 #endif /* USE_ZLIB */
-	{
+		(buf = tin_fgets(fp, FALSE)) != NULL) {
 
 #if defined(DEBUG) && defined(NNTP_ABLE)
 		if ((debug & DEBUG_NNTP) && fp == FAKE_NNTP_FP && verbose)
@@ -2182,22 +2185,24 @@ read_overview(
 				if (ofmt[count].type == OVER_T_STRING) {
 					if (!strcasecmp(ofmt[count].name, "Subject:")) {
 						if (*ptr) {
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->subject_raw = hash_str(ptr);
-
+							if (flag) {
 #ifdef HAVE_UNICODE_NORMALIZATION
-							if (IS_LOCAL_CHARSET("UTF-8"))
-								q = normalize(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
-							else
+								if (IS_LOCAL_CHARSET("UTF-8"))
+									q = normalize(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
+								else
 #endif /* HAVE_UNICODE_NORMALIZATION */
-								q = my_strdup(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
+									q = my_strdup(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
 
-							art->subject = hash_str(q);
-							free(q);
+								art->subject = hash_str(q);
+								free(q);
+							}
 						} else {
-							art->subject = hash_str("");
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->subject_raw = hash_str("");
+							if (flag)
+								art->subject = hash_str("");
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2208,16 +2213,19 @@ read_overview(
 
 					if (!strcasecmp(ofmt[count].name, "From:")) {
 						if (*ptr) {
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->from_raw = hash_str(ptr);
-							build_mailbox_list(art, ptr);
+							if (flag)
+								build_mailbox_list(art, ptr);
 						} else {
-							struct t_mailbox *mb;
-
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->from_raw = hash_str("");
-							if ((mb = add_mailbox(art)) != NULL)
-								mb->from = hash_str("");
+							if (flag) {
+								struct t_mailbox *mb;
+
+								if ((mb = add_mailbox(art)) != NULL)
+									mb->from = hash_str("");
+							}
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2312,21 +2320,24 @@ read_overview(
 				switch (count) {
 					case 1: /* Subject: */
 						if (*ptr) {
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->subject_raw = hash_str(ptr);
+							if (flag) {
 #ifdef HAVE_UNICODE_NORMALIZATION
-							if (IS_LOCAL_CHARSET("UTF-8"))
-								q = normalize(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
-							else
+								if (IS_LOCAL_CHARSET("UTF-8"))
+									q = normalize(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
+								else
 #endif /* HAVE_UNICODE_NORMALIZATION */
-								q = my_strdup(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
+									q = my_strdup(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr), FALSE)), FALSE));
 
-							art->subject = hash_str(q);
-							free(q);
+								art->subject = hash_str(q);
+								free(q);
+							}
 						} else {
-							art->subject = hash_str("");
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->subject_raw = hash_str("");
+							if (flag)
+								art->subject = hash_str("");
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2336,16 +2347,19 @@ read_overview(
 
 					case 2:	/* From: */
 						if (*ptr) {
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->from_raw = hash_str(ptr);
-							build_mailbox_list(art, ptr);
+							if (flag)
+								build_mailbox_list(art, ptr);
 						} else {
-							struct t_mailbox *mb;
-
-							if (serverrc.cache_overview_files)
+							if (!flag || serverrc.cache_overview_files)
 								art->from_raw = hash_str("");
-							if ((mb = add_mailbox(art)) != NULL)
-								mb->from = hash_str("");
+							if (flag) {
+								struct t_mailbox *mb;
+
+								if ((mb = add_mailbox(art)) != NULL)
+									mb->from = hash_str("");
+							}
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2510,7 +2524,7 @@ read_overview(
 			if (nntp_caps.type == CAPABILITIES && nntp_caps.list_headers) {
 				if (!*nntp_caps.headers_range) {
 					found = FALSE;
-					switch (new_nntp_command("LIST HEADERS RANGE", 215, cbuf, sizeof(cbuf))) {
+					switch (new_nntp_command("LIST HEADERS RANGE", 215, NULL, 0)) {
 						case 215:
 							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #	ifdef DEBUG
@@ -2562,9 +2576,11 @@ read_overview(
 					if (!strstr(ptr, "(none)")) {
 						if ((q = strchr(ptr, ' ')) == NULL) /* skip article number */
 							continue;
+
 						ptr = q;
 						while (*ptr && isspace((unsigned char) *ptr))
 							++ptr;
+
 						if ((q = strchr(ptr, '\n')) != NULL)
 							*q = '\0';
 						art->xref = my_strdup(ptr);
@@ -2669,7 +2685,7 @@ write_overview(
 	int i;
 	struct t_article *article;
 #if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
-	char *lc_all;
+	const char *lc_all;
 #endif /* HAVE_SETLOCALE && defined(NO_LOCALE) */
 #if defined(HAVE_FCHMOD) || defined(USE_ZLIB)
 	int fd;
@@ -2683,7 +2699,7 @@ write_overview(
 	/*
 	 * Can't write or caching is off or getart_limit is set
 	 */
-	if (no_write || !serverrc.cache_overview_files || ((cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit) != 0)
+	if (no_write || !serverrc.cache_overview_files || (getart_limit_set_from_scope(group) ? group->attribute->getart_limit : (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit) != 0)
 		return;
 
 	if ((fp = open_xover_fp(group, "w", T_ARTNUM_CONST(0), T_ARTNUM_CONST(0), FALSE)) == NULL)
@@ -3101,7 +3117,6 @@ do_update(
 {
 	int i, j, k = 0;
 	time_t beg_epoch = 0;
-	struct t_article *art;
 	struct t_group *group;
 
 	if (verbose)
@@ -3123,12 +3138,23 @@ do_update(
 		if (group->bogus || !group->subscribed)
 			continue;
 
-		if (!index_group(group)) {
+		if (!index_group(group, FALSE)) {
+#if 0
+			/*
+			 * can't see how this ever could leak, so why bother?
+			 * (next index_group() or tin_done() will take care
+			 * of the free()s and nothing else should modify arts[])
+			 * and if, why not also clear xref?
+			 */
+			struct t_article *art;
+
 			for_each_art(j) {
 				art = &arts[j];
 				FreeAndNull(art->refs);
 				FreeAndNull(art->msgid);
+				/* FreeAndNull(art->xref); */
 			}
+#endif /* 0 */
 			continue;
 		}
 
@@ -3146,10 +3172,40 @@ do_update(
 		}
 	}
 
-	if (verbose) { /* FIXME: translatable/plural-forms */
-		wait_message(0, _(txt_catchup_update_info),
-			(catchup ? _(txt_caughtup) : _(txt_updated)), k,
-			P_(txt_group_sp[0], txt_group_sp[1], selmenu.max), (unsigned long int) (time(NULL) - beg_epoch));
+	if (verbose) {
+		const char *action_fmt = catchup ? P_(txt_catchup_info_sp[0], txt_catchup_info_sp[1], k) : P_(txt_update_info_sp[0], txt_update_info_sp[1], k);
+		const char *elapsed_fmt;
+		char *action;
+		char *elapsed;
+		int n;
+		size_t len;
+		time_t tela = time(NULL) - beg_epoch;
+
+		if ((n = snprintf(NULL, 0, action_fmt, k)) < 0)
+			return;
+		len = (size_t) n + 1;
+		action = my_malloc(len);
+		if (snprintf(action, len, action_fmt, k) < 0) {
+			free(action);
+			return;
+		}
+
+		elapsed_fmt = P_(txt_n_second_sp[0], txt_n_second_sp[1], tela);
+		if ((n = snprintf(NULL, 0, elapsed_fmt, tela)) < 0) {
+			free(action);
+			return;
+		}
+		len = (size_t) n + 1;
+		elapsed = my_malloc(len);
+		if (snprintf(elapsed, len, elapsed_fmt, tela) < 0) {
+			free(action);
+			free(elapsed);
+			return;
+		}
+
+		wait_message(0, _(txt_catchup_update_info), action, elapsed);
+		free(action);
+		free(elapsed);
 	}
 }
 
