@@ -3,7 +3,7 @@
  *  Module    : attrib.c
  *  Author    : I. Lea
  *  Created   : 1993-12-01
- *  Updated   : 2025-12-08
+ *  Updated   : 2026-01-31
  *  Notes     : Group attribute routines
  *
  * Copyright (c) 1993-2026 Iain Lea <iain@bricbrac.de>
@@ -121,7 +121,7 @@ set_default_attributes(
 	CopyBits(sort_threads_type, tinrc.sort_threads_type);
 	CopyBits(show_author, tinrc.show_author);
 	CopyBool(show_signatures, tinrc.show_signatures);
-	CopyBool(show_art_score, tinrc.show_art_score);
+	CopyBits(show_art_info, tinrc.show_art_info);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	CopyBool(suppress_soft_hyphens, tinrc.suppress_soft_hyphens);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -238,7 +238,7 @@ set_default_state(
 	state->show_author = FALSE;
 	state->show_only_unread_arts = FALSE;
 	state->show_signatures = FALSE;
-	state->show_art_score = FALSE;
+	state->show_art_info = FALSE;
 	state->sigdashes = FALSE;
 	state->sigfile = FALSE;
 	state->signature_repost = FALSE;
@@ -500,7 +500,7 @@ read_attributes_file(
 					MATCH_INTEGER("show_author=", OPT_ATTRIB_SHOW_AUTHOR, SHOW_FROM_BOTH);
 					MATCH_BOOLEAN("show_only_unread_arts=", OPT_ATTRIB_SHOW_ONLY_UNREAD_ARTS);
 					MATCH_BOOLEAN("show_signatures=", OPT_ATTRIB_SHOW_SIGNATURES);
-					MATCH_BOOLEAN("show_art_score=", OPT_ATTRIB_SHOW_ART_SCORE);
+					MATCH_INTEGER("show_art_info=", OPT_ATTRIB_SHOW_ART_INFO, MAX_ART_INFO);
 					MATCH_BOOLEAN("sigdashes=", OPT_ATTRIB_SIGDASHES);
 					MATCH_BOOLEAN("signature_repost=", OPT_ATTRIB_SIGNATURE_REPOST);
 					MATCH_STRING("sigfile=", OPT_ATTRIB_SIGFILE);
@@ -572,8 +572,10 @@ read_attributes_file(
 			if (!global_file && upgrade && upgrade->state == RC_UPGRADE) {
 				int auto_cc_bcc;
 				int show_info;
+				int show_art_info;
 				t_bool auto_bcc;
 				t_bool auto_cc;
+				t_bool show_art_score;
 				t_bool verbatim_handling;
 
 				switch (my_tolower((unsigned char) line[0])) {
@@ -604,6 +606,15 @@ read_attributes_file(
 								found = TRUE;
 							break;
 						}
+						if (match_boolean(line, "show_art_score=", &show_art_score)) {
+							if (show_art_score)
+								show_art_info = 1;
+							else
+								show_art_info = 0;
+							set_attrib(OPT_ATTRIB_SHOW_ART_INFO, scope, line, &show_art_info);
+							found = TRUE;
+							break;
+						}
 						break;
 
 					case 'p':
@@ -983,8 +994,8 @@ set_attrib(
 			case OPT_ATTRIB_SHOW_SIGNATURES:
 				SET_BOOLEAN(show_signatures);
 
-			case OPT_ATTRIB_SHOW_ART_SCORE:
-				SET_BOOLEAN(show_art_score);
+			case OPT_ATTRIB_SHOW_ART_INFO:
+				SET_NUMBER(show_art_info);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 			case OPT_ATTRIB_SUPPRESS_SOFT_HYPHENS:
@@ -1225,7 +1236,7 @@ assign_attributes_to_groups(
 				SET_ATTRIB(sort_threads_type);
 				SET_ATTRIB(show_author);
 				SET_ATTRIB(show_signatures);
-				SET_ATTRIB(show_art_score);
+				SET_ATTRIB(show_art_info);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				SET_ATTRIB(suppress_soft_hyphens);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1526,7 +1537,7 @@ write_attributes_file(
 		SHOW_FROM_NAME, _(txt_show_from[SHOW_FROM_NAME]),
 		SHOW_FROM_BOTH, _(txt_show_from[SHOW_FROM_BOTH]));
 	fprintf(fp, "%s", _(txt_attrib_file_show_signatures));
-	fprintf(fp, "%s", _(txt_attrib_file_show_art_score));
+	fprintf(fp, "%s", _(txt_attrib_file_show_art_info));
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	fprintf(fp, "%s", _(txt_attrib_file_suppress_soft_hyphens));
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1609,13 +1620,13 @@ write_attributes_file(
 		error_message(2, _(txt_filesystem_full), ATTRIBUTES_FILE);
 		return;
 	}
-	rewind(fp);
+
+	fseek(fp, 0L, SEEK_SET);
 	attrib_file_offset = 1;
-	while ((i = fgetc(fp)) != EOF) {
-		if (i == '\n')
-			++attrib_file_offset;
-	}
-	if (fseek(fp, fpos, SEEK_SET)) {
+	while (tin_fgets(fp, FALSE) != NULL) /* count lines */
+		++attrib_file_offset;
+
+	if (fseek(fp, fpos, SEEK_SET) || attrib_file_offset == 1) {
 		clearerr(fp);
 		fclose(fp);
 		unlink(new_file);
@@ -1758,8 +1769,8 @@ write_attributes_file(
 					fprintf(fp, "show_only_unread_arts=%s\n", print_boolean(scope->attribute->show_only_unread_arts));
 				if (scope->state->show_signatures)
 					fprintf(fp, "show_signatures=%s\n", print_boolean(scope->attribute->show_signatures));
-				if (scope->state->show_art_score)
-					fprintf(fp, "show_art_score=%s\n", print_boolean(scope->attribute->show_art_score));
+				if (scope->state->show_art_info)
+					fprintf(fp, "show_art_info=%u\n", (unsigned) scope->attribute->show_art_info);
 				if (scope->state->sigdashes)
 					fprintf(fp, "sigdashes=%s\n", print_boolean(scope->attribute->sigdashes));
 				if (ATTRIB_IS_SET(sigfile))
@@ -1897,7 +1908,7 @@ skip_scope(
 		|| scope->state->show_author
 		|| scope->state->show_only_unread_arts
 		|| scope->state->show_signatures
-		|| scope->state->show_art_score
+		|| scope->state->show_art_info
 		|| scope->state->sigdashes
 		|| ATTRIB_IS_SET(sigfile)
 		|| scope->state->signature_repost
@@ -2041,7 +2052,7 @@ dump_attributes(
 			debug_print_file("ATTRIBUTES", "\tsort_threads_type=%d", group->attribute->sort_threads_type);
 			debug_print_file("ATTRIBUTES", "\tshow_author=%d", group->attribute->show_author);
 			debug_print_file("ATTRIBUTES", "\tshow_signatures=%s", print_boolean(group->attribute->show_signatures));
-			debug_print_file("ATTRIBUTES", "\tshow_art_score=%s", print_boolean(group->attribute->show_art_score));
+			debug_print_file("ATTRIBUTES", "\tshow_art_info=%d", group->attribute->show_art_info);
 			debug_print_file("ATTRIBUTES", "\tsigdashes=%s", print_boolean(group->attribute->sigdashes));
 			debug_print_file("ATTRIBUTES", "\tsignature_repost=%s", print_boolean(group->attribute->signature_repost));
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -2164,7 +2175,7 @@ dump_scopes(
 			debug_print_file(fname, "\t%ssort_threads_type=%d", DEBUG_PRINT_STATE(sort_threads_type), scope->attribute->sort_threads_type);
 			debug_print_file(fname, "\t%sshow_author=%d", DEBUG_PRINT_STATE(show_author), scope->attribute->show_author);
 			debug_print_file(fname, "\t%sshow_signatures=%s", DEBUG_PRINT_STATE(show_signatures), print_boolean(scope->attribute->show_signatures));
-			debug_print_file(fname, "\t%sshow_art_score=%s", DEBUG_PRINT_STATE(show_art_score), print_boolean(scope->attribute->show_art_score));
+			debug_print_file(fname, "\t%sshow_art_info=%d", DEBUG_PRINT_STATE(show_art_info), scope->attribute->show_art_info);
 			debug_print_file(fname, "\t%ssigdashes=%s", DEBUG_PRINT_STATE(sigdashes), print_boolean(scope->attribute->sigdashes));
 			debug_print_file(fname, "\t%ssignature_repost=%s", DEBUG_PRINT_STATE(signature_repost), print_boolean(scope->attribute->signature_repost));
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
