@@ -3,11 +3,11 @@
  *  Module    : auth.c
  *  Author    : Dirk Nimmich <nimmich@muenster.de>
  *  Created   : 1997-04-05
- *  Updated   : 2025-07-15
+ *  Updated   : 2025-12-07
  *  Notes     : Routines to authenticate to a news server via NNTP.
  *              DON'T USE get_respcode() THROUGHOUT THIS CODE.
  *
- * Copyright (c) 1997-2025 Dirk Nimmich <nimmich@muenster.de>
+ * Copyright (c) 1997-2026 Dirk Nimmich <nimmich@muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,8 +60,8 @@ static t_bool authinfo_plain(char *server, char *authuser, t_bool startup);
 static char *prompt_for_authid(const char *authuser);
 static char *prompt_for_password(void);
 #	ifdef USE_GSASL
-	static int sasl_auth(char *user, char *pass);
-	static int do_authinfo_sasl(char *authuser, char *authpass);
+	static int sasl_auth(const char *user, const char *pass);
+	static int do_authinfo_sasl(const char *authuser, const char *authpass);
 	/* static int callback(Gsasl *ctx, Gsasl_session *sctx, Gsasl_property prop); */
 #	endif /* USE_GSASL */
 
@@ -141,7 +141,7 @@ read_newsauth_file(
 		/* allow ":port" suffix in .newsauth */
 		if ((p = strchr(line, ':')) != NULL) {
 			if (*line != '[' && strrchr(line, ':') == p) { /* exact 1 x ':' must be [name|ipv4]:port */
-				snprintf(hn, sizeof(hn), "%s:%u", server, nntp_tcp_port);
+				snprintf(hn, sizeof(hn), "%s:%u", server, (unsigned int) nntp_tcp_port);
 				if ((strcasecmp(line, hn)))
 					continue;
 			} else { /* "[ipv6]"[:port] */
@@ -152,7 +152,7 @@ read_newsauth_file(
 					}
 					if ((p = strchr(q, ':')) != NULL) {
 						if (p == q + 1) {
-							snprintf(hn, sizeof(hn), "[%s]:%u", server, nntp_tcp_port);
+							snprintf(hn, sizeof(hn), "[%s]:%u", server, (unsigned int) nntp_tcp_port);
 							if ((strcasecmp(line, hn)))
 								continue;
 						}
@@ -307,7 +307,7 @@ authinfo_plain(
 #	ifdef USE_GSASL
 		if (nntp_caps.authinfo_sasl && nntp_caps.sasl_mechs && *nntp_caps.sasl_mechs)
 			ret = do_authinfo_sasl(authusername, authpassword);
-		if (ret != OK_AUTH_SASL && ret != OK_AUTH)
+		if (ret != OK_AUTH_SASL && ret != OK_AUTH /* && ret != ERR_GOODBYE */)
 #	endif /* USE_GSASL */
 		{
 			if (nntp_caps.type != CAPABILITIES || nntp_caps.authinfo_user)
@@ -341,7 +341,7 @@ authinfo_plain(
 			if (nntp_caps.authinfo_sasl && nntp_caps.sasl_mechs && *nntp_caps.sasl_mechs)
 				ret = do_authinfo_sasl(authuser, authpass);
 
-			if (ret != OK_AUTH_SASL && ret != OK_AUTH)
+			if (ret != OK_AUTH_SASL && ret != OK_AUTH /* && ret != ERR_GOODBYE */)
 #	endif /* USE_GSASL */
 			{
 				if (force_auth_on_conn_open || nntp_caps.type != CAPABILITIES || (nntp_caps.type == CAPABILITIES && nntp_caps.authinfo_user))
@@ -509,8 +509,8 @@ authenticate(
 #	ifdef USE_GSASL
 static int
 do_authinfo_sasl(
-	char *authuser,
-	char *authpass)
+	const char *authuser,
+	const char *authpass)
 {
 	char *utf8user = NULL;
 	char *utf8pass = NULL;
@@ -601,8 +601,8 @@ do_authinfo_sasl(
 #		endif /* GSASL_VERSION_NUMBER >= 0x010b00 */
 static int
 sasl_auth(
-	char *user,
-	char *pass)
+	const char *user,
+	const char *pass)
 {
 	Gsasl *ctx = NULL;
 	Gsasl_session *session = NULL;
@@ -631,7 +631,7 @@ sasl_auth(
 	sasl_prop = SASL_NEED_NONE;
 
 	/* figure out required properties - align mechs with nntplib.c:check_extensions() */
-	if (!strcasecmp(mech, "PLAIN") || !strcasecmp(mech, "LOGIN"))
+	if (!strcasecmp(mech, "PLAIN") /* || !strcasecmp(mech, "LOGIN")*/)
 		sasl_prop = (SASL_NEED_AUTHID | SASL_NEED_PASSWORD);
 	if (!strcasecmp(mech, "ANONYMOUS"))
 		sasl_prop = SASL_NEED_ANONYMOUS_TOKEN;
