@@ -3,10 +3,10 @@
  *  Module    : charset.c
  *  Author    : M. Kuhn, T. Burmester
  *  Created   : 1993-12-10
- *  Updated   : 2025-06-03
+ *  Updated   : 2026-02-10
  *  Notes     : ISO to ascii charset conversion routines
  *
- * Copyright (c) 1993-2025 Markus Kuhn <mgk25@cl.cam.ac.uk>
+ * Copyright (c) 1993-2026 Markus Kuhn <mgk25@cl.cam.ac.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -271,7 +271,7 @@ convert_iso2asc(
  */
 void
 convert_tex2iso(
-	char *from,
+	const char *from,
 	char *to)
 {
 	const char *tex_to[TEX_SUBST];
@@ -379,6 +379,9 @@ convert_to_printable(
 #else
 	unsigned char *c;
 
+	if (IS_LOCAL_CHARSET("UTF-8"))
+		utf8_valid(buf);
+
 	for (c = (unsigned char *) buf; *c; c++) {
 		if (!my_isprint(*c) && !(keep_tab && *c == '\t'))
 			*c = '?';
@@ -450,7 +453,7 @@ charset_unsupported(
 	{
 		iconv_t cd;
 
-		if ((cd = iconv_open("UCS-4", charset)) != (iconv_t) (-1)) {
+		if ((cd = iconv_open("UCS-4", charset)) != (iconv_t) -1) {
 			iconv_close(cd);
 			return FALSE;
 		}
@@ -527,6 +530,27 @@ failure:
 #endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
 
 
+void
+csguess_convert_str(
+	char **line,
+	const char *def_cs)
+{
+	size_t len = strlen(*line);
+#ifdef CHARSET_CONVERSION
+#	ifdef USE_ICU_UCSDET
+	char *guessed_charset = guess_charset(*line, 10);
+
+	process_charsets(line, &len, guessed_charset ? guessed_charset : def_cs, tinrc.mm_local_charset, FALSE);
+	FreeAndNull(guessed_charset);
+#	else
+	process_charsets(line, &len, def_cs, tinrc.mm_local_charset, FALSE);
+#	endif /* USE_ICU_UCSDET */
+#else
+	process_charsets(line, &len, def_cs, tinrc.mm_local_charset, FALSE);
+#endif /* CHARSET_CONVERSION */
+}
+
+
 /*
  * restrict it to [a-zA-Z0-9_-]+
  */
@@ -568,17 +592,17 @@ charset_compare(
 	while(cs_a[i] && cs_b[j]) {
 		/* skip all but alnums */
 		while(cs_a[i] && !isalnum((unsigned char) cs_a[i]))
-			i++;
+			++i;
 		while(cs_b[j] && !isalnum((unsigned char) cs_b[j]))
-			j++;
+			++j;
 
 		/* skip 0 not preceded by a digit */
 		while(i && cs_a[i] == '0' && !isdigit((unsigned char) cs_a[i - 1]))
-			i++;
+			++i;
 		while(j && cs_b[j] == '0' && !isdigit((unsigned char) cs_b[j - 1]))
-			j++;
+			++j;
 
-		if (tolower((unsigned char) cs_a[i]) != tolower((unsigned char) cs_b[j]))
+		if (my_tolower((unsigned char) cs_a[i]) != my_tolower((unsigned char) cs_b[j]))
 			return FALSE;
 
 		++i;
