@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2025-08-20
+ *  Updated   : 2025-12-08
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,7 +56,7 @@
  */
 static t_bool match_item(const char *line, const char *pat, char *dst, size_t dstlen);
 static t_bool rc_update(FILE *fp);
-static t_bool rc_post_update(FILE *fp, struct t_version *upgrade);
+static t_bool rc_post_update(FILE *fp, const struct t_version *upgrade);
 static void write_server_config(void);
 #ifdef HAVE_COLOR
 	static t_bool match_color(const char *line, const char *pat, int *dst, int max);
@@ -76,12 +76,21 @@ static void write_server_config(void);
 			free(wbuf); \
 		} \
 	} while (0)
+#	define SET_SC_VAL(buf, scval, defval) do { \
+		wchar_t *wbuf; \
+		if ((wbuf = char2wchar_t(buf))) { \
+			wbuf[1] = (wchar_t) '\0'; \
+			scval = !wbuf[0] ? (wchar_t) defval : DASH_TO_SPACE(wbuf[0]); \
+			free(wbuf); \
+		} \
+	} while (0)
 #else
 #	define DASH_TO_SPACE(mark)	((char) (mark == '_' ? ' ' : mark))
 #	define SPACE_TO_DASH(mark)	((char) (mark == ' ' ? '_' : mark))
 #	define SET_RC_VAL(buf, rcval, defval) do { \
 		rcval = !buf[0] ? defval : DASH_TO_SPACE(buf[0]); \
 	} while (0)
+#	define SET_SC_VAL(buf, scval, defval) SET_RC_VAL(buf, scval, defval)
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
@@ -712,6 +721,26 @@ read_config_file(
 					break;
 				}
 
+				if (match_string(buf, "scope_marked_global=", tmp, sizeof(tmp))) {
+					SET_SC_VAL(tmp, tinrc.scope_marked_global, SCOPE_MARK_GLOBAL);
+					break;
+				}
+
+				if (match_string(buf, "scope_marked_global_match=", tmp, sizeof(tmp))) {
+					SET_SC_VAL(tmp, tinrc.scope_marked_global_match, SCOPE_MARK_GLOBAL_MATCH);
+					break;
+				}
+
+				if (match_string(buf, "scope_marked_local=", tmp, sizeof(tmp))) {
+					SET_SC_VAL(tmp, tinrc.scope_marked_local, SCOPE_MARK_LOCAL);
+					break;
+				}
+
+				if (match_string(buf, "scope_marked_local_match=", tmp, sizeof(tmp))) {
+					SET_SC_VAL(tmp, tinrc.scope_marked_local_match, SCOPE_MARK_LOCAL_MATCH);
+					break;
+				}
+
 				if (match_string_ptr(buf, "select_format=", &tinrc.select_format))
 					break;
 
@@ -762,7 +791,7 @@ read_config_file(
 				if (match_boolean(buf, "show_signatures=", &tinrc.show_signatures))
 					break;
 
-				if (match_boolean(buf, "show_art_score=", &tinrc.show_art_score))
+				if (match_integer(buf, "show_art_info=", &tinrc.show_art_info, MAX_ART_INFO))
 					break;
 
 				if (match_string_ptr(buf, "slashes_regex=", &tinrc.slashes_regex))
@@ -1031,10 +1060,12 @@ write_config_file(
 		return;
 
 	/* generate tmp-filename */
+	/* TODO: use my_tmpfile() instead to avoid issues with e.g. "/dev/null" as file */
 	if ((file_tmp = get_tmpfilename(file)) == NULL)
 		return;
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
+		/* TODO: use perror_message(txt_cannot_create, ""); sleep(2) */
 		error_message(2, _(txt_filesystem_full_backup), CONFIG_FILE);
 		free(file_tmp);
 		return;
@@ -1247,8 +1278,8 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_show_signatures.tinrc));
 	fprintf(fp, "show_signatures=%s\n\n", print_boolean(tinrc.show_signatures));
 
-	fprintf(fp, "%s", _(txt_show_art_score.tinrc));
-	fprintf(fp, "show_art_score=%s\n\n", print_boolean(tinrc.show_art_score));
+	fprintf(fp, "%s", _(txt_show_art_info.tinrc));
+	fprintf(fp, "show_art_info=%d\n\n", tinrc.show_art_info);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	fprintf(fp, "%s", _(txt_suppress_soft_hyphens.tinrc));
@@ -1301,6 +1332,18 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_art_marked_read_selected.tinrc));
 	fprintf(fp, "art_marked_read_selected=%"T_CHAR_FMT"\n\n", SPACE_TO_DASH(tinrc.art_marked_read_selected));
 
+	fprintf(fp, "%s", _(txt_scope_marked_global.tinrc));
+	fprintf(fp, "scope_marked_global=%"T_CHAR_FMT"\n\n", SPACE_TO_DASH(tinrc.scope_marked_global));
+
+	fprintf(fp, "%s", _(txt_scope_marked_global_match.tinrc));
+	fprintf(fp, "scope_marked_global_match=%"T_CHAR_FMT"\n\n", SPACE_TO_DASH(tinrc.scope_marked_global_match));
+
+	fprintf(fp, "%s", _(txt_scope_marked_local.tinrc));
+	fprintf(fp, "scope_marked_local=%"T_CHAR_FMT"\n\n", SPACE_TO_DASH(tinrc.scope_marked_local));
+
+	fprintf(fp, "%s", _(txt_scope_marked_local_match.tinrc));
+	fprintf(fp, "scope_marked_local_match=%"T_CHAR_FMT"\n\n", SPACE_TO_DASH(tinrc.scope_marked_local_match));
+
 	fprintf(fp, "%s", _(txt_force_screen_redraw.tinrc));
 	fprintf(fp, "force_screen_redraw=%s\n\n", print_boolean(tinrc.force_screen_redraw));
 
@@ -1662,7 +1705,7 @@ write_config_file(
 
 t_bool
 match_boolean(
-	char *line,
+	const char *line,
 	const char *pat,
 	t_bool *dst)
 {
@@ -1774,7 +1817,7 @@ match_long(
  */
 t_bool
 match_list(
-	char *line,
+	const char *line,
 	constext *pat,
 	constext *const *table,
 	int *dst)
@@ -1801,7 +1844,7 @@ match_list(
 
 t_bool
 match_string(
-	char *line,
+	const char *line,
 	const char *pat,
 	char *dst,
 	size_t dstlen)
@@ -1819,7 +1862,7 @@ match_string(
 
 t_bool
 match_string_ptr(
-	char *line,
+	const char *line,
 	const char *pat,
 	char **dst)
 {
@@ -1980,6 +2023,7 @@ rc_update(
 	int show_info = 1;
 	t_bool auto_bcc = FALSE;
 	t_bool auto_cc = FALSE;
+	t_bool show_art_score = FALSE;
 	t_bool confirm_to_quit = FALSE;
 	t_bool confirm_action = FALSE;
 	t_bool compress_quotes = FALSE;
@@ -2100,6 +2144,9 @@ rc_update(
 				}
 				if (match_boolean(buf, "save_to_mmdf_mailbox=", &save_to_mmdf_mailbox))
 					break;
+				if (match_boolean(buf, "show_art_score=", &show_art_score)) {
+					break;
+				}
 				if (match_integer(buf, "show_info=", &show_info, 3))
 					break;
 				if (match_boolean(buf, "show_last_line_prev_page=", &show_last_line_prev_page))
@@ -2143,6 +2190,7 @@ rc_update(
 	/* update the values */
 	tinrc.auto_cc_bcc = (auto_cc ? 1 : 0) + (auto_bcc ? 2 : 0);
 	tinrc.confirm_choice = (confirm_action ? 1 : 0) + (confirm_to_quit ? 3 : 0);
+	tinrc.show_art_info = show_art_score ? 1 : 0;
 
 	if (!use_getart_limit)
 		tinrc.getart_limit = 0;
@@ -2227,7 +2275,7 @@ rc_update(
 static t_bool
 rc_post_update(
 	FILE *fp,
-	struct t_version *upgrade)
+	const struct t_version *upgrade)
 {
 	char *buf;
 	int groupname_max_length = 0;
@@ -2352,8 +2400,8 @@ rc_post_update(
  *       confusion).
  *
  * TODO: - add more server specific vars (which can't be set
- *         via add_cmd_line_opts) or even per server config-files like
- *         keymap, filter, ...?
+ *         via add_cmd_line_opts e.g. strip_newsrc) or even per
+ *         server config-files like keymap, filter, ...?
  *       - add a cmd-line option to skip reading the serverrc?
  *         (what about status vars like last_newnews or motd_hash then?)
  */
@@ -2363,7 +2411,7 @@ rc_post_update(
 	} while (0)
 #endif /* NNTP_ABLE && INET6 */
 /* avoid to add options which take a string as argument */
-#define OPTIONS ":46ACdG:knp:qQt:Tx"
+#define ADD_CMD_OPTIONS ":46ACdG:knp:qQt:Tx"
 void
 read_server_config(
 	unsigned int option_mask) /* -u or -R on the cmd.-line? */
@@ -2390,6 +2438,7 @@ read_server_config(
 		"LIST SUBSCRIPTIONS",
 		"LISTGROUP",
 		"NEWGROUPS",
+		"NEXT",
 		"OVER",
 		"XHDR",
 		"XOVER",
@@ -2462,8 +2511,8 @@ read_server_config(
 
 				if (*d) { /* ignore empty tokens */
 #ifdef NNTP_ABLE
-					if (!is_valid) /* TODO: only in debug mode? */
-						wait_message(2, "Invalid disabled_nntp_cmds \"%s\", discarding it", d); /* -> lang.c */
+					if (!is_valid)
+						wait_message(3, _(txt_invalid_disabled_nntp_cmds), d);
 					else
 #endif /* NNTP_ABLE */
 					{
@@ -2475,7 +2524,7 @@ read_server_config(
 						 * check_extensions() (after add_cmd_line_opts has
 						 * been parsed)
 						 */
-						if (nntp_caps.type != BROKEN && !strcasecmp(d, "CAPABILITIES"))
+						if (nntp_caps.type != BROKEN && !strcmp(d, "CAPABILITIES"))
 							nntp_caps.type = BROKEN;
 #endif /* NNTP_ABLE */
 					}
@@ -2571,13 +2620,12 @@ read_server_config(
 		optind = 1;
 		optopt = 0; /* AFAIK at least MINIX < 3.2.0 doesn't set optopt */
 
-		while ((ch = getopt(i, args, OPTIONS)) != -1) {
+		while ((ch = getopt(i, args, ADD_CMD_OPTIONS)) != -1) {
 			switch (ch) {
 				case '4':
 #if defined(NNTP_ABLE) && defined(INET6)
 					if (force_ipv6) {
 						USELESS_COMB("-4", "-6");
-						force_ipv6 = FALSE;
 					}
 					read_news_via_nntp = force_ipv4 = TRUE;
 					force_ipv6 = FALSE;
@@ -2588,7 +2636,6 @@ read_server_config(
 #if defined(NNTP_ABLE) && defined(INET6)
 					if (force_ipv4) {
 						USELESS_COMB("-6", "-4");
-						force_ipv4 = FALSE;
 					}
 					read_news_via_nntp = force_ipv6 = TRUE;
 					force_ipv4 = FALSE;
@@ -2707,7 +2754,7 @@ read_server_config(
 	tinrc.serverrc_compress_overview_files = serverrc.compress_overview_files;
 #endif /* USE_ZLIB */
 }
-#undef OPTIONS
+#undef ADD_CMD_OPTIONS
 #if defined(NNTP_ABLE) && defined(INET6)
 #	undef USELESS_COMB
 #endif /* NNTP_ABLE && INET6 */
@@ -2773,7 +2820,7 @@ write_server_config(
 #endif /* USE_ZLIB */
 
 	fprintf(fp, "%s", _(txt_serverrc_internal));
-	if (serverrc.motd_hash != 0)
+	if (serverrc.motd_hash != 0L)
 		fprintf(fp, "motd_hash=%lu\n", (unsigned long int) serverrc.motd_hash);
 	if ((i = find_newnews_index(nntp_server)) >= 0) {
 		if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time))))
