@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2025-08-27
+ *  Updated   : 2026-01-25
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2025 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2026 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,8 +53,7 @@
 				if (*(u + l - 1) == '\n') \
 					*(u + l - 1) = '\0'; \
 				if (*u) { \
-					state.uri = &uri; \
-					if (uriParseUriA(&state, u) == URI_SUCCESS) { \
+					if (uriParseSingleUriA(&uri, u, NULL) == URI_SUCCESS) { \
 						if (uriNormalizeSyntaxA(&uri) == URI_SUCCESS) \
 							flags |= f; \
 						uriFreeUriMembersA(&uri); \
@@ -104,9 +103,10 @@
 #define UUE_BEGIN			(1 << 4)
 #define UUE_SECTION			(1 << 5)
 #define YENC_SECTION			(1 << 6)
-#define SHAR_SECTION		(1 << 7)
-#define PGP_KEY_SECTION		(1 << 8)
-#define PGP_SIG_SECTION		(1 << 9)
+#define SHAR_SECTION			(1 << 7)
+#define PGP_TXT_SECTION		(1 << 8)
+#define PGP_KEY_SECTION		(1 << 9)
+#define PGP_SIG_SECTION		(1 << 10)
 #define PGP_ANY_SECTION		(PGP_KEY_SECTION | PGP_SIG_SECTION)
 
 #define STRIP_ALTERNATIVE(x) \
@@ -270,7 +270,7 @@ put_cooked(
 	const char *fmt,
 	...)
 {
-	char *p, *bufp, *buf, *last_space;
+	char *p, *buf, *bufp, *last_space;
 	int wrap_column;
 	int space;
 	static unsigned int saved_flags = 0;
@@ -421,7 +421,7 @@ get_filename(
 	t_param *ptr)
 {
 	const char *name;
-	char *p;
+	const char *p;
 
 	if ((name = get_param(ptr, "filename*"))) {
 		if ((p = strrchr(name, '/')))
@@ -457,14 +457,13 @@ get_filename(
 	} while (0)
 
 #define INSERT_SEP() do { \
-		if (curr->prev && (curr->flags & (ATTACH_SHOW_CONTENT | ATTACH_SHOW_BOTH)) && (space_left -= strlen(_(txt_mime_sep)) > 0)) { \
+		if (curr->prev && (curr->flags & (ATTACH_SHOW_CONTENT | ATTACH_SHOW_BOTH)) && ((space_left -= strlen(_(txt_mime_sep))) > 0)) { \
 			strcat(attach_line, _(txt_mime_sep)); \
-			al_ptr = attach_line + strlen(attach_line); \
 		} \
 	} while (0)
 
 #define INSERT_SLASH() do { \
-		if ((curr->flags & (ATTACH_SHOW_CONTENT | ATTACH_SHOW_BOTH)) && (space_left -= 2 > 0)) { \
+		if ((curr->flags & (ATTACH_SHOW_CONTENT | ATTACH_SHOW_BOTH)) && ((space_left -= 2) > 0)) { \
 			*al_ptr++ = '/'; \
 			*al_ptr = '\0'; \
 		} \
@@ -476,7 +475,7 @@ get_filename(
 				snprintf(buf, blen, curr->description, curr->content); \
 			else \
 				snprintf(buf, blen, curr->fmt, curr->content); \
-			if ((space_left -= strlen(buf) > 0)) \
+			if ((space_left -= strlen(buf)) > 0) \
 				strcat(attach_line, buf); \
 		} \
 		curr = curr->next; \
@@ -560,15 +559,14 @@ build_attach_line(
 	char *total_str = NULL;
 	char *buf;
 	char *fmt;
-	int i, str_len;
+	int i, str_len, space_left;
 	size_t blen;
-	ssize_t space_left;
 	struct t_attach_item *curr = NULL;
 	struct t_attach_item *items = NULL;
 	struct t_attach_item *last;
 	t_bool init = TRUE;
 	t_bool is_uue = (section == SECTION_UUE_COMPLETE || section == SECTION_UUE_INCOMPLETE);
-	t_bool is_yenc = (section >= SECTION_YENC_COMPLETE && section <= SECTION_YENC_CORRUPT);
+	t_bool is_yenc = (section >= SECTION_YENC_COMPLETE /* && section <= SECTION_YENC_CORRUPT */);
 	t_bool excl_seen = FALSE;
 	t_bool star_seen = FALSE;
 
@@ -1086,8 +1084,8 @@ put_attach(
 	const char *charset)
 {
 	char *attach_line;
-	t_bool is_uue = section == SECTION_UUE_COMPLETE || section == SECTION_UUE_INCOMPLETE;
-	t_bool is_yenc = section >= SECTION_YENC_COMPLETE && section <= SECTION_YENC_CORRUPT;
+	t_bool is_uue = (section == SECTION_UUE_COMPLETE || section == SECTION_UUE_INCOMPLETE);
+	t_bool is_yenc = (section >= SECTION_YENC_COMPLETE /* && section <= SECTION_YENC_CORRUPT */);
 
 	assert(((void) "put_attach() failed. part == NULL", part != NULL));
 
@@ -1183,12 +1181,12 @@ process_text_body_part(
 	int hide_inline_data)
 {
 	char *rest = NULL;
-	char *line = NULL, *buf;
+	char *line = NULL, *line_ptr, *buf;
 	char *raw_line = NULL;
 	char *y_name = NULL;			/* yenc, filename */
 	char *cp;
 	char *ncs;						/* named capture */
-	static const char xxenc[] = {	/* xxdecode table */
+	static const signed char xxenc[] = {	/* xxdecode table */
 		0x00, 0x7f, 0x01, 0x7f, 0x7f,
 		0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
 		0x0a, 0x0b, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f,
@@ -1205,6 +1203,7 @@ process_text_body_part(
 	int y_total = 0;				/* yenc, total parts */
 	int is_uubody;					/* Set when current line looks like a uuencoded=1/xxencoded=2 line */
 	int sum;
+	long y_tmp;
 	unsigned int flags;
 	unsigned int section = TEXT_SECTION;
 	unsigned int lines_skipped = 0;
@@ -1213,7 +1212,7 @@ process_text_body_part(
 	unsigned long y_pbegin = 0L;		/* yenc, part start offset */
 	unsigned long y_pend = 0L;		/* yenc, part end offset */
 	unsigned long lines_hidden = 0L;
-	size_t max_line_len = 0, len, raw_len;
+	size_t max_line_len = 0, len, raw_len, offset;
 	uint32_t y_ccrc = 0L;			/* computed crc */
 	t_bool first_line_blank = TRUE;	/* Unset when first non-blank line is reached */
 	t_bool put_blank_lines = FALSE;	/* Set when previously skipped lines needs to put */
@@ -1225,7 +1224,6 @@ process_text_body_part(
 #endif /* HAVE_LIBURIPARSER || HAVE_LIBCURL */
 #ifdef HAVE_LIBURIPARSER
 	UriUriA uri;
-	UriParserStateA state;
 #else
 #	ifdef HAVE_LIBCURL
 	CURLU *curl;
@@ -1280,18 +1278,17 @@ process_text_body_part(
 					max_line_len = strlen(buf) + 1;
 					line = my_realloc(line, max_line_len + 1);
 				}
-				strcpy(line, buf);
-
 				/*
 				 * FIXME: Some code in cook.c expects a '\n' at the end
 				 * of the line. As tin_fgets() strips trailing '\n', re-add it.
 				 * This should probably be fixed in that other code.
 				 */
-				strcat(line, "\n");
+				snprintf(line, max_line_len + 1, "%s\n", buf);
 
 				--lines_left;
 				break;
 		}
+
 		if (!(line && strlen(line))) {
 			FreeIfNeeded(rest);
 			break;	/* premature end of file, file error etc. */
@@ -1309,7 +1306,6 @@ process_text_body_part(
 		FreeAndNull(raw_line);
 		raw_line = my_strdup(line); /* we need the raw data for yenc decoding */
 		process_charsets(&line, &max_line_len, charset, tinrc.mm_local_charset, curr_group->attribute->tex2iso_conv && art->tex2iso && !(section & (VERBATIM_SECTION | VERBATIM_BEGIN)));
-		len = strlen(line);
 
 		/*
 		 * assume that yenc never shows up in encoded articles (cause then
@@ -1319,23 +1315,122 @@ process_text_body_part(
 		 */
 		if ((hide_inline_data & UUE_ALL) && section == TEXT_SECTION && (part->encoding == ENCODING_7BIT || part->encoding == ENCODING_8BIT) && !strcasecmp(charset, "US-ASCII") && (*raw_line != '=' || *(raw_line + 1) != 'y')) {
 			raw_len = strlen(raw_line);
-			/* skip check after 100 yenc-lins; yenc should run till ^=yend or EOF */
-			/* TODO: use defines instead of magic numbers */
+			/*
+			 * skip check after 100 yenc-lins (use raw_len as flag)
+			 * yenc should run till ^=yend or EOF
+			 *
+			 * TODO: use defines instead of magic numbers
+			 */
 			if (lines_hidden <= 100 && (raw_len < 128 || raw_len > 130))
 				raw_len = 0;
 		} else
 			raw_len = 0;
 
+		/*
+		 * From this point on a pointer to line is used
+		 * and offset holds the offset to line
+		 */
+		line_ptr = line;
+		offset = 0;
+
+		/*
+		 * Skip over the first space in case of Format=Flowed
+		 * (RFC 3676 4.4 Space-Stuffing)
+		 */
+		if (part->format == FORMAT_FLOWED && line_ptr[0] == ' ') {
+			++line_ptr;
+			++offset;
+		}
+
+		/* inline pgp */
+		if (!(section & (VERBATIM_SECTION))) {
+			/* detect inline pgp sections before reversing dash escaping */
+			if ((hide_inline_data & PGP_ALL) && (section == TEXT_SECTION || section == SIGNATURE_SECTION || section == PGP_TXT_SECTION) && !strcmp(line_ptr, PGP_PUBLIC_KEY_TAG)) {
+				section &= ~(SIGNATURE_SECTION|PGP_TXT_SECTION);
+				section |= PGP_KEY_SECTION;
+				++non_attach_lines;
+				continue;
+			}
+
+			if ((hide_inline_data & PGP_ALL) && section == PGP_KEY_SECTION) {
+				++non_attach_lines;
+				if (!strcmp(line_ptr, PGP_PUBLIC_KEY_END_TAG)) { /* see above, PGP_PRIVATE_KEY_END_TAG */
+					section &= ~PGP_KEY_SECTION;
+					/* -> lang.c */
+					put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), _("OpenPGP pubic key block"), non_attach_lines);
+					non_attach_lines = 0;
+				}
+				continue;
+			}
+
+			/*
+			 * TODO: (capture and) skip "Hash:" Armor Header(s) (and include in message).
+			 */
+			if (section == TEXT_SECTION && !strcmp(line_ptr, PGP_SIGNED_TAG)) {
+				section |= PGP_TXT_SECTION;
+				/* -> lang.c */
+				put_cooked(LEN, wrap_lines, C_BODY, "%s", _("[-- PGP SIGNED MESSAGE --]\n"));
+				continue;
+			}
+
+			if ((section & PGP_TXT_SECTION) && (section & PGP_ANY_SECTION))
+				section &= ~PGP_TXT_SECTION;
+
+			/*
+			 * TODO: capture optional Armor Headers (e.g: "Version:") and include in message
+			 */
+			if ((hide_inline_data & PGP_ALL) && section == PGP_SIG_SECTION) {
+				++non_attach_lines;
+				if (!strcmp(line_ptr, PGP_SIG_END_TAG)) {
+					section &= ~PGP_SIG_SECTION;
+					/* -> lang.c */
+					put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), _("OpenPGP signature block"), non_attach_lines);
+					non_attach_lines = 0;
+				}
+				continue;
+			}
+
+			if ((section & PGP_TXT_SECTION) && (!strcmp(line_ptr, PGP_SIG_TAG) || !strcmp(line_ptr, PGP_PUBLIC_KEY_TAG)))
+				section = TEXT_SECTION;
+
+			if ((hide_inline_data & PGP_ALL) && section == TEXT_SECTION && !strcmp(line_ptr, PGP_SIG_TAG)) {
+				section = PGP_SIG_SECTION;
+				++non_attach_lines;
+				continue;
+			}
+
+			/* undo dash escape */
+			if ((section & PGP_TXT_SECTION) && *line_ptr == '-') {
+				if (*(line_ptr + 1) == ' ') {
+					line_ptr += 2;
+					offset += 2;
+				}
+#ifdef INLINE_DEBUG_MIME /* this is NOT MIME, but ...*/
+				else {
+					if (*(line_ptr + 2))
+						*(line_ptr + 2) = '\0';
+					else
+						if (*(line_ptr + 1) == '\n')
+							*(line_ptr + 1) = '\0';
+
+					put_cooked(LEN, wrap_lines, C_BODY, "[-- Error: dash escaped line start: \"%s\" malformed --]", line_ptr);
+					continue;
+				}
+#endif /* INLINE_DEBUG_MIME */
+			}
+		}
+		len = strlen(line_ptr);
+
 		/* look for verbatim marks, set in_verbatim only for lines in between */
 		if (curr_group->attribute->verbatim_handling > VERBATIM_NONE) {
 			if (section & VERBATIM_BEGIN) {
 				section &= ~VERBATIM_BEGIN;
 				section |= VERBATIM_SECTION;
 			} else {
-				if (section == TEXT_SECTION && MATCH_REGEX(verbatim_begin_regex, line, len))
-					section = VERBATIM_BEGIN;
+				if ((section == TEXT_SECTION || section == PGP_TXT_SECTION) && MATCH_REGEX(verbatim_begin_regex, line_ptr, len))
+					section |= VERBATIM_BEGIN;
 			}
-			if ((section & VERBATIM_SECTION) && MATCH_REGEX(verbatim_end_regex, line, len)) {
+			if ((section & VERBATIM_SECTION) && MATCH_REGEX(verbatim_end_regex, line_ptr, len)) {
 				section &= ~VERBATIM_SECTION;
 				section |= VERBATIM_END;
 			}
@@ -1349,7 +1444,8 @@ process_text_body_part(
 			 */
 			if (strcasecmp(tinrc.mm_local_charset, "ISO-8859-11") && (IS_LOCAL_CHARSET("ISO-8859") || IS_LOCAL_CHARSET("UTF-8")) && curr_group->attribute->suppress_soft_hyphens) {
 				remove_soft_hyphens(line);
-				len = strlen(line);
+				line_ptr = line + offset;
+				len = strlen(line_ptr);
 			}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
@@ -1362,7 +1458,7 @@ process_text_body_part(
 			 */
 			if (curr_group->attribute->trim_article_body && !(section & (UUE_SECTION | YENC_SECTION))) {
 				/* skip all consecutive blanks */
-				for (cp = line; (*cp == ' ') || (*cp == '\t'); cp++)
+				for (cp = line_ptr; (*cp == ' ') || (*cp == '\t'); cp++)
 					;
 				if (*cp == '\n') {	/* line was all blank */
 					if (section & (SHAR_SECTION | PGP_ANY_SECTION)) {
@@ -1389,7 +1485,7 @@ process_text_body_part(
 					if (first_line_blank)
 						first_line_blank = FALSE;
 					if (lines_skipped && (!(section & SIGNATURE_SECTION) || curr_group->attribute->show_signatures)) {
-						if (strcmp(line, SIGDASHES) != 0 || curr_group->attribute->show_signatures) {
+						if (strcmp(line_ptr, SIGDASHES) != 0 || curr_group->attribute->show_signatures) {
 							if (curr_group->attribute->trim_article_body & COMPACT_MULTIPLE)
 								put_cooked(1, TRUE, (section & SIGNATURE_SECTION) ? C_SIG : C_BODY, "\n");
 							else
@@ -1409,7 +1505,7 @@ process_text_body_part(
 			/*
 			 * Detect and skip signatures if necessary
 			 */
-			if (!(section & SIGNATURE_SECTION) && STRCMPEQ(line, SIGDASHES)) {
+			if (!(section & SIGNATURE_SECTION) && STRCMPEQ(line_ptr, SIGDASHES)) {
 				section |= SIGNATURE_SECTION;
 				if (section & UUE_SECTION) {
 					section &= ~UUE_SECTION;
@@ -1437,8 +1533,8 @@ process_text_body_part(
 			 *         abuse raw_len as a flag)
 			 */
 			if (!(section & (SIGNATURE_SECTION | UUE_SECTION | PGP_ANY_SECTION | SHAR_SECTION))) { /* yenc? */
-				if (*line == '=' && *(line + 1) == 'y') { /* can't show up in yenc-bodies */
-					if (MATCH_REGEX(yencbegin_regex, line, len)) {
+				if (*line_ptr == '=' && *(line_ptr + 1) == 'y') { /* can't show up in yenc-bodies */
+					if (MATCH_REGEX(yencbegin_regex, line_ptr, len)) {
 #	ifdef INLINE_DEBUG_MIME
 						if (section & YENC_SECTION) {
 							DEBUG_PUT_YENC_UNEXPECTED_ITEM("=ybegin");
@@ -1446,7 +1542,7 @@ process_text_body_part(
 #	endif /* INLINE_DEBUG_MIME */
 							section |= YENC_SECTION;
 
-						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_bsize", line);
+						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_bsize", line_ptr);
 						if (ncs && *ncs) {
 							y_bsize = strtol(ncs, &cp, 10);
 							if (cp == ncs || *ncs == '-') {
@@ -1461,27 +1557,29 @@ process_text_body_part(
 						FreeAndNull(ncs);
 
 						/* part / total are optional */
-						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_part", line);
+						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_part", line_ptr);
 						if (ncs && *ncs) {
-							y_part = strtol(ncs, &cp, 10);
-							if (cp == ncs || *ncs == '-') {
+							y_tmp = strtol(ncs, &cp, 10);
+							if (cp == ncs || *ncs == '-' || y_tmp < 0 || y_tmp > INT_MAX) {
 								DEBUG_PUT_YENC_INVALID_ITEM("part", ncs, "=ybegin");
 								y_part = 0;
-							}
+							} else
+								y_part = (int) y_tmp;
 						}
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_total", line);
+						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_total", line_ptr);
 						if (ncs && *ncs) {
-							y_total = strtol(ncs, &cp, 10);
-							if (cp == ncs || *ncs == '-') {
+							y_tmp = strtol(ncs, &cp, 10);
+							if (cp == ncs || *ncs == '-' || y_tmp < 0 || y_tmp > INT_MAX) {
 								DEBUG_PUT_YENC_INVALID_ITEM("total", ncs, "=ybegin");
 								y_total = 0;
-							}
+							} else
+								y_total = (int) y_tmp;
 						}
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_line", line);
+						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_line", line_ptr);
 						if (ncs && *ncs) {
 #	ifdef INLINE_DEBUG_MIME
 							if (!strtol(ncs, &cp, 10) || cp == ncs || *ncs == '-')
@@ -1494,7 +1592,7 @@ process_text_body_part(
 #	endif /* INLINE_DEBUG_MIME */
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_name", line);
+						ncs = regex_get_substring_by_name(&yencbegin_regex, "y_name", line_ptr);
 						if (ncs && *ncs)
 							y_name = my_strdup(ncs);
 #	ifdef INLINE_DEBUG_MIME
@@ -1510,8 +1608,8 @@ process_text_body_part(
 						curruue->encoding = ENCODING_YENC;
 						FreeAndNull(y_name);
 						y_ccrc = tin_crc32(0L, NULL, 0);
-					} else if (MATCH_REGEX(yencpart_regex, line, len)) {
-						ncs = regex_get_substring_by_name(&yencpart_regex, "y_pbegin", line);
+					} else if (MATCH_REGEX(yencpart_regex, line_ptr, len)) {
+						ncs = regex_get_substring_by_name(&yencpart_regex, "y_pbegin", line_ptr);
 						if (ncs && *ncs) {
 							y_pbegin = strtol(ncs, &cp, 10);
 							if (cp == ncs || *ncs == '-') {
@@ -1525,7 +1623,7 @@ process_text_body_part(
 #	endif /* INLINE_DEBUG_MIME */
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencpart_regex, "y_pend", line);
+						ncs = regex_get_substring_by_name(&yencpart_regex, "y_pend", line_ptr);
 						if (ncs && *ncs) {
 							y_pend = strtol(ncs, &cp, 10);
 							if (cp == ncs || *ncs == '-') {
@@ -1552,7 +1650,7 @@ process_text_body_part(
 						if (!(section & YENC_SECTION))
 							DEBUG_PUT_YENC_UNEXPECTED_ITEM("=ypart");
 #	endif /* INLINE_DEBUG_MIME */
-					} else if (MATCH_REGEX(yencend_regex, line, len)) {
+					} else if (MATCH_REGEX(yencend_regex, line_ptr, len)) {
 						unsigned long y_ecrc = 0L; /* crc32 mentioned in end line */
 						unsigned long y_epcrc = 0L; /* pcrc32 mentioned in end line */
 
@@ -1562,7 +1660,7 @@ process_text_body_part(
 						}
 
 						y_esize = 0L;
-						ncs = regex_get_substring_by_name(&yencend_regex, "y_esize", line);
+						ncs = regex_get_substring_by_name(&yencend_regex, "y_esize", line_ptr);
 						if (ncs && *ncs) {
 							y_esize = strtol(ncs, &cp, 10);
 							if (cp == ncs || *ncs == '-') {
@@ -1576,7 +1674,7 @@ process_text_body_part(
 #	endif /* INLINE_DEBUG_MIME */
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencend_regex, "y_epcrc", line);
+						ncs = regex_get_substring_by_name(&yencend_regex, "y_epcrc", line_ptr);
 						if (ncs && *ncs) {
 							y_epcrc = strtol(ncs, &cp, 16);
 							if (cp == ncs || *ncs == '-') {
@@ -1592,7 +1690,7 @@ process_text_body_part(
 #	endif /* INLINE_DEBUG_MIME */
 						FreeAndNull(ncs);
 
-						ncs = regex_get_substring_by_name(&yencend_regex, "y_ecrc", line);
+						ncs = regex_get_substring_by_name(&yencend_regex, "y_ecrc", line_ptr);
 						if (ncs && *ncs) {
 							y_ecrc = strtol(ncs, &cp, 16);
 							if (cp == ncs || *ncs == '-') {
@@ -1692,7 +1790,7 @@ process_text_body_part(
 
 			section &= ~UUE_BEGIN;
 
-			if (MATCH_REGEX(uubegin_regex, line, len)) {
+			if (MATCH_REGEX(uubegin_regex, line_ptr, len)) {
 				REGEX_SIZE *ovector = regex_get_ovector_pointer(&uubegin_regex);
 
 				if (section & UUE_SECTION) { /* previous uue part incomplete and the current one follows without gap */
@@ -1702,19 +1800,19 @@ process_text_body_part(
 					section |= UUE_SECTION;
 
 				section |= UUE_BEGIN;
-				if (*(line + ovector[0] + 5) != '-')
-					curruue = new_uue(&part, line + ovector[1]);
+				if (*(line_ptr + ovector[0] + 5) != '-')
+					curruue = new_uue(&part, line_ptr + ovector[1]);
 				else { /* must be "begin-encoded" */
-					cp = my_malloc(strlen(line + ovector[1]));
+					cp = my_malloc(strlen(line_ptr + ovector[1]));
 
-					if (b642str(line + ovector[1], cp) == -1)
-						strcpy(cp, line + ovector[1]);
+					if (b642str(line_ptr + ovector[1], cp) == -1)
+						strcpy(cp, line_ptr + ovector[1]);
 					curruue = new_uue(&part, cp);
 					free(cp);
 				}
 				if (hide_inline_data & UUE_ALL)
 					continue;				/* Don't cook the 'begin' line */
-			} else if (STRNCMPEQ(line, "end\n", 4)) {
+			} else if (STRNCMPEQ(line_ptr, "end\n", 4)) {
 				if (section & UUE_SECTION) {
 					section &= ~UUE_SECTION;
 					if (hide_inline_data & UUE_ALL) {
@@ -1729,8 +1827,8 @@ process_text_body_part(
 			 */
 			is_uubody = 0;
 
-			if (MATCH_REGEX(uubody_regex, line, len)) {
-				sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
+			if (MATCH_REGEX(uubody_regex, line_ptr, len)) {
+				sum = (((*line_ptr) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
 				/*
 				 * sum = 0 in a uubody only on the last line, a single `
 				 * +1 for the \n
@@ -1740,16 +1838,16 @@ process_text_body_part(
 
 #ifdef DEBUG_ART
 				if (debug & DEBUG_MISC)
-					fprintf(stderr, "%d sum=%d len=%lu %s", is_uubody, sum, len, line);
+					fprintf(stderr, "%d sum=%d len=%lu %s", is_uubody, sum, (unsigned long) len, line_ptr);
 #endif /* DEBUG_ART */
 			} else {
-				if (MATCH_REGEX(xxbody_regex, line, len)) {
+				if (MATCH_REGEX(xxbody_regex, line_ptr, len)) {
 					/*
 					 * we don't bother to sum != 0x7f as it will be
 					 * caught be the check against len (limited via
 					 * the regex) anyway
 					 */
-					if (*line >= '+' && *line <= 'z' && (sum = xxenc[(*line - '+')]) >= 0) {
+					if (*line_ptr >= '+' && *line_ptr <= 'z' && (sum = xxenc[(*line_ptr - '+')]) >= 0) {
 						sum = sum * 4 / 3;
 						if ((sum == 0 && len == 1 + 1) || len == (unsigned) (sum + 1 + 1))
 							is_uubody = 2;
@@ -1759,7 +1857,7 @@ process_text_body_part(
 						sum = 0;
 
 					if (debug & DEBUG_MISC)
-						fprintf(stderr, "%d sum=%d len=%lu %s", is_uubody, sum, len, line);
+						fprintf(stderr, "%d sum=%d len=%lu %s", is_uubody, sum, (unsigned long) len, line_ptr);
 #endif /* DEBUG_ART */
 				}
 			}
@@ -1769,7 +1867,7 @@ process_text_body_part(
 					curruue->line_count++;
 					curruue->bytes += len;
 				} else {
-					if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
+					if (line_ptr[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
 						/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
 						section &= ~UUE_SECTION;
 						if (hide_inline_data & UUE_ALL)
@@ -1817,7 +1915,7 @@ process_text_body_part(
 		 * Don't do any further handling of uue || verbatim lines
 		 */
 		if (section & (UUE_SECTION | YENC_SECTION)) {
-			put_cooked(max_line_len, wrap_lines, flags, "%s", line);
+			put_cooked(max_line_len, wrap_lines, flags, "%s", line_ptr);
 			continue;
 		} else if ((section & VERBATIM_BEGIN) && curr_group->attribute->verbatim_handling >= VERBATIM_HIDE_MARKS) {
 			if (curr_group->attribute->verbatim_handling == VERBATIM_HIDE_ALL)
@@ -1826,7 +1924,8 @@ process_text_body_part(
 		} else if (section & VERBATIM_SECTION) {
 			if (curr_group->attribute->verbatim_handling < VERBATIM_HIDE_ALL) {
 				expand_ctrl_chars(&line, &max_line_len, 8);
-				put_cooked(max_line_len, wrap_lines, flags, "%s", line);
+				line_ptr = line + offset;
+				put_cooked(max_line_len, wrap_lines, flags, "%s", line_ptr);
 			}
 			continue;
 		} else if (section & VERBATIM_END) {
@@ -1838,20 +1937,20 @@ process_text_body_part(
 #ifdef HAVE_COLOR
 		/* keep order in sync with color.c:draw_pager_line() */
 		if (quote_regex3.re) {
-			if (MATCH_REGEX(quote_regex3, line, len))
+			if (MATCH_REGEX(quote_regex3, line_ptr, len))
 				flags |= C_QUOTE3;
 			else if (quote_regex2.re) {
-				if (MATCH_REGEX(quote_regex2, line, len))
+				if (MATCH_REGEX(quote_regex2, line_ptr, len))
 					flags |= C_QUOTE2;
 				else if (curr_group->attribute->extquote_handling && extquote_regex.re) {
-					if (MATCH_REGEX(extquote_regex, line, len))
+					if (MATCH_REGEX(extquote_regex, line_ptr, len))
 						flags |= C_EXTQUOTE;
 					else if (quote_regex.re) {
-						if (MATCH_REGEX(quote_regex, line, len))
+						if (MATCH_REGEX(quote_regex, line_ptr, len))
 							flags |= C_QUOTE1;
 					}
 				} else if (quote_regex.re) {
-					if (MATCH_REGEX(quote_regex, line, len))
+					if (MATCH_REGEX(quote_regex, line_ptr, len))
 						flags |= C_QUOTE1;
 				}
 			}
@@ -1863,38 +1962,19 @@ process_text_body_part(
 		CHECK_URI(url_regex, C_URL);
 		CHECK_URI(mail_regex, C_MAIL);
 #else
-		if (MATCH_REGEX(url_regex, line, len))
+		if (MATCH_REGEX(url_regex, line_ptr, len))
 			flags |= C_URL;
-		if (MATCH_REGEX(mail_regex, line, len))
+		if (MATCH_REGEX(mail_regex, line_ptr, len))
 			flags |= C_MAIL;
 #endif /* HAVE_LIBURIPARSER */
 		/* see debian bug #687198 */
-		if (MATCH_REGEX(news_regex, line, len))
+		if (MATCH_REGEX(news_regex, line_ptr, len))
 			flags |= C_NEWS;
 
 		if (expand_ctrl_chars(&line, &max_line_len, tabwidth))
 			flags |= C_CTRLL;				/* Line contains form-feed */
-
-		buf = line;
-
-		/*
-		 * Skip over the first space in case of Format=Flowed
-		 * (RFC 3676 4.4 Space-Stuffing)
-		 * #if 1
-		 * _except_ for lines starting with " ", " >" or " From "
-		 * (this is NOT what the RFC says).
-		 * #endif
-		 * quote depth adjustment, 1st level only.
-		 */
-		if (part->format == FORMAT_FLOWED && line[0] == ' ') {
-			if (line[1] == '>' && (flags & C_QUOTE1))
-				flags &= ~C_QUOTE1;
-
-#if 0
-			if (line[1] != '>' && line[1] != ' ' && strncmp(line + 1, "From ", 5))
-#endif /* 0 */
-				++buf;
-		}
+		line_ptr = line + offset;
+		len = strlen(line_ptr);
 
 		/*
 		 * skip over what "looks" like an unexpected yenc line.
@@ -1906,7 +1986,7 @@ process_text_body_part(
 #else
 			flags &= ~(C_CTRLL | C_QUOTE1 | C_QUOTE2 | C_QUOTE3);
 #endif /* HAVE_COLOR */
-			if (flags == C_BODY && (*line != '=' || *(line + 1) != 'y') && (strcmp(raw_line, line) || lines_hidden > 100)) {
+			if (flags == C_BODY && (*line_ptr != '=' || *(line_ptr + 1) != 'y') && (strcmp(raw_line + offset, line_ptr) || lines_hidden > 100)) {
 				if (!(section & (SHAR_SECTION | PGP_ANY_SECTION))) {
 					++lines_hidden;
 					continue;
@@ -1914,54 +1994,17 @@ process_text_body_part(
 			}
 		}
 
-		/* TODO: shall we also look for PGP_PRIVATE_KEY_TAG? */
-		if ((hide_inline_data & PGP_ALL) && section == TEXT_SECTION && !strcmp(line, PGP_PUBLIC_KEY_TAG)) {
-			section |= PGP_KEY_SECTION;
-			++non_attach_lines;
-			continue;
-		}
-		if ((hide_inline_data & PGP_ALL) && section == PGP_KEY_SECTION) {
-			if (!strcmp(line, PGP_PUBLIC_KEY_END_TAG)) { /* see above, PGP_PRIVATE_KEY_END_TAG */
-				++non_attach_lines;
-				section &= ~PGP_KEY_SECTION;
-				/* -> lang.c && _("OpenPGP public key block") */
-				put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), "OpenPGP pubic key block", non_attach_lines);
-				non_attach_lines = 0;
-				continue;
-			} else
-				++non_attach_lines;
-			continue;
-		}
-
-		if ((hide_inline_data & PGP_ALL) && section == TEXT_SECTION && !strcmp(line, PGP_SIG_TAG)) {
-			section |= PGP_SIG_SECTION;
-			++non_attach_lines;
-			continue;
-		}
-		if ((hide_inline_data & PGP_ALL) && section == PGP_SIG_SECTION) {
-			if (!strcmp(line, PGP_SIG_END_TAG)) {
-				++non_attach_lines;
-				section &= ~PGP_SIG_SECTION;
-				/* -> lang.c && _("OpenPGP signature block") */
-				put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), "OpenPGP signature block", non_attach_lines);
-				non_attach_lines = 0;
-				continue;
-			} else
-				++non_attach_lines;
-			continue;
-		}
-
 		if ((hide_inline_data & SHAR_ALL) && section == SHAR_SECTION) {
 			++non_attach_lines;
-			if (MATCH_REGEX(shar_end_regex, line, len)) {
-				/* -> lang.c && _("shell archive (shar)") */
-				put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), "shell archive (shar)", non_attach_lines);
+			if (MATCH_REGEX(shar_end_regex, line_ptr, len)) {
+				/* -> lang.c */
+				put_cooked(LEN, wrap_lines, C_BODY, P_(txt_cook_lines_hidden_sp[0], txt_cook_lines_hidden_sp[1], non_attach_lines), _("shell archive (shar)"), non_attach_lines);
 				non_attach_lines = 0;
 				section &= ~SHAR_SECTION;
 			}
 			continue;
 		} else {
-			if ((hide_inline_data & SHAR_ALL) && section == TEXT_SECTION && MATCH_REGEX(shar_regex, line, len)) {
+			if ((hide_inline_data & SHAR_ALL) && section == TEXT_SECTION && MATCH_REGEX(shar_regex, line_ptr, len)) {
 				++non_attach_lines;
 				section |= SHAR_SECTION;
 				continue;
@@ -1973,10 +2016,10 @@ process_text_body_part(
 		 *
 		 * be aware that trim_article_body may have kicked in before!
 		 */
-		if (MATCH_REGEX(hideline_regex, buf, len))
+		if (MATCH_REGEX(hideline_regex, line_ptr, len))
 			continue;
 
-		put_cooked(max_line_len, wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", buf);
+		put_cooked(max_line_len, wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", line_ptr);
 	} /* while */
 
 	/*
@@ -2231,7 +2274,7 @@ cook_article(
 			} while (!found && *(++strptr) != NULL);
 
 			/* unstructured but must not be decoded (see also rfc2047.c:do_rfc15211522_encode()) */
-			if (l == NULL && (!strncasecmp(line, "References: ", 12) || !strncasecmp(line, "Message-ID: ", 12) || !strncasecmp(line, "Date: ", 6) || !strncasecmp(line, "Newsgroups: ", 12) || !strncasecmp(line, "Distribution: ", 14) || !strncasecmp(line, "Followup-To: ", 13) || !strncasecmp(line, "X-Face: ", 8) || !strncasecmp(line, "Cancel-Lock: ", 13) || !strncasecmp(line, "Cancel-Key: ", 12) || !strncasecmp(line, "Supersedes: ", 12) || !strncasecmp(line, "Path: ", 6) || !strncasecmp(line, "Xref: ", 6)))
+			if (l == NULL && (!strncasecmp(line, "Date: ", 6) || !strncasecmp(line, "Path: ", 6) || !strncasecmp(line, "Xref: ", 6) || !strncasecmp(line, "X-Face: ", 8) || !strncasecmp(line, "Newsgroups: ", 12) || !strncasecmp(line, "References: ", 12) || !strncasecmp(line, "Message-ID: ", 12) || !strncasecmp(line, "Supersedes: ", 12) || !strncasecmp(line, "Cancel-Key: ", 12) || !strncasecmp(line, "Cancel-Lock: ", 13) || !strncasecmp(line, "Followup-To: ", 13) || !strncasecmp(line, "Distribution: ", 14)))
 				l = my_strdup(line);
 
 			if (l == NULL)
