@@ -335,12 +335,12 @@ debug_print_filters(
 		 */
 		num = glob_filter.num;
 		the_filter = glob_filter.filter;
-		fprintf(fp, "\n*** BEG GLOBAL FILTER=[%3d] ***\n", num);
+		fprintf(fp, "\n*** BEG GLOBAL FILTER=[%3d] ***\n\n", num);
 		for (i = 0; i < num; i++) {
 			debug_print_filter(fp, i, &the_filter[i]);
 			fprintf(fp, "\n");
 		}
-		fprintf(fp, "*** END GLOBAL FILTER ***\n");
+		fprintf(fp, "*** END GLOBAL FILTER ***\n\n");
 
 #	ifdef HAVE_FCHMOD
 		fchmod(fileno(fp), (S_IRUGO|S_IWUGO));
