@@ -3,10 +3,10 @@
  *  Module    : feed.c
  *  Author    : I. Lea
  *  Created   : 1991-08-31
- *  Updated   : 2025-01-14
+ *  Updated   : 2025-12-08
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@ struct t_counters {
 static char *get_save_filename(const struct t_group *group, int function, char *filename, int filelen);
 static t_bool expand_feed_filename(char *outpath, size_t outpath_len, const char *path);
 static t_bool feed_article(int art, int function, struct t_counters *counter, t_bool use_current, const char *data, const struct t_group *group);
-static t_function get_feed_key(int function, int level, struct t_art_stat *thread);
+static t_function get_feed_key(int function, int level, const struct t_art_stat *thread);
 static t_function get_post_proc_type(void);
 static void print_save_summary(t_function type, int fed);
 #ifndef DISABLE_PRINTING
@@ -230,7 +230,7 @@ static t_function
 get_feed_key(
 	int function,
 	int level,
-	struct t_art_stat *thread)
+	const struct t_art_stat *thread)
 {
 	constext *prompt;
 	t_function default_func, func;
@@ -350,6 +350,8 @@ get_feed_key(
  * Print a message like:
  * -- [Article|Thread|Tagged Articles] saved to [mailbox] [filenames] --
  * 'fed' is the number of articles we tried to save
+ *
+ * FIXME: translatable/plural-forms and fixed length buffer
  */
 static void
 print_save_summary(
@@ -363,12 +365,12 @@ print_save_summary(
 		wait_message(2, P_(txt_warn_not_all_arts_saved_sp[0], txt_warn_not_all_arts_saved_sp[1], num_save), fed, num_save);
 
 	switch (type) {
-		case FEED_HOT: /* FIXME: translatable/plural-forms */
-			snprintf(what, sizeof(what), _(txt_prefix_hot), P_(txt_article_sp[0], txt_article_sp[1], fed));
+		case FEED_HOT:
+			snprintf(what, sizeof(what), "%s", P_(txt_hot_article_sp[0], txt_hot_article_sp[1], fed));
 			break;
 
-		case FEED_TAGGED: /* FIXME: translatable/plural-forms */
-			snprintf(what, sizeof(what), _(txt_prefix_tagged), P_(txt_article_sp[0], txt_article_sp[1], fed));
+		case FEED_TAGGED:
+			snprintf(what, sizeof(what), "%s", P_(txt_tagged_article_sp[0], txt_tagged_article_sp[1], fed));
 			break;
 
 		case FEED_THREAD:
@@ -377,7 +379,7 @@ print_save_summary(
 
 		case FEED_ARTICLE:
 		case FEED_PATTERN:
-		default: /* FIXME: translatable/plural-forms */
+		default:
 			snprintf(what, sizeof(what), "%s", P_(txt_article_sp[0], txt_article_sp[1], fed));
 			break;
 	}
@@ -385,6 +387,7 @@ print_save_summary(
 	/*
 	 * We report the range of saved-to files for regular saves of > 1 articles
 	 */
+	/* FIXME: translatable/plural-forms */
 	if (num_save == 1 || save[0].mailbox)
 		snprintf(buf, sizeof(buf), _(txt_saved_to),
 			what, (save[0].mailbox ? _(txt_mailbox) : ""), save[0].path);
@@ -443,6 +446,9 @@ feed_article(
 		case FEED_AUTOSAVE:
 			progress_mesg = fmt_string("%s (%d/%d)", _(txt_saving), counter->total, counter->max);
 			break;
+
+		default:
+			break;
 	}
 
 	if (progress_mesg != NULL) {
@@ -492,7 +498,7 @@ feed_article(
 
 #ifndef DONT_HAVE_PIPING
 		case FEED_PIPE:
-			rewind(openartptr->raw);
+			fseek(openartptr->raw, 0L, SEEK_SET);
 			if (copy_fp(openartptr->raw, pipe_fp) == EPIPE) {	/* broken pipe in copy_fp() */
 				got_epipe = TRUE;
 				ok = FALSE;
@@ -567,7 +573,7 @@ feed_articles(
 	int function,
 	int level,
 	t_function type,
-	struct t_group *group,
+	const struct t_group *group,
 	int respnum)
 {
 	char *prompt;
@@ -723,10 +729,9 @@ feed_articles(
 				char *tmp;
 				char *smsg;
 #ifndef FORGERY
-				char from_name[PATH_LEN];
-
-				get_from_name(from_name, (struct t_group *) 0);
+				char from_name[HEADER_LEN];
 
+				get_from_name(from_name, sizeof(from_name), (const struct t_group *) 0);
 				if (strstr(from_name, arts[respnum].mailbox.from)) {
 #endif /* !FORGERY */
 					char buf[LEN];
@@ -884,8 +889,8 @@ feed_articles(
 				for_each_art_in_thread(art, i) {
 					if (!arts[art].matched)
 						continue;
-					arts[art].matched = FALSE;
 
+					arts[art].matched = FALSE;
 					/* Keep going - don't abort on errors */
 					if (feed_article(art, function, &counter, use_current, outpath, group)) {
 						if (feed_type == FEED_HOT)
@@ -1022,16 +1027,26 @@ got_epipe_while_piping:
 				info_message(_(txt_no_next_unread_art));
 			else {
 				if (counter.success && level != PAGE_LEVEL) {
-					const char *ptr;
+					switch (feed_type) {
+						case FEED_THREAD:
+							info_message((function == FEED_MARK_READ) ?
+								_(txt_marked_thread_as_read) :
+								_(txt_marked_thread_as_unread));
+							break;
 
-					ptr = function == FEED_MARK_READ ? _(txt_marked_as_read) : _(txt_marked_as_unread);
-					if (feed_type == FEED_THREAD) {
-						info_message(ptr, _(txt_thread_upper));
-					} else if (feed_type == FEED_ARTICLE) {
-						info_message(ptr, _(txt_article_upper));
-					} else {
-						ptr = function == FEED_MARK_READ ? _(txt_marked_arts_as_read) : _(txt_marked_arts_as_unread);
-						info_message(ptr, counter.success, counter.max, P_(txt_article_sp[0], txt_article_sp[1], counter.max));
+						case FEED_ARTICLE:
+							info_message((function == FEED_MARK_READ) ?
+								_(txt_marked_article_as_read) :
+								_(txt_marked_article_as_unread));
+							break;
+
+						default:
+							/* FIXME: translatable/plural-forms */
+							info_message((function == FEED_MARK_READ) ?
+								_(txt_marked_arts_as_read) :
+								_(txt_marked_arts_as_unread),
+								counter.success, counter.max, P_(txt_article_sp[0], txt_article_sp[1], counter.max));
+							break;
 					}
 				}
 			}
@@ -1084,7 +1099,7 @@ print_file(
 		return ok;
 	}
 
-	rewind(artinfo->raw);
+	fseek(artinfo->raw, 0L, SEEK_SET);
 	if (!curr_group->attribute->print_header && !(fseek(artinfo->raw, hdr->ext->offset, SEEK_SET))) {	/* -> start of body */
 		if (hdr->newsgroups)
 			fprintf(fp, "Newsgroups: %s\n", hdr->newsgroups);
