@@ -3,10 +3,10 @@
  *  Module    : group.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-02-25
+ *  Updated   : 2026-02-05
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -158,7 +158,6 @@ group_page(
 	int thread_depth;	/* Starting depth in threads we enter */
 	t_artnum old_artnum;
 	struct t_art_stat sbuf;
-	struct t_article *art;
 	t_bool flag;
 	t_bool xflag = FALSE;	/* 'X'-flag */
 	t_bool repeat_search;
@@ -182,12 +181,29 @@ group_page(
 	/*
 	 * update index file. quit group level if user aborts indexing
 	 */
-	if (!index_group(group)) {
+	if (!index_group(group, TRUE)) {
+		/*
+		 * can't see how this ever could leak, so why bother?
+		 * (next index_group() or tin_done() will take care
+		 * of the free()s and nothing else should modify arts[])
+		 * and if, why not also clear xref and mailbox.next?
+		 * (use free_art_array() then)
+		 */
+#if 0
+		struct t_article *art;
+
 		for_each_art(i) {
 			art = &arts[i];
 			FreeAndNull(art->refs);
 			FreeAndNull(art->msgid);
+			/* FreeAndNull(art->xref); */
+			/* free_mailbox_list(art->mailbox.next); */
 		}
+		/*
+		 * the obove is more or less free_art_array()
+		 * why not also do hash_reclaim() here?
+		 */
+#endif /* 0 */
 		curr_group = NULL;
 		tin_errno = 0;
 		return GRP_RETSELECT;
@@ -386,6 +402,88 @@ group_page(
 				show_group_page();
 				break;
 
+			/*
+			 * mark all articles older than <days> or <rfc5322-date-string> as read.
+			 * useful when subscribing to a new group with high-traffic.
+			 *
+			 * TODO: - do we need a [GROUP_]MARK_ANCIENT_ART_UNREAD?
+			 *       - shares a lot of code with
+			 *         select.c:selection_page():MARK_ANCIENT_ART_READ
+			 */
+			case MARK_ANCIENT_ART_READ:
+				{
+					char *p;
+					char tmp[LEN];
+					int days;
+					unsigned int cnt = 0;
+					t_bool only_unread = group->attribute->show_only_unread_arts;
+					time_t past = -1;
+
+					clear_message();
+					if (prompt_default_string(_(txt_group_mark_ancient_art_read), tmp, sizeof(tmp), NULL, HIST_OTHER)) {
+#ifdef HAVE_STRPTIME
+						struct tm tm;
+#endif /* HAVE_STRPTIME */
+
+						str_trim(tmp);
+
+#ifdef HAVE_STRPTIME
+						memset(&tm, 0, sizeof(tm));
+						if (strptime(tmp, *group->attribute->date_format, &tm))
+							past = mktime(&tm);
+#endif /* HAVE_STRPTIME */
+						if (past < 0 && (p = strchr(tmp, ' '))) { /* could be a date string, try to parse */
+							if ((past = parsedate(tmp, (struct _TIMEINFO *) 0)) < 0 && isalpha((unsigned char) tmp[0])) /* failed, try again with first "word" skipped */
+								past = parsedate(p + 1, (struct _TIMEINFO *) 0);
+						}
+
+						if (past < 0) { /* not a valid date string, then take the leading digits as days */
+							p = tmp;
+							if (*p == '-' || *p == '+') /* ignore one leading sign if present */
+								++p;
+							if (isdigit((unsigned char) *p)) {
+								if ((days = s2i(p, 0, 65535)) > 0) { /* Epoch + 65535 days == 2149-06-06T02:00:00+02:00 */
+									if (days <= (time((time_t *) 0) / DAY))
+										past = (time((time_t *) 0) - (days * DAY));
+								} else
+									past = -2;
+							}
+						}
+
+						if (past >= time((time_t *) 0)) /* also avoid "catchup" with past == time(0) */
+							past = -2;
+
+						switch (past) {
+							case -2:
+								info_message(_(txt_error_date_future), tmp);
+								break;
+
+							case -1:
+								info_message(_(txt_error_date_unparsable), tmp);
+								break;
+
+							default:
+								if (only_unread)
+									toggle_read_unread(FALSE);
+								for (n = 0; n < grpmenu.max; n++) {
+									for_each_art_in_thread(i, n) {
+										if (arts[i].status > ART_READ && arts[i].date < past) {
+											art_mark(curr_group, &arts[i], ART_READ);
+											++cnt;
+										}
+									}
+								}
+								if (only_unread)
+									toggle_read_unread(FALSE);
+								if (cnt)
+									show_group_page();
+								info_message(P_(txt_group_ancient_arts_marked_read_sp[0], txt_group_ancient_arts_marked_read_sp[1], cnt), cnt);
+								break;
+						}
+					}
+				}
+				break;
+
 			case GLOBAL_QUICK_FILTER_SELECT:		/* quickly auto-select article */
 			case GLOBAL_QUICK_FILTER_KILL:			/* quickly kill article */
 				if (grpmenu.curr < 0) {
@@ -513,11 +611,11 @@ group_page(
 						const char *eyde = input_history[HIST_MESSAGE_ID][(hist_pos[HIST_MESSAGE_ID] - 1 + HIST_SIZE) % HIST_SIZE];
 						int sc = selmenu.curr;
 
-						if (*eyde) {
+						if (eyde && *eyde) {
 							switch (show_article_by_msgid(eyde)) {
 								case LOOKUP_OK:
 									selmenu.curr = sc;
-									if (!index_group(curr_group))
+									if (!index_group(curr_group, TRUE))
 										ret_code = GRP_RETSELECT; /* TODO: return GRP_RETSELECT; ? */
 									break;
 
@@ -603,7 +701,8 @@ group_page(
 				break;
 
 			case GLOBAL_QUIT:	/* return to group selection page */
-				if (num_of_tagged_arts && prompt_yn(_(txt_quit_despite_tags), TRUE) != 1)
+				/* TODO: add %d-num_of_tagged_arts counter in prompt? */
+				if (num_of_tagged_arts && prompt_yn(P_(txt_quit_despite_tags_sp[0], txt_quit_despite_tags_sp[1], num_of_tagged_arts), TRUE) != 1)
 					break;
 				if (xflag && TINRC_CONFIRM_SELECT && (prompt_yn(_(txt_confirm_select_on_exit), FALSE) != 1)) {
 					undo_auto_select_arts();
@@ -613,7 +712,8 @@ group_page(
 				break;
 
 			case GLOBAL_QUIT_TIN:		/* quit */
-				if (num_of_tagged_arts && prompt_yn(_(txt_quit_despite_tags), TRUE) != 1)
+				/* TODO: add %d-num_of_tagged_arts counter in prompt? */
+				if (num_of_tagged_arts && prompt_yn(P_(txt_quit_despite_tags_sp[0], txt_quit_despite_tags_sp[1], num_of_tagged_arts), TRUE) != 1)
 					break;
 				if (xflag && TINRC_CONFIRM_SELECT && (prompt_yn(_(txt_confirm_select_on_exit), FALSE) != 1)) {
 					undo_auto_select_arts();
@@ -744,7 +844,7 @@ group_page(
 					else
 						draw_subject_arrow();
 
-					info_message(tagged ? _(txt_prefix_untagged) : _(txt_prefix_tagged), txt_thread_sp[0]);
+					info_message(tagged ? _(txt_untagged_thread) : _(txt_tagged_thread));
 				}
 				break;
 
@@ -796,8 +896,6 @@ group_page(
 				if (grpmenu.curr < 0)
 					info_message(_(txt_no_arts));
 				else {
-					const char *ptr;
-
 					if (range_active) {
 						/*
 						 * We are tied to following base[] here, not arts[], as we operate on
@@ -813,16 +911,13 @@ group_page(
 						}
 						range_active = FALSE;
 						show_group_page();
-						ptr = _(txt_base_article_range);
-					} else {
+					} else
 						art_mark(group, &arts[base[grpmenu.curr]], ART_WILL_RETURN);
-						ptr = _(txt_base_article);
-					}
 
 					show_group_title(TRUE);
 					build_sline(grpmenu.curr);
 					draw_subject_arrow();
-					info_message(_(txt_marked_as_unread), ptr);
+					info_message(range_active ? _(txt_base_article_range_marked_read) : _(txt_base_article_marked_read));
 				}
 				break;
 
@@ -1129,8 +1224,7 @@ toggle_read_unread(
 	if (force)
 		curr_group->attribute->show_only_unread_arts = TRUE;	/* Yes - really, we change it in a bit */
 
-	wait_message(0, _(txt_reading_arts), /* TODO: plural-forms? ("unread" fr:("non lu", "non lus")) */
-		(curr_group->attribute->show_only_unread_arts) ? _(txt_all) : _(txt_unread));
+	wait_message(0, (curr_group->attribute->show_only_unread_arts) ? _(txt_reading_all_arts) : _(txt_reading_unread_arts));
 
 	if (grpmenu.curr >= 0) {
 		if (curr_group->attribute->show_only_unread_arts || new_responses(grpmenu.curr))
@@ -1303,8 +1397,8 @@ build_sline(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wtmp, *wtmp2;
 #else
-	int fill, gap;
-	size_t len_start;
+	int fill;
+	size_t len_start, gap;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 #ifdef USE_CURSES
@@ -1369,7 +1463,7 @@ build_sline(
 				free(buf);
 				break;
 
-			case 'F':	/* from */
+			case 'F':	/* from */ /* TODO: BiDi */
 				if (curr_group->attribute->show_author != SHOW_FROM_NONE) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 					get_author(FALSE, &arts[j], tmp, sizeof(tmp) - 1);
@@ -1386,11 +1480,12 @@ build_sline(
 #else
 					len_start = strwidth(buffer);
 					get_author(FALSE, &arts[j], buffer + strlen(buffer), grp_fmt.len_from);
-					fill = grp_fmt.len_from - (strwidth(buffer) - len_start);
-					gap = strlen(buffer);
-					for (k = 0; k < fill; k++)
-						buffer[gap + k] = ' ';
-					buffer[gap + fill] = '\0';
+					if ((fill = grp_fmt.len_from - (strwidth(buffer) - len_start)) > 0) {
+						gap = strlen(buffer);
+						for (k = 0; k < fill; k++)
+							buffer[gap + k] = ' ';
+						buffer[gap + fill] = '\0';
+					}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				}
 				break;
@@ -1471,7 +1566,7 @@ build_sline(
 				strcat(buffer, tin_ltoa(sbuf.score, grp_fmt.len_score));
 				break;
 
-			case 's':	/* thread/subject */
+			case 's':	/* thread/subject */ /* TODO: BiDi */
 				len = curr_group->attribute->show_author != SHOW_FROM_NONE ? grp_fmt.len_subj : grp_fmt.len_subj + grp_fmt.len_from;
 
 				if (sbuf.multipart_have > 1) /* We have a multipart msg so lets built our new header info. */
@@ -1493,11 +1588,12 @@ build_sline(
 #else
 				len_start = strwidth(buffer);
 				strncat(buffer, arts_sub, len);
-				fill = len - (strwidth(buffer) - len_start);
-				gap = strlen(buffer);
-				for (k = 0; k < fill; k++)
-					buffer[gap + k] = ' ';
-				buffer[gap + fill] = '\0';
+				if ((fill = len - (strwidth(buffer) - len_start)) > 0) {
+					gap = strlen(buffer);
+					for (k = 0; k < fill; k++)
+						buffer[gap + k] = ' ';
+					buffer[gap + fill] = '\0';
+				}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				break;
 
@@ -1529,7 +1625,7 @@ show_group_title(
 {
 	char buf[LEN], tmp[LEN], keyhelp[MAXKEYLEN], flag;
 	char *grpname;
-	int i, len, keyhelplen, art_cnt = 0, recent_art_cnt = 0, selected_art_cnt = 0, read_selected_art_cnt = 0, killed_art_cnt = 0;
+	int i, len, keyhelplen, art_cnt = 0, recent_art_cnt = 0, selected_art_cnt = 0, read_selected_art_cnt = 0, killed_art_cnt = 0, curr_getart_limit;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wtmp, *wtmp2;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1570,32 +1666,33 @@ show_group_title(
 		grpmenu.max, *txt_threading[curr_group->attribute->thread_articles]);
 
 	/* article count */
-	if ((cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit)
-		snprintf(tmp, sizeof(tmp), " %d/%d%"T_CHAR_FMT,
-			(cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit, art_cnt,
-			(wint_t) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
+	curr_getart_limit = getart_limit_set_from_scope(curr_group) ? curr_group->attribute->getart_limit : (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
+
+	if (curr_getart_limit)
+		snprintf(tmp, sizeof(tmp), " %d/%d%"T_CHAR_FMT, curr_getart_limit, art_cnt,
+			(T_CHAR_TYPE) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	else
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
 			art_cnt,
-			(wint_t) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
+			(T_CHAR_TYPE) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
 	/* selected articles */
 	if (curr_group->attribute->show_only_unread_arts)
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-			selected_art_cnt, (wint_t) tinrc.art_marked_selected);
+			selected_art_cnt, (T_CHAR_TYPE) tinrc.art_marked_selected);
 	else
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT" %d%"T_CHAR_FMT,
-			selected_art_cnt, (wint_t) tinrc.art_marked_selected,
-			read_selected_art_cnt, (wint_t) tinrc.art_marked_read_selected);
+			selected_art_cnt, (T_CHAR_TYPE) tinrc.art_marked_selected,
+			read_selected_art_cnt, (T_CHAR_TYPE) tinrc.art_marked_read_selected);
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
 	/* recent articles */
 	if (tinrc.recent_time) {
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-			recent_art_cnt, (wint_t) tinrc.art_marked_recent);
+			recent_art_cnt, (T_CHAR_TYPE) tinrc.art_marked_recent);
 
 		if (sizeof(buf) > strlen(buf) + strlen(tmp))
 			strcat(buf, tmp);
@@ -1603,7 +1700,7 @@ show_group_title(
 
 	/* killed articles */
 	snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-		killed_art_cnt, (wint_t) tinrc.art_marked_killed);
+		killed_art_cnt, (T_CHAR_TYPE) tinrc.art_marked_killed);
 
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
@@ -1826,7 +1923,8 @@ group_catchup(
 	char buf[LEN];
 	int pyn = 1;
 
-	if (num_of_tagged_arts && prompt_yn(_(txt_catchup_despite_tags), TRUE) != 1)
+	/* TODO: add %d-num_of_tagged_arts counter in prompt? */
+	if (num_of_tagged_arts && prompt_yn(P_(txt_catchup_despite_tags_sp[0], txt_catchup_despite_tags_sp[1], num_of_tagged_arts), TRUE) != 1)
 		return 0;
 
 	snprintf(buf, sizeof(buf), _(txt_mark_arts_read), (func == CATCHUP_NEXT_UNREAD) ? _(txt_enter_next_unread_group) : "");
@@ -1874,6 +1972,10 @@ prompt_getart_limit(
 	clear_message();
 	if ((p = tin_getline(_(txt_enter_getart_limit), 2, NULL, 0, FALSE, HIST_OTHER)) != NULL) {
 		tinrc.getart_limit = s2i(p, INT_MIN, INT_MAX);
+		if (scopes[0].attribute->getart_limit != tinrc.getart_limit) {
+			scopes[0].attribute->getart_limit = tinrc.getart_limit;
+			assign_attributes_to_groups();
+		}
 		if (!errno)
 			ret = TRUE;
 	}
