@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 2025-04-07
+ *  Updated   : 2025-12-12
  *
- * Copyright (c) 1997-2025 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2026 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -97,7 +97,7 @@ get_domain_name(
 	if (domain[0] == '/' && domain[1]) {
 		/* read domainname from specified file */
 		if ((fp = tin_fopen(domain, "r")) != NULL) {
-			while (fgets(buff, (int) sizeof(buff), fp) != NULL) {
+			while (fgets(buff, sizeof(buff), fp) != NULL) {
 				if (buff[0] == '#' || buff[0] == '\n')
 					continue;
 
@@ -122,7 +122,14 @@ get_domain_name(
 #ifdef HAVE_GETHOSTBYNAME
 #	define MAXLINELEN	1024
 #	define WSP	" \f\t\v"
-/* find FQDN - gethostbyaddr() */
+/*
+ * find FQDN - gethostbyaddr()
+ *
+ * TODO: we rely on gethostbyname() and gethostbyaddr(), but i.e,
+ *       __sortix__ "only" has the newer getaddrinfo()/getnameinfo()
+ *       so we fail to detect the FQDN (if not set via --with-domain-name)
+ *       and end up with force_no_post = TRUE
+ */
 const char *
 get_fqdn(
 	const char *host)
@@ -134,7 +141,7 @@ get_fqdn(
 	struct hostent *hp;
 	struct in_addr in = { 0 };
 
-	*fqdn = '\0';
+	fqdn[0] = '\0';
 	domain = NULL;
 	name[MAXHOSTNAMELEN] = '\0';
 
@@ -277,14 +284,21 @@ get_full_name(
 void
 get_from_name(
 	char *from_name,
+	size_t len,
 	const struct t_group *thisgrp)
 {
 	const char *fromhost = domain_name;
 
+	assert(((void) "len must not be < 3", len > 2));	/* <@> */
+
 	if (thisgrp && thisgrp->attribute->from && *thisgrp->attribute->from && strchr(*thisgrp->attribute->from, '@'))
-		strcpy(from_name, *thisgrp->attribute->from);
-	else
-		sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), BlankIfNull(get_full_name()), userid, BlankIfNull(fromhost));
+		snprintf(from_name, len, "%s", *thisgrp->attribute->from);
+	else {
+		if (tinrc.mail_address && *tinrc.mail_address && strchr(tinrc.mail_address, '@'))
+			snprintf(from_name, len, "%s", tinrc.mail_address);
+		else
+			snprintf(from_name, len, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), BlankIfNull(get_full_name()), userid, BlankIfNull(fromhost));
+	}
 
 #ifdef DEBUG
 	if (debug & DEBUG_MISC)
@@ -304,26 +318,24 @@ build_sender(
 {
 	const char *ptr;
 	static char sender[HEADER_LEN];
+	int n = 0;
 
 	sender[0] = '\0';
-
 	ptr = get_full_name();
-	if (*ptr) /* TODO: rfc2047 encode */
-		snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
-
-	snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "<%.*s@", LOGIN_NAME_MAX, userid);
+	if (*ptr) { /* TODO: rfc2047 encode */
+		if ((n = snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\" " : "%s "), ptr)) < 2) /* >= 1 char real name + space */
+			n = 0;
+	}
 
 #	ifdef HAVE_GETHOSTBYNAME
 	ptr = get_fqdn(get_host_name());
 #	else
 	ptr = get_host_name();
 #	endif /* HAVE_GETHOSTBYNAME */
+	if (!*ptr)
+		return NULL;
 
-		/* intentionally do not fall back to *domain_name */
-
-	if (*ptr)
-		snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "%s>", ptr);
-	else
+	if (snprintf(sender + n, sizeof(sender) - n, "<%.*s@%s>", LOGIN_NAME_MAX, userid, ptr) < 5) /* at least "<u@d>" */
 		return NULL;
 
 	return sender;
