@@ -3,7 +3,7 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2025-12-20
+ *  Updated   : 2026-02-17
  *  Notes     :
  *
  * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
@@ -56,6 +56,10 @@
 #ifndef BUGREP_H
 #	include "bugrep.h"
 #endif /* !BUGREP_H */
+#ifdef HAVE_LIBEXTTEXTCAT
+#	include <textcat.h>
+#endif /* HAVE_LIBEXTTEXTCAT */
+
 
 /*
  * local prototypes
@@ -64,6 +68,9 @@ static int read_site_config(void);
 #ifdef HAVE_COLOR
 	static void preinit_colors(void);
 #endif /* HAVE_COLOR */
+#ifdef HAVE_LIBEXTTEXTCAT
+	static void init_textcat(void);
+#endif /* HAVE_LIBEXTTEXTCAT */
 
 #ifndef NNTP_ONLY
 char active_times_file[PATH_LEN];
@@ -194,6 +201,10 @@ t_bool xref_supported = TRUE;
 #endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 
+#ifdef HAVE_LIBEXTTEXTCAT
+	void *textcat_handle;
+#endif /* HAVE_LIBEXTTEXTCAT */
+
 /* Currently active menu parameters */
 t_menu *currmenu;
 
@@ -380,6 +391,7 @@ struct t_config tinrc = {
 	-100,	/* score_kill */
 	100,	/* score_select */
 	1,		/* scroll_lines */
+	0,		/* show_art_info */
 	SHOW_FROM_NAME,	/* show_author */
 	SHOW_SIGN_BOTH,	/* show help/mail sign in level title */
 	SORT_ARTICLES_BY_DATE_ASCEND,	/* sort_article_type */
@@ -444,7 +456,6 @@ struct t_config tinrc = {
 	TRUE,	/* show_only_unread_arts */
 	FALSE,	/* show_only_unread_groups */
 	TRUE,	/* show_signatures */
-	FALSE,	/* show_art_score */
 	TRUE,	/* sigdashes */
 	TRUE,	/* signature_repost */
 #ifndef USE_CURSES
@@ -567,7 +578,7 @@ struct t_config tinrc = {
 	FALSE,	/* attrib_prompt_followupto */
 	TRUE,	/* attrib_show_only_unread_arts */
 	TRUE,	/* attrib_show_signatures */
-	FALSE,	/* attrib_show_art_score */
+	0,		/* attrib_show_art_info */
 	TRUE,	/* attrib_sigdashes */
 	TRUE,	/* attrib_signature_repost */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -644,7 +655,6 @@ struct t_capabilities nntp_caps = {
 };
 
 static char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
-static mode_t real_umask;
 
 #ifdef HAVE_COLOR
 
@@ -736,6 +746,7 @@ init_selfinfo(
 	FILE *fp;
 	const char *p;
 	char tmp[PATH_LEN];
+	mode_t real_umask;
 	size_t space;
 	const struct passwd *myentry;
 	struct stat sb;
@@ -745,6 +756,7 @@ init_selfinfo(
 
 	process_id = getpid();
 
+	/* what for? */
 	real_umask = umask(0);
 	(void) umask(real_umask);
 
@@ -1144,6 +1156,10 @@ init_selfinfo(
 #ifdef HAVE_PGP_GPG
 	init_pgp();
 #endif /* HAVE_PGP_GPG */
+
+#ifdef HAVE_LIBEXTTEXTCAT
+	init_textcat();
+#endif /* HAVE_LIBEXTTEXTCAT */
 }
 
 
@@ -1180,6 +1196,9 @@ read_site_config(
 	if (!fp)
 		return -1;
 
+#ifdef DEBUG
+	i = 0; /* line count */
+#endif /* DEBUG */
 	/*
 	 * no (_(txt_reading_(global_)config_file), global_defaults_file)
 	 * here as read_site_config() is called (from init_selfinfo()) before
@@ -1187,8 +1206,11 @@ read_site_config(
 	 * batch mode or not ...
 	 */
 	while ((buf = tin_fgets(fp, FALSE)) != NULL) {
+#ifdef DEBUG
+		++i;
+#endif /* DEBUG */
 		/* ignore comments */
-		if (*buf == '#' || *buf == ';' || *buf == ' ')
+		if (*buf == '#' || *buf == ';' || *buf == '\0')
 			continue;
 #ifndef NNTP_ONLY
 		if (match_string(buf, "spooldir=", spooldir, sizeof(spooldir)))
@@ -1233,6 +1255,13 @@ read_site_config(
 			continue;
 		if (match_boolean(buf, "disable_sender=", &disable_sender))
 			continue;
+#ifdef DEBUG
+		/*
+		 * we don't know if we're are in debug or batch mode yet.
+		 * TODO: -> lang.c and _()?
+		 */
+		my_fprintf(stderr, "%s:%d: \"%s\" unrecognized.\n", global_defaults_file, i, buf);
+#endif /* DEBUG */
 	}
 
 	fclose(fp);
@@ -1360,3 +1389,13 @@ postinit_regexp(
 	compile_regex(SHAR_END_REGEX, &shar_end_regex, REGEX_ANCHORED);
 	compile_regex(LATEX_REGEX, &latex_regex, 0);
 }
+
+
+#ifdef HAVE_LIBEXTTEXTCAT
+static void
+init_textcat(
+	void)
+{
+	textcat_handle = special_textcat_Init(TEXTCAT_CONFIG_FILE, TEXTCAT_FINGERPRINTS_PATH);
+}
+#endif /* HAVE_LIBEXTTEXTCAT */
