@@ -3,7 +3,7 @@
  *  Module    : joinpath.c
  *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-01-10
- *  Updated   : 2008-12-04
+ *  Updated   : 2026-01-21
  *  Notes     :
  *
  * Copyright (c) 1997-2026 Thomas Dickey <dickey@invisible-island.net>
@@ -55,15 +55,18 @@ joinpath(
 {
 	size_t result_len;
 
-	(void) strncpy(result, dir, result_size - 1);
-	result[result_size - 1] = '\0';
-	result_len = strlen(result);
-	if ((result_len < (result_size - 1)) && (result[0] == '\0' || result[strlen(result) - 1] != '/')) {
-		(void) strcat(result, "/");
-		result_len++;
-	}
-	if (result_len < (result_size - 1)) {
-		(void) strncat(result, BlankIfNull(file), result_size - result_len - 1);
+	*result = '\0';
+	if (result_size > 1) {
+		(void) strncpy(result, dir, result_size - 1);
 		result[result_size - 1] = '\0';
+		result_len = strlen(result);
+		if ((result_len < (result_size - 1)) && (result[0] == '\0' || result[strlen(result) - 1] != '/')) {
+			(void) strcat(result, "/");
+			result_len++;
+		}
+		if (result_len < (result_size - 1)) {
+			(void) strncat(result, BlankIfNull(file), result_size - result_len - 1);
+			result[result_size - 1] = '\0';
+		}
 	}
 }
