@@ -6,7 +6,7 @@
  *  Updated   : 2024-02-27
  *  Notes     :
  *
- * Copyright (c) 1998-2025 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1998-2026 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,7 @@
 #	include "tin.h"
 #endif /* !TIN_H */
 
-#if !defined(USE_FLOCK) && !defined(USE_LOCKF) && !defined(USE_FCNTL)
+#if !defined(NO_LOCKING) && !defined(USE_FLOCK) && !defined(USE_LOCKF) && !defined(USE_FCNTL)
 #	ifdef HAVE_FCNTL
 #		define USE_FCNTL 1
 #	else
@@ -53,7 +53,7 @@
 #			endif /* HAVE_FLOCK */
 #		endif /* HAVE_LOCKF */
 #	endif /* HAVE_FCNTL */
-#endif /* !USE_FLOCK && !USE_LOCKF && !USE_FCNTL */
+#endif /* !NO_LOCKING && !USE_FLOCK && !USE_LOCKF && !USE_FCNTL */
 
 /*
  * TODO: add support for $LOCKEXT
