@@ -3,7 +3,7 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-12-06
+ *  Updated   : 2025-01-14
  *  Notes     :
  *
  * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -561,7 +561,7 @@ main(
 
 /*
  * process command line options
- * [01235789beEijJKOyY] are unused
+ * [01235789beEijKOyY] are unused
  * [W] is reserved
  * [BPU] have been in use at some time, but now are unused:
  *   B BBS mode (M_AMIGA only)
@@ -571,7 +571,7 @@ main(
  *   C was count articles, now is activate COMPRESS DEFLATE
  *   p was set printer-cmd., now is nntp-port
  *
- * unused: [01235789bBeEijJKOPUyY]
+ * unused: [01235789bBeEijKOPUyY]
  */
 #define DO_FREE() do { \
 		FreeIfNeeded(trcv); \
@@ -588,7 +588,7 @@ main(
 #define USELESS_COMB(keep,ignore) do { \
 		wait_message(2, _(txt_useless_combination), keep, ignore, ignore, BlankIfNull(trcv)); \
 	} while (0)
-#define OPTIONS ":46aAcCdD:f:F:g:G:hHI:klL:m:M:nNop:qQrRs:St:TuvVwxXzZ"
+#define OPTIONS ":46aAcCdD:f:F:g:G:hHI:J:klL:m:M:nNop:qQrRs:St:TuvVwxXzZ"
 static char **
 read_cmd_line_options(
 	int argc,
@@ -946,6 +946,11 @@ read_cmd_line_options(
 				joinpath(index_newsdir, sizeof(index_newsdir), optarg, INDEX_NEWSDIR);
 				break;
 
+			case 'J':
+				strfpath(optarg, local_config_file, sizeof(local_config_file), NULL, FALSE);
+				/* TODO: error handling? permission check? e.g. try fopen(local_config_file, "w")? */
+				break;
+
 			case 'k':
 #ifdef NNTPS_ABLE
 				insecure_nntps = TRUE;
@@ -1528,6 +1533,7 @@ usage(
 	error_message(0, _(txt_usage_help_message));
 	error_message(0, _(txt_usage_help_information), theProgname);
 	error_message(0, _(txt_usage_index_newsdir), index_newsdir);
+	error_message(0, _(txt_usage_tinrc_file), local_config_file);
 
 #ifdef NNTP_ABLE
 #	ifdef NNTPS_ABLE
