@@ -3,10 +3,10 @@
  *  Module    : makecfg.c
  *  Author    : Thomas E. Dickey
  *  Created   : 1997-08-23
- *  Updated   : 2025-05-12
+ *  Updated   : 2025-12-05
  *  Notes     : #defines and structs for options_menu.c
  *
- * Copyright (c) 1997-2025 Thomas E. Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2026 Thomas E. Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -116,7 +116,6 @@ MYDATA {
 };
 
 static MYDATA *all_data;
-static int line_no;
 
 static _Noreturn void
 failed(
@@ -192,7 +191,6 @@ parse_tbl(
 		s++;
 	buffer = s;
 
-	line_no++;
 	if (*buffer != ';' && *buffer != '\0') {	/* ignore comments */
 		if (*buffer == '#') {
 			store_data(buffer, "");
@@ -390,15 +388,15 @@ generate_ptr(
 	int after;
 
 	switch (mode) {
-	case 0:
-		fprintf(ofp, "\n%s %s_list[] = %c\n", ptr_type, opt_type, L_CURL);
-		break;
-	case 1:
-		fprintf(ofp, "\ntypedef OTYP %c\n", L_CURL);
-		break;
-	case 2:
-		fprintf(ofp, "\n");
-		break;
+		case 0:
+			fprintf(ofp, "\n%s %s_list[] = %c\n", ptr_type, opt_type, L_CURL);
+			break;
+		case 1:
+			fprintf(ofp, "\ntypedef OTYP %c\n", L_CURL);
+			break;
+		case 2:
+			fprintf(ofp, "\n");
+			break;
 	}
 	after = FALSE;
 
@@ -421,41 +419,41 @@ generate_ptr(
 				after = TRUE;
 			}
 			switch (mode) {
-			case 0:
-				fprintf(ofp, "\t&tinrc.%s,%*s/* %2d: %s__ */\n",
-					p->name,
-					MAXNAME - (int) (1 + strlen(p->name)),
-					" ",
-					index_of(p),
-					p->name);
-				break;
-			case 1:
-				fprintf(ofp, "\tOVAL(oinx_%.*s, %s__)\n",
-					MAXTYPE, opt_type,
-					p->name);
-				break;
-			case 2:
-				fprintf(ofp, "#define OINX_%-*.*s OINX(oinx_%.*s, %s__)\n",
-					MAXNAME, MAXNAME,
-					p->name,
-					MAXTYPE, opt_type,
-					p->name);
-				break;
+				case 0:
+					fprintf(ofp, "\t&tinrc.%s,%*s/* %2d: %s__ */\n",
+						p->name,
+						MAXNAME - (int) (1 + strlen(p->name)),
+						" ",
+						index_of(p),
+						p->name);
+					break;
+				case 1:
+					fprintf(ofp, "\tOVAL(oinx_%.*s, %s__)\n",
+						MAXTYPE, opt_type,
+						p->name);
+					break;
+				case 2:
+					fprintf(ofp, "#define OINX_%-*.*s OINX(oinx_%.*s, %s__)\n",
+						MAXNAME, MAXNAME,
+						p->name,
+						MAXTYPE, opt_type,
+						p->name);
+					break;
 			}
 		}
 	}
 
 	switch (mode) {
-	case 0:
-		fprintf(ofp, "%c;\n", R_CURL);
-		break;
-	case 1:
-		fprintf(ofp, "\tOVAL(oinx_%.*s, s_MAX)\n", MAXTYPE, opt_type);
-		fprintf(ofp, "\tOEND(oinx_%.*s, Q1)\n", MAXTYPE, opt_type);
-		fprintf(ofp, "%c oinx_%.*s;\n", R_CURL, MAXTYPE, opt_type);
-		break;
-	case 2:
-		break;
+		case 0:
+			fprintf(ofp, "%c;\n", R_CURL);
+			break;
+		case 1:
+			fprintf(ofp, "\tOVAL(oinx_%.*s, s_MAX)\n", MAXTYPE, opt_type);
+			fprintf(ofp, "\tOEND(oinx_%.*s, Q1)\n", MAXTYPE, opt_type);
+			fprintf(ofp, "%c oinx_%.*s;\n", R_CURL, MAXTYPE, opt_type);
+			break;
+		case 2:
+			break;
 	}
 }
 
@@ -532,7 +530,6 @@ makecfg(
 	/*
 	 * Process the input file.
 	 */
-	line_no = 0;
 	while (fgets(buffer, sizeof(buffer) - 1, ifp))
 		parse_tbl(buffer);
 	fclose(ifp);
