@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-06-15
+ *  Updated   : 2026-02-13
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,7 +60,7 @@
  * Local prototypes
  */
 static FILE *open_subscription_fp(void);
-static char *parse_newsrc_line(char *line, int *sub);
+static char *parse_newsrc_line(const char *line, int *sub);
 static char *parse_subseq(struct t_group *group, char *seq, t_artnum *low, t_artnum *high, t_artnum *sum);
 static char *parse_get_seq(char *seq, t_artnum *low, t_artnum *high);
 static int write_newsrc_line(FILE *fp, char *line);
@@ -540,11 +540,13 @@ backup_newsrc(
 						int n;
 
 						if ((n = snprintf(NULL, 0, "overwriting %s-link \"%s\", target \"%s\" not below \"%s\"", OLDNEWSRC_FILE, filebuf, target, dirbuf)) > 0) {
-							debug_mesg = my_malloc(++n);
-							snprintf(debug_mesg, n, "overwriting %s-link \"%s\", target \"%s\" not below \"%s\"", OLDNEWSRC_FILE, filebuf, target, dirbuf);
-							debug_print_comment(debug_mesg);
-							error_message(2, "Warning: %s", debug_mesg);
-							free(debug_mesg);
+							if (n < INT_MAX) {
+								debug_mesg = my_malloc(++n);
+								snprintf(debug_mesg, n, "overwriting %s-link \"%s\", target \"%s\" not below \"%s\"", OLDNEWSRC_FILE, filebuf, target, dirbuf);
+								debug_print_comment(debug_mesg);
+								error_message(2, "Warning: %s", debug_mesg);
+								free(debug_mesg);
+							}
 						}
 					}
 #		endif /* DEBUG */
@@ -567,8 +569,6 @@ backup_newsrc(
 	if (!backup_file(newsrc, filebuf))
 		error_message(2, _(txt_filesystem_full_backup), newsrc);
 #endif /* HAVE_REALPATH && (O_NOFOLLOW || HAVE_LSTAT) */
-
-	return;
 }
 
 
@@ -1280,11 +1280,11 @@ parse_get_seq(
 	t_artnum *low,
 	t_artnum *high)
 {
-	*low = strtoartnum(seq, &seq, 10);
+	*low = strtoartnum(seq, &seq);
 
 	if (*seq == '-') {	/* Range of articles */
 		++seq;
-		*high = strtoartnum(seq, &seq, 10);
+		*high = strtoartnum(seq, &seq);
 	} else	/* Single article */
 		*high = *low;
 
@@ -1680,6 +1680,10 @@ catchup_newsrc_file(
 {
 	int i;
 	struct t_group *group;
+	time_t beg_epoch = 0;
+
+	if (verbose)
+		(void) time(&beg_epoch);
 
 	for (i = 0; i < selmenu.max; i++) {
 		group = &active[my_group[i]];
@@ -1691,6 +1695,41 @@ catchup_newsrc_file(
 		group->newsrc.num_unread = 0;
 		group->newsrc.xbitlen = 0;
 	}
+	if (verbose) {
+		const char *action_fmt = P_(txt_catchup_info_sp[0], txt_catchup_info_sp[1], i);
+		const char *elapsed_fmt;
+		char *action;
+		char *elapsed;
+		int n;
+		size_t len;
+		time_t tela = (unsigned long int) (time(NULL) - beg_epoch);
+
+		if ((n = snprintf(NULL, 0, action_fmt, i)) < 0)
+			return;
+		len = (size_t) n + 1;
+		action = my_malloc(len);
+		if (snprintf(action, len, action_fmt, i) < 0) {
+			free(action);
+			return;
+		}
+
+		elapsed_fmt = P_(txt_n_second_sp[0], txt_n_second_sp[1], tela);
+		if ((n = snprintf(NULL, 0, elapsed_fmt, tela)) < 0) {
+			free(action);
+			return;
+		}
+		len = (size_t) n + 1;
+		elapsed = my_malloc(len);
+		if (snprintf(elapsed, len, elapsed_fmt, tela) < 0) {
+			free(action);
+			free(elapsed);
+			return;
+		}
+
+		wait_message(0, _(txt_catchup_update_info), action, elapsed);
+		free(action);
+		free(elapsed);
+	}
 }
 
 
@@ -1701,7 +1740,7 @@ catchup_newsrc_file(
  */
 static char *
 parse_newsrc_line(
-	char *line,
+	const char *line,
 	int *sub)
 {
 	char *ptr, *tmp;
@@ -1711,7 +1750,7 @@ parse_newsrc_line(
 	if ((ptr = strpbrk(line, "!:")) == NULL)			/* space|SUBSCRIBED|UNSUBSCRIBED */
 		return NULL;
 
-	*sub = *ptr;						/* Save the subscription status */
+	*sub = (unsigned char) *ptr;						/* Save the subscription status */
 	tmp = ptr;							/* Keep this blank for later */
 	*(ptr++) = '\0';					/* Terminate the group name */
 
