@@ -3,7 +3,7 @@
  *  Module    : nntplib.c
  *  Author    : S. Barber & I. Lea
  *  Created   : 1991-01-12
- *  Updated   : 2025-08-29
+ *  Updated   : 2026-01-12
  *  Notes     : NNTP client routines taken from clientlib.c 1.5.11 (1991-02-10)
  *  Copyright : (c) Copyright 1991-99 by Stan Barber & Iain Lea
  *              Permission is hereby granted to copy, reproduce, redistribute
@@ -101,13 +101,13 @@ char *nntp_server = NULL;
 #ifdef NNTP_ABLE
 	static int mode_reader(t_bool *sec);
 	static int reconnect(int retry);
-	static int server_init(char *machine, const char *cservice, unsigned short port, char *text, size_t mlen);
+	static int server_init(const char *machine, const char *cservice, unsigned short port, char *text, size_t mlen);
 	static void close_server(t_bool send_no_quit);
-	static t_distrib_pat *add_distrib_pat(int weight, char *pat, char *dist, char *desc);
+	static t_distrib_pat *add_distrib_pat(int weight, const char *pat, const char *dist, const char *desc);
 	static void list_distrib_pats(void);
 	static void list_distributions(void);
-	static long set_tcp_user_ini_timeout(int sockfd);
-	static long set_tcp_user_rxt_timeout(int sockfd);
+	static long set_tcp_user_ini_timeout(int sockfd, t_bool set);
+	static long set_tcp_user_rxt_timeout(int sockfd, t_bool set);
 	static long int list_motd(FILE *stream);
 #	ifdef INET6
 		static int get_tcp6_socket(const char *machine, unsigned short port);
@@ -363,7 +363,7 @@ getserverbyfile(
 #ifdef NNTP_ABLE
 static int
 server_init(
-	char *machine,
+	const char *machine,
 	const char *cservice,	/* usually a literal */
 	unsigned short port,
 	char *text,
@@ -430,7 +430,7 @@ server_init(
 #	endif /* AF_UNSPEC */
 
 	/* set user timeout (if any) after connect()! */
-	set_tcp_user_rxt_timeout(sock_fd);
+	set_tcp_user_rxt_timeout(sock_fd, TRUE);
 
 	{
 		struct sockaddr sa;
@@ -705,7 +705,7 @@ get_tcp_socket(
 #				endif /* HAVE_GETSOCKOPT && HAVE_SETSOCKOPT */
 #			endif /* __hpux && SVR4 */
 
-		set_tcp_user_ini_timeout(s);
+		set_tcp_user_ini_timeout(s, TRUE);
 
 		if ((x = connect(s, (struct sockaddr *) &sock_in, sizeof(sock_in))) == 0)
 			break;
@@ -740,7 +740,7 @@ get_tcp_socket(
 		return -1;
 	}
 
-	set_tcp_user_ini_timeout(s);
+	set_tcp_user_ini_timeout(s, TRUE);
 
 	/* And connect */
 	if (connect(s, (struct sockaddr *) &sock_in) < 0) {
@@ -763,7 +763,7 @@ get_tcp_socket(
 	memcpy((char *) &sock_in.sin_addr, hp->h_addr, hp->h_length);
 #				endif /* HAVE_HOSTENT_H_ADDR_LIST */
 
-	set_tcp_user_ini_timeout(s);
+	set_tcp_user_ini_timeout(s, TRUE);
 
 	if (connect(s, (struct sockaddr *) &sock_in, sizeof(sock_in)) < 0) {
 		save_errno = errno;
@@ -838,7 +838,7 @@ get_tcp6_socket(
 			continue;
 		}
 
-		set_tcp_user_ini_timeout(s);
+		set_tcp_user_ini_timeout(s, TRUE);
 
 		if (connect(s, res->ai_addr, res->ai_addrlen) != 0) {
 			ec = errno;
@@ -1100,7 +1100,7 @@ reconnect(
 	}
 
 #	if defined(HAVE_ALARM) && defined(SIGALRM)
-		alarm(0);
+	alarm(0);
 #	endif /* HAVE_ALARM && SIGALRM */
 
 	return retry;
@@ -1253,6 +1253,12 @@ static void
 close_server(
 	t_bool send_no_quit)
 {
+	 /* we want to quit, don't bother to reconnect */
+#if defined(TIMEOUT_INI_NAME) && TIMEOUT_INI_NAME != TIMEOUT_RXT_NAME
+	set_tcp_user_rxt_timeout(nntp_buf.fd, FALSE);
+#endif /* TIMEOUT_INI_NAME && TIMEOUT_INI_NAME != TIMEOUT_RXT_NAME */
+	set_tcp_user_ini_timeout(nntp_buf.fd, FALSE);
+
 	if (!send_no_quit && nntpbuf_is_open(&nntp_buf)) {
 		if ((!batch_mode || verbose) && cCOLS > 1) {
 			char *msg = strunc(_(txt_disconnecting), (size_t) (cCOLS - 1));
@@ -1261,7 +1267,7 @@ close_server(
 			my_fputc('\n', stdout);
 			free(msg);
 		}
-		nntp_command("QUIT", OK_GOODBYE, NULL, 0);
+		new_nntp_command("QUIT", OK_GOODBYE, NULL, 0);
 		quitting = TRUE;										/* Don't reconnect just for this */
 	}
 	nntpbuf_close(&nntp_buf);
@@ -1292,8 +1298,8 @@ check_extensions(
 	static const char *tin_mechs[] = { /* SASL mechanisms our code can handle */
 		"PLAIN",			/* RFC 4616 */
 		"ANONYMOUS",		/* RFC 4505 */
-		"LOGIN",			/* just for testing, remove b4 release */
 #if 0
+		"LOGIN",			/* just for testing, remove b4 release */
 		"EXTERNAL",			/* RFC 4422 */
 		"OTP",				/* RFC 2444 */
 		"SECURID",			/* RFC 2808 */
@@ -1543,7 +1549,7 @@ check_extensions(
 			 * XanaNewz 2 Server Version 2.0.0.3 doesn't know CAPABILITIES
 			 * but responds with 400 _without_ closing the connection. If
 			 * you use tin on a XanaNewz 2 Server comment out the following
-			 * case.
+			 * case (or use serverrc:disabled_nntp_cmds=CAPABILITIES).
 			 */
 #	if 1
 			case ERR_GOODBYE:
@@ -1949,7 +1955,7 @@ nntp_open(
 
 		nntp_caps.over_cmd = NULL;
 		for (i = 0; i < 2 && j >= 0; i++) {
-			j = new_nntp_command(&xover_cmds[i], ERR_NCING, line, sizeof(line));
+			j = new_nntp_command(&xover_cmds[i], ERR_NCING, NULL, 0);
 			switch (j) {
 				case ERR_COMMAND:
 					break;
@@ -1977,7 +1983,7 @@ nntp_open(
 		/* TODO; delay check till needed */
 		if (nntp_caps.hdr) {
 			for (i = j = 0; i < 2 && j >= 0; i++) {
-				j = new_nntp_command(&xhdr_cmds[i], ERR_CMDSYN, line, sizeof(line));
+				j = new_nntp_command(&xhdr_cmds[i], ERR_CMDSYN, NULL, 0);
 				switch (j) {
 					case ERR_COMMAND:
 						break;
@@ -2019,7 +2025,7 @@ nntp_open(
 			/*
 			 * CAPABILITIES didn't mention OVER or XOVER, try XOVER
 			 */
-			switch (new_nntp_command(xover_cmds, ERR_NCING, line, sizeof(line))) {
+			switch (new_nntp_command(xover_cmds, ERR_NCING, NULL, 0)) {
 				case ERR_COMMAND:
 					break;
 
@@ -2048,7 +2054,7 @@ nntp_open(
 			/*
 			 * CAPABILITIES didn't mention HDR or XHDR, try XHDR
 			 */
-			switch (new_nntp_command(xhdr_cmds, ERR_NCING, line, sizeof(line))) {
+			switch (new_nntp_command(xhdr_cmds, ERR_NCING, NULL, 0)) {
 				case ERR_COMMAND:
 					break;
 
@@ -2216,10 +2222,16 @@ get_only_respcode(
 	 * what about other LIST cmds? (ACTIVE|COUNTS|OVERVIEW.FMT|...)
 	 */
 	if (*last_put && ((respcode == ERR_FAULT && (!strncmp(last_put, "ARTICLE", 7) || !strcmp(last_put, "POST") || !strcmp(last_put, "LIST"))) || respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && strcmp(last_put, "QUIT")) {
-		if (respcode == ERR_GOODBYE && !strncmp(last_put, "HEAD ", 5)) {
+		if (respcode == ERR_GOODBYE) {
+			/* don't retry to auth on ERR_GOODBYE */
+			if (!strncmp(last_put, "AUTHINFO ", 9))
+				tin_done(NNTP_ERROR_EXIT, ptr);
+
 			/* usenetfarm may send ERR_GOODBYE in response to HEAD, we don't want to retry that */
-			return respcode;
+			if (!strncmp(last_put, "HEAD ", 5))
+				return respcode;
 		}
+
 		/*
 		 * Maybe server timed out.
 		 * If so, retrying will force a reconnect.
@@ -2413,7 +2425,7 @@ DEBUG_IO((stderr, "%snew_nntp_command(%s)\n", logtime(), command));
 		if ((respcode = get_respcode(message, mlen)) != success) {
 #	ifdef DEBUG
 			if ((debug & DEBUG_NNTP) && verbose > 1)
-				debug_print_file("NNTP", "new_nntp_command(%s) NOT_OK - Expected: %d, got: %d", command, success, respcode);
+				debug_print_file("NNTP", "new_nntp_command(%s) NOT_OK - Expected: %d, got: %d %s", command, success, respcode, message);
 #	endif /* DEBUG */
 			return respcode;
 		}
@@ -2487,7 +2499,12 @@ list_distrib_pats(
 #	endif /* DEBUG */
 				if ((d = strpbrk(ptr, ":")) == NULL)
 					continue;
-				weight = strtol(ptr, NULL, 10);
+				errno = 0;
+				weight = s2i(ptr, 0, INT_MAX);
+#	ifdef DEBUG
+				if (errno && (debug & DEBUG_NNTP) && verbose > 1)
+					debug_print_file("NNTP", "Invalid weight");
+#	endif /* DEBUG */
 				pat = d + 1;
 				if ((d = strpbrk(pat, ":")) == NULL)
 					continue;
@@ -2508,9 +2525,9 @@ list_distrib_pats(
 static t_distrib_pat *
 add_distrib_pat(
 	int weight,
-	char *pat,
-	char *dist,
-	char *desc)
+	const char *pat,
+	const char *dist,
+	const char *desc)
 {
 	t_distrib_pat *p, *temp;
 
@@ -2541,16 +2558,14 @@ list_motd(
 	FILE *stream)
 {
 	char *ptr, *p, *m;
-	char buf[NNTP_STRLEN];
 	long m_hash = 0L;
 
 	if (!stream || !nntp_caps.list_motd)
 		return m_hash;
 
 	m = my_calloc(1, 1);
-	buf[0] = '\0';
 
-	switch (new_nntp_command("LIST MOTD", OK_MOTD, buf, sizeof(buf))) {
+	switch (new_nntp_command("LIST MOTD", OK_MOTD, NULL, 0)) {
 		case OK_MOTD:
 			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #	ifdef DEBUG
@@ -2692,8 +2707,6 @@ static void
 enable_deflate(
 	struct nntpbuf *nntpbuf)
 {
-	char buf[NNTP_STRLEN];
-
 	if (nntpbuf->z_rd || nntpbuf->z_wr)
 		return;
 
@@ -2705,8 +2718,7 @@ enable_deflate(
 	if (nntpbuf->z_rd == NULL || nntpbuf->z_wr == NULL)
 		goto error_out;
 
-	buf[0] = '\0';
-	switch (new_nntp_command("COMPRESS DEFLATE", OK_COMPRESS, buf, sizeof(buf))) {
+	switch (new_nntp_command("COMPRESS DEFLATE", OK_COMPRESS, NULL, 0)) {
 		case OK_COMPRESS:
 			deflate_active = TRUE;
 			return;
@@ -3137,29 +3149,29 @@ nntp_conninfo(
 #	endif /* USE_ZLIB */
 			}
 #	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
-		if (nntp_caps.maxartnum) {
-			char *buf;
-			int n;
-			size_t len;
+			if (nntp_caps.maxartnum) {
+				char *buf;
+				int n;
+				size_t len;
 
-			if ((n = snprintf(NULL, 0, "%"T_ARTNUM_PFMT, nntp_caps.maxartnum)) > 0) {
-				len = (size_t) n + 1;
-				buf = my_malloc(len);
+				if ((n = snprintf(NULL, 0, "%"T_ARTNUM_PFMT, nntp_caps.maxartnum)) > 0) {
+					len = (size_t) n + 1;
+					buf = my_malloc(len);
 
-				if (snprintf(buf, len, "%"T_ARTNUM_PFMT, nntp_caps.maxartnum) == n)
-					fprintf(stream, _(txt_conninfo_maxartnum), buf);
-				free(buf);
+					if (snprintf(buf, len, "%"T_ARTNUM_PFMT, nntp_caps.maxartnum) == n)
+						fprintf(stream, _(txt_conninfo_maxartnum), buf);
+					free(buf);
+				}
 			}
-		}
 #	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
 #	ifdef USE_GSASL
-		if (nntp_caps.authinfo_sasl) {
-			fprintf(stream, _(txt_usable_sasl_mechs), nntp_caps.sasl_mechs ? nntp_caps.sasl_mechs : _(txt_none));
-			fprintf(stream, _(txt_used_sasl_mech), nntp_caps.sasl_mech_used ? nntp_caps.sasl_mech_used : _(txt_none));
-		}
+			if (nntp_caps.authinfo_sasl) {
+				fprintf(stream, _(txt_usable_sasl_mechs), nntp_caps.sasl_mechs ? nntp_caps.sasl_mechs : _(txt_none));
+				fprintf(stream, _(txt_used_sasl_mech), nntp_caps.sasl_mech_used ? nntp_caps.sasl_mech_used : _(txt_none));
+			}
 #	endif /* USE_GSASL */
-		if (nntp_caps.implementation && *nntp_caps.implementation)
-			fprintf(stream, _(txt_conninfo_implementation), nntp_caps.implementation);
+			if (nntp_caps.implementation && *nntp_caps.implementation)
+				fprintf(stream, _(txt_conninfo_implementation), nntp_caps.implementation);
 		}
 	}
 
@@ -3200,7 +3212,6 @@ set_maxartnum(
 	t_bool reconnect)
 {
 	char cmd[NNTP_STRLEN];
-	char line[NNTP_STRLEN] = { '\0' };
 	static int cnt = 0;
 
 	if (nntp_caps.maxartnum < T_ARTNUM_CONST(2147483647) || (!reconnect && cnt))
@@ -3210,7 +3221,7 @@ set_maxartnum(
 
 	snprintf(cmd, sizeof(cmd), "MAXARTNUM %"T_ARTNUM_PFMT, nntp_caps.maxartnum);
 
-	switch (new_nntp_command(cmd, OK_EXTENSIONS, line, sizeof(line))) {
+	switch (new_nntp_command(cmd, OK_EXTENSIONS, NULL, 0)) {
 		case OK_EXTENSIONS:
 			nntp_caps.mode_reader = FALSE;
 			nntp_caps.reader = TRUE;
@@ -3228,15 +3239,16 @@ set_maxartnum(
 
 /*
  * timeout for new, non established TCP connections
- * return set timeout in ms if any or -1 on error
+ * return timeout in ms if any or -1 on error
  */
 static long
 set_tcp_user_ini_timeout(
-	int sockfd)
+	int sockfd,
+	t_bool set)
 {
 #	if defined(TIMEOUT_INI_NAME)
 	if (TIN_NNTP_TIMEOUT) {
-		TIMEOUT_TYPE optval = (TIN_NNTP_TIMEOUT + 1) * TIMEOUT_MUL;
+		TIMEOUT_TYPE optval = set ? (TIN_NNTP_TIMEOUT + 1) * TIMEOUT_MUL : 250;
 		TIMEOUT_TYPE ival;
 		socklen_t vlen = sizeof(TIMEOUT_TYPE);
 
@@ -3254,8 +3266,9 @@ set_tcp_user_ini_timeout(
 		}
 	}
 #	else
-	DEBUG_IO((stderr, "%s!TIMEOUT_INI_NAME\n",logtime()));
+	DEBUG_IO((stderr, "%s!TIMEOUT_INI_NAME\n", logtime()));
 	(void) sockfd;
+	(void) set;
 #	endif /* TIMEOUT_INI_NAME */
 	return 0L;
 }
@@ -3267,11 +3280,12 @@ set_tcp_user_ini_timeout(
  */
 static long
 set_tcp_user_rxt_timeout(
-	int sockfd)
+	int sockfd,
+	t_bool set)
 {
 #	if defined(TIMEOUT_RXT_NAME)
 	if (TIN_NNTP_TIMEOUT) {
-		TIMEOUT_TYPE optval = (250 + (TIN_NNTP_TIMEOUT < 12 ? TIN_NNTP_TIMEOUT * 500 : 6000));
+		TIMEOUT_TYPE optval = 250 + (set ? (TIN_NNTP_TIMEOUT < 12 ? TIN_NNTP_TIMEOUT * 500 : 6000) : 0);
 		TIMEOUT_TYPE ival;
 		socklen_t vlen = sizeof(TIMEOUT_TYPE);
 
@@ -3301,6 +3315,7 @@ set_tcp_user_rxt_timeout(
 #	else
 	DEBUG_IO((stderr, "%s!TIMEOUT_RXT_NAME\n",logtime()));
 	(void) sockfd;
+	(void) set;
 #	endif /* TIMEOUT_RXT_NAME */
 	return 0L;
 }
