@@ -3,13 +3,13 @@
  *  Module    : nrctbl.c
  *  Author    : Sven Paulus <sven@tin.org>
  *  Created   : 1996-10-06
- *  Updated   : 2024-12-05
+ *  Updated   : 2025-12-27
  *  Notes     : This module does the NNTP server name lookup in
  *              ~/.tin/newsrctable and returns the real hostname
  *              and the name of the newsrc file for a given
  *              alias of the server.
  *
- * Copyright (c) 1996-2025 Sven Paulus <sven@tin.org>
+ * Copyright (c) 1996-2026 Sven Paulus <sven@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,7 +74,7 @@ write_newsrctable_file(
 	if ((fp = fopen(newsrctable_file, "w")) == NULL)
 		return;
 
-	fprintf(fp, _(txt_nrctbl_info), PRODUCT, VERSION);
+	fprintf(fp, _(txt_nrctbl_info), PRODUCT);
 #ifdef HAVE_FCHMOD
 	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
 #else
@@ -97,7 +97,7 @@ void
 get_nntpserver(
 	char *nntpserver_name,
 	size_t nntpserver_name_len,
-	char *nick_name)
+	const char *nick_name)
 {
 	FILE *fp;
 	char *line_entry;
@@ -148,29 +148,30 @@ get_newsrcname(
 	if ((fp = tin_fopen(newsrctable_file, "r")) != NULL) {
 		char line[LEN];
 		char name_found[PATH_LEN] = { '\0' };
-		char *ns = NULL, *nsp = NULL, *p, *q;
+		char *ns = NULL, *nsp = NULL;
 		char *line_entry;
+		const char *p, *q;
 		int found = 0;
 		int line_entry_counter;
 		size_t l = strlen(nntpserver_name) + 8; /* []:65535\0 */
 		t_bool do_cpy = FALSE;
 
+		/* add [] and/or port if required */
 		if ((p = strchr(nntpserver_name, ':')) != NULL) {
 			if ((q = strrchr(nntpserver_name, ':')) != NULL) {
 				if (p != q) {
 					ns = my_malloc(l);
 					nsp = my_malloc(l);
 					sprintf(ns, "[%s]", nntpserver_name);
-					sprintf(nsp, "[%s]:%u", nntpserver_name, nntp_tcp_port);
+					sprintf(nsp, "[%s]:%u", nntpserver_name, (unsigned int) nntp_tcp_port);
 				}
 			}
 		}
 		if (!ns)
 			ns = my_strdup(nntpserver_name);
 		if (!nsp) {
-			/* add [] and/or port if required */
 			nsp = my_malloc(l);
-			sprintf(nsp, "%s:%u", nntpserver_name, nntp_tcp_port);
+			sprintf(nsp, "%s:%u", nntpserver_name, (unsigned int) nntp_tcp_port);
 		}
 
 		while ((fgets(line, sizeof(line), fp) != NULL) && (found != 1)) {
@@ -181,7 +182,7 @@ get_newsrcname(
 			while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
 				++line_entry_counter;
 
-				if ((line_entry_counter == 1) && ((!strcasecmp(line_entry,nsp)) || (!strcasecmp(line_entry, ns)))) {
+				if ((line_entry_counter == 1) && ((!strcasecmp(line_entry, nsp)) || (!strcasecmp(line_entry, ns)))) {
 					found = 1;
 					do_cpy = TRUE;
 				}
@@ -248,20 +249,29 @@ get_newsrcname(
 			}
 
 			if (error) {
-				char ch, default_ch = 'a';
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				wint_t ch, default_ch = 'a';
+#else
+				char default_ch = 'a';
+				int ch;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 				do {
 					/* very ugly code, but curses is not initialized yet */
 					if (error >= 2) {
 						default_ch = 'c';
-						my_printf("%s%c\b", _(txt_nrctbl_create), default_ch);
+						my_printf("%s%c\b", _(txt_nrctbl_create), (char) default_ch);
 					} else
-						my_printf("%s%c\b", _(txt_nrctbl_default), default_ch);
+						my_printf("%s%c\b", _(txt_nrctbl_default), (char) default_ch);
 
-					if ((ch = (char) ReadCh()) == '\r' || ch == '\n')
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					if (((ch = ReadWch()) == '\n') || (ch == '\r'))
+#else
+					if (((ch = (char) ReadCh()) == '\r') || (ch == '\n'))
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 						ch = default_ch;
 				} while (ch != ESC && ch != 'a' && ch != 'c' && ch != 'd' && ch != 'q');
-				my_printf("%c\n", ch);
+				my_printf("%c\n", (char) ch);
 
 				/* NOTE: these keys can not be remapped */
 				switch (ch) {
