@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-07-10
+ *  Updated   : 2026-02-13
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -116,6 +116,7 @@ static t_function url_left(void);
 static t_function url_right(void);
 static void build_url_line(int i);
 static void draw_page_header(t_bool tex2iso_conv);
+static void draw_page_footer(void);
 static void draw_percent_mark(long cur_num, long max_num);
 static void draw_url_arrow(void);
 static void free_url_list(void);
@@ -401,12 +402,11 @@ show_page(
 				if (activate_last_ctrl_l())
 					draw_page(0);
 				else {
-					if (curr_line == 0)
-						info_message(_(txt_begin_of_art));
-					else {
+					if (curr_line) {
 						curr_line -= ((tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES);
 						draw_page(0);
-					}
+					} else
+						info_message(_(txt_begin_of_art));
 				}
 				break;
 
@@ -471,14 +471,12 @@ page_goto_next_unread:
 				if (activate_last_ctrl_l())
 					draw_page(0);
 				else {
-					if (curr_line == 0) {
+					if (curr_line) {
+						i = scroll_page(KEYMAP_UP);
+						curr_line += i;
+						draw_page(i);
+					} else
 						info_message(_(txt_begin_of_art));
-						break;
-					}
-
-					i = scroll_page(KEYMAP_UP);
-					curr_line += i;
-					draw_page(i);
 				}
 				break;
 
@@ -486,14 +484,12 @@ page_goto_next_unread:
 				if (deactivate_next_ctrl_l())
 					draw_page(0);
 				else {
-					if (curr_line + ARTLINES >= artlines) {
+					if (curr_line + ARTLINES < artlines) {
+						i = scroll_page(KEYMAP_DOWN);
+						curr_line += i;
+						draw_page(i);
+					} else
 						info_message(_(txt_end_of_art));
-						break;
-					}
-
-					i = scroll_page(KEYMAP_DOWN);
-					curr_line += i;
-					draw_page(i);
 				}
 				break;
 
@@ -520,12 +516,12 @@ page_goto_next_unread:
 					const char *eyde = input_history[HIST_MESSAGE_ID][(hist_pos[HIST_MESSAGE_ID] - 1 + HIST_SIZE) % HIST_SIZE];
 					int tr = this_resp;
 
-					if (*eyde) {
+					if (eyde && *eyde) {
 						switch (show_article_by_msgid(eyde)) {
 							case LOOKUP_OK:
 								art_close(&pgart);
 								this_resp = tr;
-								if (!index_group(curr_group))
+								if (!index_group(curr_group, TRUE))
 									return GRP_RETSELECT;
 								if (!art_open(TRUE, &arts[this_resp], group, &pgart, TRUE, _(txt_reading_article))) {
 									if ((i = load_article(this_resp, group)) < 0)
@@ -550,31 +546,30 @@ page_goto_next_unread:
 				break;
 
 			case PAGE_GOTO_PARENT:		/* Goto parent of this article */
-			{
-				const struct t_msgid *parent = arts[this_resp].refptr->parent;
+				{
+					const struct t_msgid *parent = arts[this_resp].refptr->parent;
 
-				if (parent == NULL) {
-					info_message(_(txt_art_parent_none));
-					break;
+					if (parent == NULL) {
+						info_message(_(txt_art_parent_none));
+						break;
+					}
+
+					if (parent->article == ART_UNAVAILABLE) {
+						info_message(_(txt_art_parent_unavail));
+						break;
+					}
+
+					if (arts[parent->article].killed && tinrc.kill_level == KILL_NOTHREAD) {
+						info_message(_(txt_art_parent_killed));
+						break;
+					}
+
+					if ((i = load_article(parent->article, group)) < 0) {
+						XFACE_CLEAR();
+						return i;
+					}
 				}
-
-				if (parent->article == ART_UNAVAILABLE) {
-					info_message(_(txt_art_parent_unavail));
-					break;
-				}
-
-				if (arts[parent->article].killed && tinrc.kill_level == KILL_NOTHREAD) {
-					info_message(_(txt_art_parent_killed));
-					break;
-				}
-
-				if ((i = load_article(parent->article, group)) < 0) {
-					XFACE_CLEAR();
-					return i;
-				}
-
 				break;
-			}
 
 			case GLOBAL_PIPE:		/* pipe article/thread/tagged arts to command */
 				XFACE_SUPPRESS();
@@ -1053,8 +1048,9 @@ return_to_index:
 				}
 				break;
 
-			case GLOBAL_TOGGLE_INFO_LAST_LINE: /* this is _not_ correct, we do not toggle status here */
-				info_message("%s", arts[this_resp].subject);
+			case GLOBAL_TOGGLE_INFO_LAST_LINE:
+				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
+				draw_page_footer();
 				break;
 
 			case PAGE_TOGGLE_HIGHLIGHTING:
@@ -1298,62 +1294,7 @@ draw_page(
 	/*
 	 * Print an appropriate footer
 	 */
-	if (curr_line + ARTLINES >= artlines) {
-		char buf[LEN], *buf2;
-		int len;
-
-		STRCPY(buf, (arts[this_resp].thread != -1) ? _(txt_next_resp) : _(txt_last_resp));
-		buf2 = strunc(buf, cCOLS - 1);
-		len = strwidth(buf2);
-		clear_message();
-		MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
-#ifdef HAVE_COLOR
-		fcol(tinrc.col_normal);
-#endif /* HAVE_COLOR */
-		StartInverse();
-		my_fputs(buf2, stdout);
-		EndInverse();
-		my_flush();
-		free(buf2);
-	} else
-		draw_percent_mark(curr_line + ARTLINES, artlines);
-
-	if (CURR_GROUP.attribute->show_art_score && arts[this_resp].score != 0) {
-		char *buf;
-#ifdef HAVE_COLOR
-		int tmp_col = tinrc.col_invers_bg;
-#endif /* HAVE_COLOR */
-		size_t buflen = strlen(_(txt_art_score)) + 3;
-
-		buf = my_malloc(buflen + 1);
-		snprintf(buf, buflen, _(txt_art_score), tin_ltoa(arts[this_resp].score, 4));
-		MoveCursor(cLINES, 0);
-#ifdef HAVE_COLOR
-		fcol(tinrc.col_normal);
-		if (tinrc.inverse_okay) {
-			if (arts[this_resp].score < 0)
-				tinrc.col_invers_bg = tinrc.col_score_neg;
-			else
-				tinrc.col_invers_bg = tinrc.col_score_pos;
-		} else {
-			if (arts[this_resp].score < 0)
-				fcol(tinrc.col_score_neg);
-			else
-				fcol(tinrc.col_score_pos);
-		}
-#endif /* HAVE_COLOR */
-		StartInverse();
-		my_fputs(buf, stdout);
-		EndInverse();
-		my_flush();
-#ifdef HAVE_COLOR
-		if (tinrc.inverse_okay)
-			tinrc.col_invers_bg = tmp_col;
-		else
-			fcol(tinrc.col_normal);
-#endif /* HAVE_COLOR */
-		free(buf);
-	}
+	draw_page_footer();
 
 #ifdef XFACE_ABLE
 	if (tinrc.use_slrnface && !show_raw_article)
@@ -1364,6 +1305,104 @@ draw_page(
 }
 
 
+static void
+draw_page_footer(
+	void)
+{
+	char *more_buf = NULL;
+	int more_len = 0, score_len = 0, subj_len = 0;
+	long curr_artline = curr_line + ARTLINES;
+	t_bool show_score = (CURR_GROUP.attribute->show_art_info == 1 || CURR_GROUP.attribute->show_art_info == 3) && arts[this_resp].score != 0;
+
+	if (curr_artline >= artlines) {
+		char buf[LEN];
+
+		STRCPY(buf, (arts[this_resp].thread != -1) ? _(txt_next_resp) : _(txt_last_resp));
+		more_buf = strunc(buf, cCOLS - 1);
+		more_len = strwidth(more_buf);
+	} else if (tinrc.info_in_last_line) {
+		/* current format string is "%s(%d%%) [%ld/%ld]" but txt_more might contain multibyte chars */
+		more_len = snprintf(NULL, 0, "(%d%%) [%ld/%ld]", (int) (curr_artline * 100 / artlines), curr_artline, (long) artlines);
+		more_len += strwidth(_(txt_more));
+	}
+
+	if (tinrc.info_in_last_line) {
+		if (show_score)
+			score_len = strwidth(_(txt_art_score)) + 3;
+
+		info_message("%s", arts[this_resp].subject);
+
+		if ((subj_len = strwidth(arts[this_resp].subject) + 3) > cCOLS - MIN(more_len, score_len) * 2) {
+			FreeIfNeeded(more_buf);
+			return;
+		}
+	} else
+		clear_message();
+
+	if (!tinrc.info_in_last_line || subj_len < cCOLS - more_len * 2) {
+		if (curr_artline >= artlines) {
+			MoveCursor(cLINES, cCOLS - more_len - (1 + BLANK_PAGE_COLS));
+#ifdef HAVE_COLOR
+			fcol(tinrc.col_normal);
+#endif /* HAVE_COLOR */
+			StartInverse();
+			my_fputs(more_buf, stdout);
+			EndInverse();
+			my_flush();
+		} else
+			draw_percent_mark(curr_artline, artlines);
+	}
+
+	FreeIfNeeded(more_buf);
+	if (show_score && (!tinrc.info_in_last_line || subj_len < cCOLS - score_len * 2)) {
+		char *buf;
+#ifdef HAVE_COLOR
+		int tmp_bcol = tinrc.col_invers_bg, tmp_fcol = tinrc.col_invers_fg;
+#endif /* HAVE_COLOR */
+		/* txt_art_score: "Score: %s", tin_ltoa( , 4) -> strlen - 2 + 4 + 1 */
+		size_t buflen = strlen(_(txt_art_score)) + 3;
+
+		buf = my_malloc(buflen + 1);
+		snprintf(buf, buflen, _(txt_art_score), tin_ltoa(arts[this_resp].score, 4));
+#ifdef HAVE_COLOR
+		if (tinrc.inverse_okay) {
+			if (arts[this_resp].score < 0)
+				tinrc.col_invers_bg = tinrc.col_score_neg;
+			else
+				tinrc.col_invers_bg = tinrc.col_score_pos;
+
+			if (tinrc.col_invers_bg == -1)
+				tinrc.col_invers_fg = -1;
+			else {
+				if (tinrc.col_invers_bg < 6)
+					tinrc.col_invers_fg = 15;
+				else
+					tinrc.col_invers_fg = 0;
+			}
+		} else {
+			if (arts[this_resp].score < 0)
+				fcol(tinrc.col_score_neg);
+			else
+				fcol(tinrc.col_score_pos);
+		}
+#endif /* HAVE_COLOR */
+		MoveCursor(cLINES, 0);
+		StartInverse();
+		my_fputs(buf, stdout);
+		EndInverse();
+#ifdef HAVE_COLOR
+		if (tinrc.inverse_okay) {
+			tinrc.col_invers_bg = tmp_bcol;
+			tinrc.col_invers_fg = tmp_fcol;
+		} else
+			fcol(tinrc.col_normal);
+#endif /* HAVE_COLOR */
+		free(buf);
+		my_flush();
+	}
+}
+
+
 /*
  * Start external metamail program
  */
@@ -1403,8 +1442,8 @@ invoke_metamail(
 	{
 		t_bool seek_error = FALSE;
 
-		rewind(fp);
-		while (fgets(buf, (int) sizeof(buf), fp) != NULL)
+		fseek(fp, 0L, SEEK_SET);
+		while (fgets(buf, sizeof(buf), fp) != NULL)
 			fputs(buf, mime_fp);
 
 		fflush(mime_fp);
@@ -1690,7 +1729,7 @@ skip:
 			cur_pos += strwidth(tmp2);
 		}
 		free(tmp);
-		free(tmp2);
+		FreeAndNull(tmp2);
 	}
 
 	/* tex2iso */
@@ -1706,6 +1745,21 @@ skip:
 		free(tmp);
 	}
 
+	/* language */
+	if (curr_group->attribute->show_art_info > 1 && note_h->ext->language) {
+		if ((n = snprintf(buf, line_len, "%s", note_h->ext->language)) > 0) {
+			tlen = (size_t) n + 1;
+			tmp = my_malloc(tlen);
+			if (snprintf(tmp, tlen, "%s", buf) == n) {
+				tmp2 = strunc(tmp, CCOLS_THIRD - cur_pos);
+				my_fputs(tmp2, stdout);
+				cur_pos += strwidth(tmp2);
+			}
+			free(tmp);
+			FreeIfNeeded(tmp2);
+		}
+	}
+
 	/* subject */
 #	ifdef HAVE_COLOR
 	fcol(tinrc.col_subject);
@@ -1814,7 +1868,7 @@ shrug:
 
 		tmp2 = expand_tab(buf, tabwidth);
 		tmp = strunc(tmp2, cCOLS - cur_pos - 1);
-		len = cCOLS - (int) strwidth(tmp) - 1;
+		len = cCOLS - strwidth(tmp) - 1;
 		for (; cur_pos < len; cur_pos++)
 			my_fputc(' ', stdout);
 		my_fputs(tmp, stdout);
@@ -2057,7 +2111,7 @@ toggle_raw(
 			long offset;
 
 			j = 0;
-			rewind(pgart.raw);
+			fseek(pgart.raw, 0L, SEEK_SET);
 			pgart.rawl = my_malloc(sizeof(t_lineinfo) * (size_t) chunk);
 			offset = ftell(pgart.raw);
 
@@ -2390,10 +2444,9 @@ preprocess_info_message(
 	int chunk = 50;
 
 	FreeAndNull(infoline);
-	if (!info_fh)
+	if (!info_fh || fseek(info_fh, 0L, SEEK_SET) != 0)
 		return;
 
-	rewind(info_fh);
 	infoline = my_malloc(sizeof(t_lineinfo) * (size_t) chunk);
 	num_info_lines = 0;
 
@@ -2678,7 +2731,6 @@ process_url(
 	int ulen = -1;
 #endif /* HAVE_LIBURIPARSER || HAVE_LIBCURL */
 #ifdef HAVE_LIBURIPARSER
-	UriParserStateA state;
 	UriUriA uri;
 #else
 #	ifdef HAVE_LIBCURL
@@ -2689,58 +2741,59 @@ process_url(
 	lptr = find_url(n);
 	len = strlen(lptr->url) << 1; /* double size; room for editing URL */
 	url = my_malloc(len + 1);
-	if (prompt_default_string("URL:", url, (int) len, lptr->url, HIST_URL)) {
+	if (prompt_default_string("URL: ", url, (int) len, lptr->url, HIST_URL)) {
 		if (!*url) {			/* Don't try and open nothing */
 			free(url);
 			return FALSE;
 		}
 
-#ifdef HAVE_LIBURIPARSER
-		/*
-		 * Syntax-Based Normalization RFC 3986 6.2.2
-		 *
-		 * We could keep the error code, but that would be likely
-		 * always URI_ERROR_SYNTAX, so no big win.
-		 * With -DDEBUG MISC|URI we would write some details
-		 * (uri.scheme, uri.hostText, ...) to a log.
-		 * And/or make use of uri(Une|E)scape* ...
-		 */
-		state.uri = &uri;
-		if (uriParseUriA(&state, url) == URI_SUCCESS) {
-			if (uriNormalizeSyntaxA(&uri) == URI_SUCCESS) {
-				if (uriToStringCharsRequiredA(&uri, &ulen) == URI_SUCCESS) {
-					uri_norm = my_malloc(++ulen);
-					if (uriToStringA(uri_norm, &uri, ulen, NULL) == URI_SUCCESS) {
-						free(url);
-						url = uri_norm;
-					} else {
-						free(uri_norm);
-						ulen = -1;
+#if defined(HAVE_LIBURIPARSER) || defined(HAVE_LIBCURL)
+		if (!(MATCH_REGEX(news_regex, url, strlen(url)))) { /* normalize all but news */
+#	ifdef HAVE_LIBURIPARSER
+			/*
+			 * Syntax-Based Normalization RFC 3986 6.2.2
+			 *
+			 * We could keep the error code, but that would be
+			 * likely always URI_ERROR_SYNTAX, so no big win.
+			 * With -DDEBUG MISC|URI we could write some details
+			 * (uri.scheme, uri.hostText, ...) to a log.
+			 * And/or make use of uri(Une|E)scape* ...
+			 */
+			if (uriParseSingleUriA(&uri, url, NULL) == URI_SUCCESS) {
+				if (uriNormalizeSyntaxA(&uri) == URI_SUCCESS) {
+					if (uriToStringCharsRequiredA(&uri, &ulen) == URI_SUCCESS) {
+						uri_norm = my_malloc(++ulen);
+						if (uriToStringA(uri_norm, &uri, ulen, NULL) == URI_SUCCESS) {
+							free(url);
+							url = uri_norm;
+						} else {
+							free(uri_norm);
+							ulen = -1;
+						}
 					}
 				}
+				uriFreeUriMembersA(&uri);
 			}
-			uriFreeUriMembersA(&uri);
-		}
-#else
-#	ifdef HAVE_LIBCURL
-		/* same as above but with libcurl, again no detailed error logging yet */
-		if ((curl = curl_url())) {
-			if (curl_url_set(curl, CURLUPART_URL, url, CURLU_URLENCODE) == CURLUE_OK) {
-				if (curl_url_get(curl, CURLUPART_URL, &uri_norm, 0) == CURLUE_OK) {
-					free(url);
-					url = uri_norm;
-					ulen = strlen(url);
+#	else
+#		ifdef HAVE_LIBCURL
+			/* same as above but with libcurl, again no detailed error logging yet */
+			if ((curl = curl_url())) {
+				if (curl_url_set(curl, CURLUPART_URL, url, CURLU_URLENCODE) == CURLUE_OK) {
+					if (curl_url_get(curl, CURLUPART_URL, &uri_norm, 0) == CURLUE_OK) {
+						free(url);
+						url = uri_norm;
+						ulen = strlen(url);
+					}
 				}
+				curl_url_cleanup(curl);
+			}
+#		endif /* HAVE_LIBCURL */
+#	endif /* HAVE_LIBURIPARSER */
+			if (ulen < 0) { /* should only happen if the URL was modified in the prompt above */
+				error_message(2, txt_error_uri_normalization, url);
+				free(url);
+				return FALSE;
 			}
-			curl_url_cleanup(curl);
-		}
-#	endif /* HAVE_LIBCURL */
-#endif /* HAVE_LIBURIPARSER */
-#if defined(HAVE_LIBURIPARSER) || defined(HAVE_LIBCURL)
-		if (ulen < 0) { /* should only happen if the URL was modified in the prompt above */
-			error_message(2, "URI Normalization failed: %s", url); /* TODO: -> lang.c; _()? */
-			free(url);
-			return FALSE;
 		}
 #endif /* HAVE_LIBURIPARSER || HAVE_LIBCURL */
 		url_esc = escape_shell_meta(url, no_quote);
@@ -2847,7 +2900,10 @@ show_article_info_page(
 
 
 #define MIME_FLAG(what) (ptr->mime_hints.flags & what)
-/* TODO: strings to lang.c and _() ? */
+/*
+ * TODO: - strings to lang.c and _() ?
+ *       - examine headers too? (e.g. look for unencoded 8bit chars)
+ */
 void
 log_article_info(
 	t_openartinfo *artinfo)
@@ -2873,7 +2929,10 @@ log_article_info(
 		if (i == 1) {
 			fprintf(fp, "\tMIME Version:\n");
 			fprintf(fp, "\t\tArticle:\t%s", MIME_FLAG(MIME_VERSION_MISSING) ? "Missing" : MIME_FLAG(MIME_VERSION_UNSUPPORTED) ? "Unsupported" : MIME_SUPPORTED_VERSION);
-			fprintf(fp, "\n\t\tUsed:\t\t%s\n", MIME_SUPPORTED_VERSION);
+			if (MIME_FLAG(MIME_VERSION_MISSING))
+				fprintf(fp, "\n\t\tUsed:\t\tNone\n");
+			else
+				fprintf(fp, "\n\t\tUsed:\t\t%s\n", MIME_SUPPORTED_VERSION);
 		}
 
 		fprintf(fp, "\tContent Type:\n");
@@ -2950,7 +3009,7 @@ log_article_info(
 		}
 		if (ptr->language != NULL) {
 			fprintf(fp, "\n\tContent Language:\n");
-			fprintf(fp, "\t\t%s", ptr->language);
+			fprintf(fp, "\t\t%s%s", ptr->language, MIME_FLAG(MIME_LANGUAGE_GUESSED) ? " (guessed)" : "");
 		}
 		if (ptr->disposition != DISP_NONE) {
 			fprintf(fp, "\n\tContent Disposition:\n");
@@ -2964,8 +3023,10 @@ log_article_info(
 		if (filename && *filename && name && *name && strcmp(filename, name))
 			fprintf(fp, "\n!!\tFilename: %s != Name: %s", filename, name);
 #endif /* DEBUG */
+
 		fprintf(fp, "\n");
 	}
+	fflush(fp);
 }
 
 
@@ -2995,7 +3056,8 @@ draw_percent_mark(
 	}
 
 	len = strwidth(buf);
-	clear_message();
+	if (!tinrc.info_in_last_line)
+		clear_message();
 	MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
