@@ -3,7 +3,7 @@
  *  Module    : regex.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1997-02-21
- *  Updated   : 2025-07-09
+ *  Updated   : 2026-01-29
  *  Notes     : Regular expression subroutines
  *  Credits   :
  *
@@ -339,7 +339,7 @@ char *
 regex_get_substring_by_name(
 	struct regex_cache *re,		/* precompiled regex */
 	const char *sname,			/* name of subpattern */
-	char *subject)				/* data to match against */
+	const char *subject)				/* data to match against */
 {
 	char *ms;
 	int snum;
