@@ -3,10 +3,10 @@
  *  Module    : rfc1524.c
  *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-05-15
- *  Updated   : 2025-01-30
+ *  Updated   : 2025-12-27
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2025 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2026 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -393,7 +393,7 @@ expand_mailcap_meta(
 		if (*ptr == '%') {
 			++ptr;
 			if (*ptr == '{') {	/* Content-Type parameter */
-				char *end;
+				const char *end;
 
 				if ((end = strchr(ptr, '}')) != NULL) {
 					if (part->params != NULL) {
