@@ -3,7 +3,7 @@
  *  Module    : rfc2046.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-02-18
- *  Updated   : 2025-12-23
+ *  Updated   : 2026-01-20
  *  Notes     : RFC 2046 MIME article parsing
  *
  * Copyright (c) 2000-2026 Jason Faultless <jason@altarstone.com>
@@ -41,7 +41,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-
+#ifdef HAVE_LIBEXTTEXTCAT
+#	include <textcat.h>
+#endif /* HAVE_LIBEXTTEXTCAT */
 
 /*
  * local prototypes
@@ -77,20 +79,6 @@ static t_bool subtype_syntactically_valid(char *subtype);
  */
 static int art_lines = 0;		/* lines in art on spool */
 static const char *progress_mesg = NULL;	/* message progress() should display */
-/* RFC 2231 decoding table */
-static const char xtbl[] = {
-/*        0  1  2  3    4  5  6  7    8  9  a  b    c  d  e  f */
-/* 0 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 1 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 2 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 3 */   0, 1, 2, 3,   4, 5, 6, 7,   8, 9,-1,-1,  -1,-1,-1,-1,
-/* 4 */  -1,10,11,12,  13,14,15,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 5 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 6 */  -1,10,11,12,  13,14,15,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
-/* 7 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1
-};
-
-#define XVAL(c) (xtbl[(unsigned int) (c)])
 #define PARAM_SEP	"; \n"
 /* default parameters for Content-Type; at least before RFC 6657 */
 #define CT_DEFPARMS	"charset=US-ASCII"
@@ -394,11 +382,24 @@ get_charset(
  * RFC 2231: Decode parameter value according to the given
  *           character set
  */
+#define XVAL(c) (xtbl[(unsigned int) (c)])
 static void
 decode_value(
 	const char *charset,
 	t_param *part)
 {
+	/* RFC 2231 decoding table */
+	static const char xtbl[] = {
+		/*        0  1  2  3    4  5  6  7    8  9  a  b    c  d  e  f */
+		/* 0 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 1 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 2 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 3 */   0, 1, 2, 3,   4, 5, 6, 7,   8, 9,-1,-1,  -1,-1,-1,-1,
+		/* 4 */  -1,10,11,12,  13,14,15,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 5 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 6 */  -1,10,11,12,  13,14,15,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,
+		/* 7 */  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1,  -1,-1,-1,-1
+	};
 	char *rptr, *wptr;
 	const char *cset;
 	size_t max_line_len = strlen(part->value);
@@ -879,6 +880,7 @@ parse_content_type(
 #endif /* CHARSET_CONVERSION */
 			{
 				content->mime_hints.flags &= ~MIME_CHARSET_MISSING;
+				FreeIfNeeded(content->mime_hints.charset);
 				content->mime_hints.charset = my_strdup(charset);
 			}
 		}
@@ -1496,9 +1498,33 @@ parse_rfc822_headers(
 		}
 		if ((ptr = parse_header(line, "Content-Language", TRUE, FALSE, FALSE))) {
 			remove_cwsp(ptr);
-			FreeIfNeeded(hdr->ext->language);
+			FreeAndNull(hdr->ext->language);
 			/* TODO: add RFC 5646 check */
+#if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H) && defined(HAVE_LIBEXTTEXTCAT)
+			{
+				char *cl, *des;
+				const char *pfx;
+
+				cl = strtok(ptr, ",");
+				while (cl != NULL) {
+					if ((des = tag2lang(cl))) {
+						pfx = hdr->ext->language ? ", " : "";
+						hdr->ext->language = append_to_string(hdr->ext->language, pfx);
+						hdr->ext->language = append_to_string(hdr->ext->language, des);
+						if (strpbrk(cl, "-_")) {
+							hdr->ext->language = append_to_string(hdr->ext->language, " (");
+							hdr->ext->language = append_to_string(hdr->ext->language, cl);
+							hdr->ext->language = append_to_string(hdr->ext->language, ")");
+						}
+						free(des);
+					}
+					cl = strtok(NULL, ",");
+				}
+			}
+#else
 			hdr->ext->language = my_strdup(ptr);
+#endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H && HAVE_LIBEXTTEXTCAT */
+			hdr->ext->mime_hints.flags &= ~MIME_LANGUAGE_MISSING;
 			continue;
 		}
 	}
@@ -1581,6 +1607,9 @@ parse_multipart_article(
 	unsigned long bytes;
 	t_bool is_rfc822 = FALSE;
 	t_part *curr_part = NULL, *rfc822_part = NULL;
+#ifdef HAVE_LIBEXTTEXTCAT
+	char *buffer = NULL;
+#endif /* HAVE_LIBEXTTEXTCAT */
 
 	while ((line = tin_fgets(infile, (state == M_HDR))) != NULL) {
 /* fprintf(stderr, "%d---:%s\n", depth, line); */
@@ -1635,6 +1664,9 @@ parse_multipart_article(
 				if (boundary_cmp(line, bd) == BOUND_END)
 					depth = 0;
 			}
+#ifdef HAVE_LIBEXTTEXTCAT
+			FreeAndNull(buffer);
+#endif /* HAVE_LIBEXTTEXTCAT */
 #if 0 /* doesn't count tailing lines after envelop mime part - correct but confusing */
 			if (read_news_via_nntp && depth == 0)
 				while ((line = tin_fgets(infile, FALSE)) != NULL)
@@ -1698,11 +1730,15 @@ parse_multipart_article(
 							rfc822_part->bytes += curr_part->bytes - old_bytes;
 						}
 					} else if (curr_part->type == TYPE_MESSAGE && !strcasecmp("RFC822", curr_part->subtype)) {
+						const char *l = curr_part->language;
+
 						is_rfc822 = TRUE;
 						rfc822_part = curr_part;
 						state = M_HDR;
 						curr_part = new_part(part);
 						curr_part->depth = ++depth;
+						if (l && *l) /* inherit language */
+							curr_part->language = my_strdup(l);
 					}
 					break;
 				}
@@ -1734,9 +1770,33 @@ parse_multipart_article(
 				}
 				if ((ptr = parse_header(line, "Content-Language", TRUE, FALSE, FALSE))) {
 					remove_cwsp(ptr);
-					FreeIfNeeded(curr_part->language);
+					FreeAndNull(curr_part->language);
 					/* TODO: add RFC 5646 check */
+#if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H) && defined(HAVE_LIBEXTTEXTCAT)
+					{
+						char *cl, *des;
+						const char *pfx;
+
+						cl = strtok(ptr, ",");
+						while (cl != NULL) {
+							if ((des = tag2lang(cl))) {
+								pfx = curr_part->language ? ", " : "";
+								curr_part->language = append_to_string(curr_part->language, pfx);
+								curr_part->language = append_to_string(curr_part->language, des);
+								if (strpbrk(cl, "-_")) {
+									curr_part->language = append_to_string(curr_part->language, " (");
+									curr_part->language = append_to_string(curr_part->language, cl);
+									curr_part->language = append_to_string(curr_part->language, ")");
+								}
+								free(des);
+							}
+							cl = strtok(NULL, ",");
+						}
+					}
+#else
 					curr_part->language = my_strdup(ptr);
+#endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H && HAVE_LIBEXTTEXTCAT */
+					curr_part->mime_hints.flags &= ~MIME_LANGUAGE_MISSING;
 					break;
 				}
 				break;
@@ -1745,6 +1805,10 @@ parse_multipart_article(
 				switch (bnd) {
 					case BOUND_NONE:
 /* fprintf(stderr, "BOD:%s\n", line); */
+#ifdef HAVE_LIBEXTTEXTCAT
+						if (!buffer || curr_part->bytes < 768) /* ~12 lines (half screen) of ~60 bytes lines */
+							buffer = append_to_string(buffer, line);
+#endif /* HAVE_LIBEXTTEXTCAT */
 						curr_part->line_count++;
 						curr_part->bytes += bytes;
 						break;
@@ -1763,6 +1827,55 @@ parse_multipart_article(
 							FreeIfNeeded(curr_part->subtype);
 							curr_part->subtype = my_strdup("octet-stream");
 						}
+
+#ifdef HAVE_LIBEXTTEXTCAT
+						/*
+						 * guess language
+						 * for TYPE_TEXT && !ENCODING_BASE64
+						 */
+						if (curr_part && curr_part->type == TYPE_TEXT && curr_part->encoding != ENCODING_BASE64 && textcat_handle && buffer && *buffer) {
+							const char *pfx;
+
+							if (!curr_part->language) {
+								char *cp;
+								char *lng = textcat_Classify(textcat_handle, buffer, strlen(buffer));
+
+								if (*lng++ == '[') { /* ignore UNKNOWN or SHORT */
+									if ((cp = strchr(lng, '-'))) /* cut off charset */
+										*cp = '\0';
+#	if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H)
+									if ((cp = tag2lang(lng)))
+#	else
+									cp = lng;
+#	endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H */
+									{
+										pfx = "~";
+										/* current part */
+										curr_part->language = append_to_string(curr_part->language, pfx);
+										curr_part->language = append_to_string(curr_part->language, cp);
+										if (artinfo->hdr.ext != curr_part) {
+											if (artinfo->hdr.ext->language && *artinfo->hdr.ext->language)
+												pfx = ", ~";
+											artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, pfx);
+											artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, cp);
+										}
+										curr_part->mime_hints.flags &= ~MIME_LANGUAGE_MISSING;
+										curr_part->mime_hints.flags |= MIME_LANGUAGE_GUESSED;
+									}
+#	if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H)
+									FreeIfNeeded(cp);
+#	endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H */
+								}
+							} else {
+								if (artinfo->hdr.ext != curr_part) {
+									pfx = artinfo->hdr.ext->language ? ", " : "";
+									artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, pfx);
+									artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, curr_part->language);
+								}
+							}
+						}
+						FreeAndNull(buffer);
+#endif /* HAVE_LIBEXTTEXTCAT */
 						state = M_HDR;
 						curr_part = new_part(part);
 						curr_part->depth = depth;
@@ -1776,6 +1889,9 @@ parse_multipart_article(
 				break;
 		} /* switch (state) */
 	} /* while() */
+#ifdef HAVE_LIBEXTTEXTCAT
+	FreeIfNeeded(buffer);
+#endif /* HAVE_LIBEXTTEXTCAT */
 
 	/*
 	 * We only reach this point when we (unexpectedly) reach the end of the
@@ -1796,9 +1912,11 @@ parse_normal_article(
 {
 	char *line;
 #if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
-	char *buffer = NULL;
 	char *guessed_charset = NULL;
 #endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+#if defined(HAVE_LIBEXTTEXTCAT) || (defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET))
+	char *buffer = NULL;
+#endif /* HAVE_LIBEXTTEXTCAT || (CHARSET_CONVERSION && USE_ICU_UCSDET) */
 
 	while ((line = tin_fgets(in, FALSE)) != NULL) {
 		if (read_news_via_nntp) {
@@ -1809,16 +1927,18 @@ parse_normal_article(
 #endif /* DEBUG && NNTP_ABLE */
 		}
 
-#if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
-		if ((artinfo->hdr.ext->mime_hints.flags & MIME_CHARSET_MISSING) && artinfo->hdr.ext->type == TYPE_TEXT && artinfo->hdr.ext->encoding != ENCODING_QP && artinfo->hdr.ext->encoding != ENCODING_BASE64 && curr_group->attribute->undeclared_cs_guess && !(curr_group->attribute->undeclared_charset && *curr_group->attribute->undeclared_charset))
+#if defined(HAVE_LIBEXTTEXTCAT) || (defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET))
+		/* TODO: replace magic number (max. bytes to inspect) */
+		if (!buffer || strlen(buffer) < 768) /* ~12 lines (half screen) of ~60 bytes lines */
 			buffer = append_to_string(buffer, line);
-#endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+#endif /* HAVE_LIBEXTTEXTCAT || (CHARSET_CONVERSION && USE_ICU_UCSDET) */
 
 		++artinfo->hdr.ext->line_count;
 
 		if (show_progress_meter)
 			progress(artinfo->hdr.ext->line_count);
 	}
+
 #if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
 	if ((artinfo->hdr.ext->mime_hints.flags & MIME_CHARSET_MISSING) && artinfo->hdr.ext->type == TYPE_TEXT && artinfo->hdr.ext->encoding != ENCODING_QP && artinfo->hdr.ext->encoding != ENCODING_BASE64 && curr_group->attribute->undeclared_cs_guess && buffer && !(curr_group->attribute->undeclared_charset && *curr_group->attribute->undeclared_charset)) {
 		unsigned char *cp;
@@ -1832,7 +1952,6 @@ parse_normal_article(
 		}
 		if (need_guess)
 			guessed_charset = guess_charset(buffer, 10); /* is 10 suitable? */
-		FreeIfNeeded(buffer);
 		if (guessed_charset) {
 			char *guessed_cs_hdr = my_malloc(strlen(guessed_charset) + 17); /* 17=len('guessed_charset=\0') */
 
@@ -1846,6 +1965,43 @@ parse_normal_article(
 		}
 	}
 #endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+
+#ifdef HAVE_LIBEXTTEXTCAT
+	/*
+	 * guess language via libexttextcat
+	 * for TYPE_TEXT && !ENCODING_BASE64
+	 * (ICUs ucsdet_getLanguage() doesn't work with UTF-8 encoded
+	 * data)
+	 */
+	if (artinfo->hdr.ext->type == TYPE_TEXT && artinfo->hdr.ext->encoding != ENCODING_BASE64  && textcat_handle && buffer && *buffer && !artinfo->hdr.ext->language) {
+		char *cp;
+		char *lng = textcat_Classify(textcat_handle, buffer, strlen(buffer));
+
+		if (*lng++ == '[') { /* ignore UNKNOWN or SHORT */
+			if ((cp = strchr(lng, '-'))) /* cut off charset */
+				*cp = '\0';
+#	if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H)
+			if ((cp = tag2lang(lng)))
+#	else
+			cp = lng;
+#	endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H */
+			{
+				artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, "~");
+				artinfo->hdr.ext->language = append_to_string(artinfo->hdr.ext->language, cp);
+				artinfo->hdr.ext->mime_hints.flags &= ~MIME_LANGUAGE_MISSING;
+				artinfo->hdr.ext->mime_hints.flags |= MIME_LANGUAGE_GUESSED;
+			}
+#	if defined(HAVE_LIBICUUC) && defined(HAVE_UNICODE_UCOL_H)
+			FreeIfNeeded(cp);
+#	endif /* HAVE_LIBICUUC && HAVE_UNICODE_UCOL_H */
+		}
+	}
+#endif /* HAVE_LIBEXTTEXTCAT */
+
+#if defined(HAVE_LIBEXTTEXTCAT) || (defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET))
+	FreeIfNeeded(buffer);
+#endif /* HAVE_LIBEXTTEXTCAT || (CHARSET_CONVERSION && USE_ICU_UCSDET) */
+
 	return tin_errno;
 }
 
