@@ -3,10 +3,10 @@
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
  *  Created   : 1995-09-01
- *  Updated   : 2025-07-04
+ *  Updated   : 2026-01-31
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2025 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2026 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -93,7 +93,7 @@ enum {
  */
 static FILE *compose_message_rfc822(FILE *articlefp, t_bool *is_8bit);
 static FILE *compose_multipart_mixed(FILE *textfp, FILE *articlefp);
-static int do_b_encode(char *w, char *b, size_t max_ewsize, t_bool isstruct_head);
+static int do_b_encode(const char *w, char *b, size_t max_ewsize, t_bool isstruct_head);
 static int sizeofnextword(char *w);
 static int which_encoding(char *w);
 static t_bool contains_8bit_characters(FILE *fp);
@@ -103,7 +103,7 @@ static t_bool rfc1522_do_encode(char *what, char **where, const char *charset, t
 static t_bool split_mail(const char *filename, FILE **headerfp, FILE **textfp);
 static unsigned hex2bin(int x);
 static void build_base64_rank_table(void);
-static void do_rfc15211522_encode(FILE *f, constext *mime_encoding, struct t_group *group, t_bool allow_8bit_header, t_bool ismail, t_bool contains_headers);
+static void do_rfc15211522_encode(FILE *f, constext *mime_encoding, const struct t_group *group, t_bool allow_8bit_header, t_bool ismail, t_bool contains_headers);
 static void generate_mime_boundary(char *boundary, FILE *f, FILE *g);
 static void generate_random_mime_boundary(char *boundary, size_t len);
 static void str2b64(const char *from, char *to);
@@ -272,18 +272,13 @@ rfc1522_decode(
 #else
 	if (CURR_GROUP.attribute != NULL) {
 #	ifdef USE_ICU_UCSDET
-	if (CURR_GROUP.attribute->undeclared_cs_guess && !(CURR_GROUP.attribute->undeclared_charset && *CURR_GROUP.attribute->undeclared_charset)) {
-		char *guessed_charset;
-
-		if ((guessed_charset = guess_charset(c, 10)) != NULL) {
-			process_charsets(&c, &max_len, guessed_charset, tinrc.mm_local_charset, FALSE);
-			free(guessed_charset);
-		}
-	} else
+		if (CURR_GROUP.attribute->undeclared_cs_guess && !(CURR_GROUP.attribute->undeclared_charset && *CURR_GROUP.attribute->undeclared_charset))
+			csguess_convert_str(&c, "US-ASCII");
+		else
 #	endif /* USE_ICU_UCSDET */
-		process_charsets(&c, &max_len, (CURR_GROUP.attribute->undeclared_charset && *CURR_GROUP.attribute->undeclared_charset) ? (*CURR_GROUP.attribute->undeclared_charset) : "US-ASCII", tinrc.mm_local_charset, FALSE);
+			csguess_convert_str(&c, (CURR_GROUP.attribute->undeclared_charset && *CURR_GROUP.attribute->undeclared_charset) ? *CURR_GROUP.attribute->undeclared_charset : "US-ASCII");
 	} else
-		process_charsets(&c, &max_len, "US-ASCII", tinrc.mm_local_charset, FALSE);
+		csguess_convert_str(&c, "US-ASCII");
 #endif /* !CHARSET_CONVERSION */
 	sc = c;
 
@@ -345,7 +340,7 @@ rfc1522_decode(
 							int chars_to_copy;
 
 							t[i] = '\0';
-							max_len = (size_t) (i);
+							max_len = (size_t) i;
 							tmpbuf = my_strndup(t, max_len);
 							process_charsets(&tmpbuf, &max_len, charset, tinrc.mm_local_charset, FALSE);
 							chars_to_copy = (int) strlen(tmpbuf);
@@ -411,7 +406,7 @@ b642str(
 	char *out)
 {
 	char *t;
-	int bits = 0;
+	int bits;
 	unsigned pattern;
 	unsigned char x;
 
@@ -447,7 +442,7 @@ b642str(
 
 static int
 do_b_encode(
-	char *w,
+	const char *w,
 	char *b,
 	size_t max_ewsize,
 	t_bool isstruct_head)
@@ -981,7 +976,7 @@ static void
 do_rfc15211522_encode(
 	FILE *f,
 	constext *mime_encoding,
-	struct t_group *group,
+	const struct t_group *group,
 	t_bool allow_8bit_header,
 	t_bool ismail,
 	t_bool contains_headers)
@@ -1022,7 +1017,7 @@ do_rfc15211522_encode(
 		 * TODO: - what about 8bit chars in the mentioned headers
 		 *         when !allow_8bit_header?
 		 */
-		if (allow_8bit_header || (!strncasecmp(header, "References: ", 12) || !strncasecmp(header, "Message-ID: ", 12) || !strncasecmp(header, "Date: ", 6) || !strncasecmp(header, "Newsgroups: ", 12) || !strncasecmp(header, "Distribution: ", 14) || !strncasecmp(header, "Followup-To: ", 13) || !strncasecmp(header, "X-Face: ", 8) || !strncasecmp(header, "Cancel-Lock: ", 13) || !strncasecmp(header, "Cancel-Key: ", 12) || !strncasecmp(header, "Supersedes: ", 12) || !strncasecmp(header, "Path: ", 6)))
+		if (allow_8bit_header || (!strncasecmp(header, "Date: ", 6) || !strncasecmp(header, "Path: ", 6) || !strncasecmp(header, "Xref: ", 6) || !strncasecmp(header, "X-Face: ", 8) || !strncasecmp(header, "Newsgroups: ", 12) || !strncasecmp(header, "References: ", 12) || !strncasecmp(header, "Message-ID: ", 12) || !strncasecmp(header, "Supersedes: ", 12) || !strncasecmp(header, "Cancel-Key: ", 12) || !strncasecmp(header, "Cancel-Lock: ", 13) || !strncasecmp(header, "Followup-To: ", 13) || !strncasecmp(header, "Distribution: ", 14)))
 			fputs(header, g);
 		else {
 #ifdef CHARSET_CONVERSION
@@ -1060,8 +1055,12 @@ do_rfc15211522_encode(
 		free(buf);
 	}
 
-	rewind(g);
-	rewind(f);
+	if (fseek(g, 0L, SEEK_SET) != 0) {
+		fclose(g);
+		return;
+	}
+	
+	fseek(f, 0L, SEEK_SET);
 #ifdef HAVE_FTRUNCATE
 	errno = 0;
 	i = fileno(f);
@@ -1150,7 +1149,7 @@ void
 rfc15211522_encode(
 	const char *filename,
 	constext *mime_encoding,
-	struct t_group *group,
+	const struct t_group *group,
 	t_bool allow_8bit_header,
 	t_bool ismail)
 {
@@ -1287,7 +1286,7 @@ split_mail(
 void
 compose_mail_text_plain(
 	const char *filename,
-	struct t_group *group)
+	const struct t_group *group)
 {
 	rfc15211522_encode(filename, txt_mime_encodings[(group ? group->attribute->mail_mime_encoding : tinrc.mail_mime_encoding)], group, (group ? group->attribute->mail_8bit_header : tinrc.mail_8bit_header), TRUE);
 }
@@ -1306,7 +1305,7 @@ compose_mail_mime_forwarded(
 	const char *filename,
 	FILE *articlefp,
 	t_bool include_text,
-	struct t_group *group)
+	const struct t_group *group)
 {
 	FILE *fp;
 	FILE *headerfp;
@@ -1318,15 +1317,14 @@ compose_mail_mime_forwarded(
 	t_bool _8bit;
 
 	/* Split mail into headers and text */
-	if (!split_mail(filename, &headerfp, include_text ? &textfp : NULL))
+	if (!split_mail(filename, &headerfp, include_text ? &textfp : NULL) || fseek(headerfp, 0L, SEEK_SET) != 0)
 		return;
 
 	/* Encode header and text */
-	rewind(headerfp);
 	do_rfc15211522_encode(headerfp, encoding, group, allow_8bit_header, TRUE, TRUE);
 
 	if (textfp) {
-		rewind(textfp);
+		fseek(textfp, 0L, SEEK_SET);
 		do_rfc15211522_encode(textfp, encoding, group, allow_8bit_header, TRUE, FALSE);
 		entityfp = compose_multipart_mixed(textfp, articlefp);	/* Compose top-level MIME entity */
 		fclose(textfp);
@@ -1345,14 +1343,14 @@ compose_mail_mime_forwarded(
 	}
 
 	/* Put it all together */
-	rewind(headerfp);
+	fseek(headerfp, 0L, SEEK_SET);
 	while ((line = tin_fgets(headerfp, TRUE))) {
 		if (*line != '\0')
 			fprintf(fp, "%s\n", line);
 	}
 	fclose(headerfp);
 	fprintf(fp, txt_mime_version, MIME_SUPPORTED_VERSION);
-	rewind(entityfp);
+	fseek(entityfp, 0L, SEEK_SET);
 	copy_fp(entityfp, fp);
 
 	/* Clean up */
@@ -1383,7 +1381,7 @@ compose_message_rfc822(
 	fputc('\n', fp);
 
 	/* Body: articlefp */
-	rewind(articlefp);
+	fseek(articlefp, 0L, SEEK_SET);
 	copy_fp(articlefp, fp);
 
 	return fp;
@@ -1434,13 +1432,13 @@ compose_multipart_mixed(
 	 */
 	/* text */
 	fprintf(fp, txt_mime_boundary, boundary);
-	rewind(textfp);
+	fseek(textfp, 0L, SEEK_SET);
 	copy_fp(textfp, fp);
 	fputc('\n', fp);
 
 	/* message/rfc822 part */
 	fprintf(fp, txt_mime_boundary, boundary);
-	rewind(messagefp);
+	fseek(messagefp, 0L, SEEK_SET);
 	copy_fp(messagefp, fp);
 	fclose(messagefp);
 	fputc('\n', fp);
@@ -1462,11 +1460,12 @@ contains_8bit_characters(
 {
 	char *line;
 
-	rewind(fp);
-	while ((line = tin_fgets(fp, FALSE))) {
-		for (; *line != '\0'; line++) {
-			if (is_EIGHT_BIT(line))
-				return TRUE;
+	if (fseek(fp, 0L, SEEK_SET) == 0) {
+		while ((line = tin_fgets(fp, FALSE))) {
+			for (; *line != '\0'; line++) {
+				if (is_EIGHT_BIT(line))
+					return TRUE;
+			}
 		}
 	}
 
@@ -1484,10 +1483,11 @@ contains_string(
 {
 	char *line;
 
-	rewind(fp);
-	while ((line = tin_fgets(fp, FALSE))) {
-		if (strstr(line, str))
-			return TRUE;
+	if (fseek(fp, 0L, SEEK_SET) == 0) {
+		while ((line = tin_fgets(fp, FALSE))) {
+			if (strstr(line, str))
+				return TRUE;
+		}
 	}
 
 	return FALSE;
