@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-01-26
+ *  Updated   : 2026-01-21
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -148,7 +148,6 @@ check_start_save_any_news(
 	int i, j, k;
 	int art_count, hot_count;
 	int saved_arts = 0;					/* Total # saved arts */
-	struct t_article *art;
 	struct t_group *group;
 	t_bool unread_news = FALSE;
 	time_t epoch;
@@ -241,12 +240,29 @@ check_start_save_any_news(
 			}
 		}
 
-		if (!index_group(group)) {
+		if (!index_group(group, TRUE)) {
+			/*
+			 * can't see how this ever could leak, so why bother?
+			 * (next index_group() or tin_done() will take care
+			 * of the free()s and nothing else should modify arts[])
+			 * and if, why not also clear xref and mailbox.next?
+			 * (use free_art_array() then)
+			 */
+#if 0
+			struct t_article *art;
+
 			for_each_art(j) {
 				art = &arts[j];
 				FreeAndNull(art->refs);
 				FreeAndNull(art->msgid);
+				/* FreeAndNull(art->xref); */
+				/* free_mailbox_list(art->mailbox.next); */
 			}
+			/*
+			 * the obove is more or less free_art_array()
+			 * why not also do hash_reclaim() here?
+			 */
+#endif /* 0 */
 			continue;
 		}
 
@@ -539,7 +555,7 @@ save_and_process_art(
 		if (artinfo->hdr.from)
 			strip_name(artinfo->hdr.from, from);
 		else /* shouldn't show up */
-			snprintf(from, sizeof(from), "%s@%s", PATHMASTER, get_host_name());
+			snprintf(from, sizeof(from), "%s@%s", PATHMASTER, BlankIfNull(get_host_name()));
 
 		(void) time(&epoch);
 		fprintf(fp, "From %s %s\n", from, BlankIfNull(str_trim(ctime(&epoch))));
@@ -675,6 +691,7 @@ generate_savepath(
 		char buf[2048];
 		char extension[NAME_LEN + 1];
 
+		buf[0] = '\0';
 		lookup_extension(extension, sizeof(extension), content_types[part->type], part->subtype);
 		generate_filename(buf, sizeof(buf), extension);
 		mbox = expand_save_filename(savepath, PATH_LEN, buf);
@@ -721,6 +738,7 @@ expand_save_filename(
 	 * Make sure that externally supplied filename is a filename only and fits
 	 * into buffer
 	 */
+	base_filename[0] = '\0';
 	STRCPY(buf_path, path);
 	base_name(buf_path, base_filename);
 
@@ -923,7 +941,7 @@ post_process_uud(
 		if ((fp_in = tin_fopen(save[i].path, "r")) == NULL)
 			continue;
 
-		while (fgets(s, (int) sizeof(s), fp_in) != NULL) {
+		while (fgets(s, sizeof(s), fp_in) != NULL) {
 			switch (state) {
 				case INITIAL:
 					if (STRNCMPEQ("begin ", s, 6)) {
@@ -1052,7 +1070,7 @@ view_file(
 	const char *path,
 	const char *file)
 {
-	char *ext;
+	const char *ext;
 	t_part *part;
 
 	part = new_part(NULL);
@@ -1141,9 +1159,9 @@ post_process_sh(
 
 		wait_message(0, _(txt_extracting_shar), save[i].path);
 
-		while (fgets(buf, (int) sizeof(buf), fp_in) != NULL) {
+		while (fgets(buf, sizeof(buf), fp_in) != NULL) {
 			/* find #!/bin/sh style patterns */
-			if ((fp_out == NULL) && match_regex_ex(buf, (REGEX_SIZE) strlen(buf), 0, 0, &shar_regex) >= 0)
+			if ((fp_out == NULL) && MATCH_REGEX(shar_regex, buf, strlen(buf)))
 				fp_out = fopen(file_out, "w");
 
 			/* write to temp file */
@@ -1519,7 +1537,7 @@ show_attachment_page(
 	char *attach_line;
 	char *buf;
 	const char *charset;
-	int i, tmp_len, max_depth;
+	int i, tmp_len, max_depth, prev_mark_offset = mark_offset;
 	size_t blen;
 	t_part *part;
 
@@ -1543,7 +1561,7 @@ show_attachment_page(
 	for (i = 0; i < attmenu.max; ++i) {
 		part = get_part(i);
 		charset = get_param(part->params, "charset");
-		attach_line = build_attach_line(part, 0, cCOLS - 2, 0, NULL, charset);
+		attach_line = build_attach_line(part, 0, cCOLS - 2, SECTION_DEFAULT, NULL, charset);
 		snprintf(buf, blen, "  %s", attach_line);
 		FreeIfNeeded(attach_line);
 		tmp_len = strwidth(buf);
@@ -1574,6 +1592,7 @@ show_attachment_page(
 	}
 
 	draw_attachment_arrow();
+	mark_offset = prev_mark_offset;
 }
 
 
@@ -1627,6 +1646,11 @@ attachment_page(
 				show_attachment_page();
 				break;
 
+			case GLOBAL_CONNECTION_INFO:
+				show_connection_page();
+				show_attachment_page();
+				break;
+
 			case GLOBAL_BUGREPORT:
 				bug_report();
 				show_attachment_page();
@@ -1676,7 +1700,8 @@ attachment_page(
 
 			case GLOBAL_TOGGLE_INFO_LAST_LINE:
 				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
-				show_attachment_page();
+				clear_message();
+				draw_attachment_arrow();
 				break;
 
 			case GLOBAL_VERSION:
@@ -1854,7 +1879,7 @@ build_attachment_line(
 	if (namelen + len + info_len + 8 <= cCOLS)
 		namelen = cCOLS - 8 - info_len - len;
 
-	attach_line = build_attach_line(part, 0, info_len - 2, 0, NULL, charset);
+	attach_line = build_attach_line(part, 0, info_len - 2, SECTION_DEFAULT, NULL, charset);
 	snprintf(buf, sizeof(buf), "  %s", attach_line);
 	FreeIfNeeded(attach_line);
 
@@ -1894,14 +1919,13 @@ build_tree(
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	int prefix_ptr, tmpdepth;
 	int depth_level = 0;
+	int odd = (maxlen % 2) ? 0 : 1;
 	t_bool found = FALSE;
 	t_partl *lptr, *lptr2;
 
 	lptr2 = find_part(i);
 	prefix_ptr = depth * 2 - 1;
-	if (prefix_ptr > maxlen - 1 - !(maxlen % 2)) {
-		int odd = ((maxlen % 2) ? 0 : 1);
-
+	if (prefix_ptr > maxlen - 1 - odd) {
 		prefix_ptr -= maxlen - ++depth_level - 2 - odd;
 		while (prefix_ptr > maxlen - 2 - odd) {
 			if (depth_level < maxlen / 5)
@@ -2273,7 +2297,7 @@ process_part(
 	 * uuencoded parts must be read from the cooked article,
 	 * otherwise they might be additionally encoded with b64 or qp
 	 */
-	if (part->encoding == ENCODING_UUE)
+	if (part->encoding == ENCODING_UUE || part->encoding == ENCODING_YENC)
 		infile = art->cooked;
 	else
 		infile = art->raw;
@@ -2307,7 +2331,7 @@ process_part(
 		 * page.c:new_uue() sets offset to the 'begin ...' line
 		 * -> skip over the first line in uuencoded parts
 		 */
-		if (part->encoding == ENCODING_UUE && i == 0) {
+		if ((part->encoding == ENCODING_UUE || part->encoding == ENCODING_YENC) && i == 0) {
 			++line_count;
 			continue;
 		}
