@@ -3,10 +3,10 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-02-13
+ *  Updated   : 2025-12-09
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@
 
 int mark_offset = 0;
 
-static FILE* msglog = NULL;
+static FILE *msglog = NULL;
 
 #ifndef USE_CURSES
 	struct t_screen *screen;
@@ -616,8 +616,8 @@ show_progress(
 	time_t curr_time;
 	static char last_display[LEN];
 	static int last_ratio;
-	static t_artnum last_count;
-	static t_artnum last_total;
+	static t_artnum last_count = T_ARTNUM_CONST(0);
+	static t_artnum last_total = T_ARTNUM_CONST(0);
 	static time_t last_update;
 #if defined(HAVE_CLOCK_GETTIME) || defined(HAVE_GETTIMEOFDAY)
 	static int average;
@@ -628,8 +628,8 @@ show_progress(
 	int time_diff;
 	int secs_left;
 	t_artnum count_diff;
-	static struct t_tintime last_time;
-	static struct t_tintime this_time;
+	static struct t_tintime last_time = {0, 0};
+	static struct t_tintime this_time = {0, 0};
 #endif /* HAVE_CLOCK_GETTIME || HAVE_GETTIMEOFDAY */
 
 	if (batch_mode || count <= 0 || total <= 1)
@@ -670,9 +670,7 @@ show_progress(
 		time_diff = (int) (time_diff + ((this_time.tv_nsec - last_time.tv_nsec) / 1000));
 		count_diff = (count - last_count);
 
-		if (!count_diff) /* avoid div by zero */
-			++count_diff;
-
+		count_diff += (count_diff ? 0 : 1); /* avoid div by zero */
 		/*
 		 * Calculate a running average based on the last 20 samples. For the
 		 * first 19 samples just add all and divide by the number of samples.
@@ -774,21 +772,11 @@ open_msglog(
 	void)
 {
 	char logfile[PATH_LEN];
-	char serverdir[PATH_LEN];
 
 	if (msglog != NULL)
 		return;
 
-#ifdef NNTP_ABLE
-	if (read_news_via_nntp && !read_saved_news && nntp_tcp_port != IPPORT_NNTP)
-		snprintf(logfile, sizeof(logfile), "%s:%u", nntp_server, nntp_tcp_port);
-	else
-#endif /* NNTP_ABLE */
-	{
-		snprintf(logfile, sizeof(logfile), "%s", nntp_server);
-	}
-	joinpath(serverdir, sizeof(serverdir), rcdir, logfile);
-	joinpath(logfile, sizeof(logfile), serverdir, "msglog");
+	joinpath(logfile, sizeof(logfile), serverdir, MSGLOG_FILE);
 
 	if ((msglog = fopen(logfile, "w")) != NULL) {
 #ifdef HAVE_FCHMOD
@@ -812,7 +800,7 @@ close_msglog(
 }
 
 
-void
+static void
 log_formatted_msg(
 	const char *tag,
 	const char *msg)
