@@ -3,10 +3,10 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-05-27
+ *  Updated   : 2026-01-26
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,15 +77,16 @@ copy_offsets(
 	REGEX_NOFFSET dst_size,
 	struct regex_cache *re)
 {
-	REGEX_NOFFSET i;
-	REGEX_NOFFSET limit = 2 * regex_get_ovector_count(re);
+	REGEX_NOFFSET limit = MIN(dst_size, 2 * regex_get_ovector_count(re));
 	REGEX_SIZE *ovector = regex_get_ovector_pointer(re);
 
-	if (limit > dst_size)
-		limit = dst_size;
+	if (!limit)
+		return;
 
-	for (i = 0; i < limit; i++)
-		dst[i] = ovector[i];
+	do {
+		--limit;
+		dst[limit] = ovector[limit];
+	} while(limit);
 }
 
 
@@ -103,8 +104,8 @@ get_search_pattern(
 	int which_hist)
 {
 	static char tmpbuf[LEN];	/* Hold the last pattern used */
-	static char last_pattern[LEN];	/* last search pattern used; for repeated search */
-	static t_bool last_forward;
+	static char last_pattern[LEN] = { '\0' };	/* last search pattern used; for repeated search */
+	static t_bool last_forward = FALSE;
 
 	if (repeat) {
 		*forward = last_forward;
@@ -771,13 +772,14 @@ search_body(
 	t_bool forward_fake = TRUE;
 
 	if (!(buf = get_search_pattern(
-			&forward_fake,				/* we pass a dummy var since body search has no `forward' */
-			repeat,
-			_(txt_search_body),
-			_(txt_search_body),
-			&tinrc.default_search_art,
-			HIST_ART_SEARCH
-	))) return -1;
+		&forward_fake,				/* we pass a dummy var since body search has no `forward' */
+		repeat,
+		_(txt_search_body),
+		_(txt_search_body),
+		&tinrc.default_search_art,
+		HIST_ART_SEARCH
+	)))
+		return -1;
 
 	last_search = GLOBAL_SEARCH_BODY;	/* store last search type for repeated search */
 	total_cnt = curr_cnt = 0;			/* Reset global counter of articles done */
