@@ -3,7 +3,7 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2025-11-26
+ *  Updated   : 2026-02-05
  *  Notes     :
  *
  * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -353,9 +353,9 @@ selection_page(
 						if (past < 0) { /* not a valid date string, then take the leading digits as days */
 							p = tmp;
 							if (*p == '-' || *p == '+') /* ignore one leading sign if present */
-								p++;
+								++p;
 							if (isdigit((unsigned char) *p)) {
-								if ((days = strtol(p, NULL, 10)) > 0) {
+								if ((days = s2i(p, 0, 65535)) > 0) { /* Epoch + 65535 days == 2149-06-06T02:00:00+02:00 */
 									if (days <= (time((time_t *) 0) / DAY))
 										past = (time((time_t *) 0) - (days * DAY));
 								} else
@@ -622,20 +622,13 @@ selection_page(
 						}
 						free(ngcpy);
 						if (reject && *reject) {
-							i = strlen(reject);
-							if (*(reject + i - 1) == ',')
-								*(reject + i - 1) = '\0';
 							/* TODO: improve error/warningf (-n) message */
-							error_message((n & 0x07), _(txt_not_in_active_file), reject);
+							error_message((n & 0x07), _(txt_not_in_active_file), rem_tail_char(reject, ','));
 							free(reject);
 						}
 						if (new_list && *new_list) {
-							if (n) { /* without errors no need to use new_list */
-								i = strlen(new_list);
-								if (*(new_list + i - 1) == ',')
-									*(new_list + i - 1) = '\0';
-								strcpy(tinrc.default_post_newsgroups, new_list);
-							}
+							if (n) /* without errors no need to use new_list */
+								strcpy(tinrc.default_post_newsgroups, rem_tail_char(new_list, ','));
 							free(new_list);
 						}
 						if (!*buf) {
@@ -2246,10 +2239,10 @@ show_article_by_msgid(
 		 * why not also do hash_reclaim() here?
 		 */
 #endif /* 0 */
-		selmenu.curr = save_restore_curr_group(FALSE);
 		tin_errno = 0;
 		ret = LOOKUP_FAILED;
 	}
+	selmenu.curr = save_restore_curr_group(FALSE);
 
 	if (!ret) {
 		grpmenu.first = 0;
