@@ -6,7 +6,7 @@
  *  Updated   : 2025-03-16
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2025 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2026 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -344,10 +344,10 @@ parse_range(
 	while (*ptr && state != DONE) {
 		if (isdigit((unsigned char) *ptr)) {
 			if (state == FINDMAX) {
-				*range_end = atoi(ptr);
+				*range_end = strtol(ptr, NULL, 10);
 				state = DONE;
 			} else
-				*range_start = atoi(ptr);
+				*range_start = strtol(ptr, NULL, 10);
 			while (isdigit((unsigned char) *ptr))
 				++ptr;
 		} else {
