@@ -3,11 +3,11 @@
  *  Module    : tcurses.c
  *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-03-02
- *  Updated   : 2024-08-15
+ *  Updated   : 2026-01-29
  *  Notes     : This is a set of wrapper functions adapting the termcap
  *	             interface of tin to use SVr4 curses (e.g., ncurses).
  *
- * Copyright (c) 1997-2025 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2026 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -220,7 +220,7 @@ StartInverse(
 {
 	if (tinrc.inverse_okay) {
 #	ifdef HAVE_COLOR
-		if (use_color) {
+		if (use_color && (tinrc.col_invers_bg != -1 || tinrc.col_invers_fg != -1)) {
 			bcol(tinrc.col_invers_bg);
 			fcol(tinrc.col_invers_fg);
 		} else
@@ -456,7 +456,7 @@ word_highlight_string(
 	if (col < 0)
 		col = 0;
 	MoveCursor(row, col);
-	my_innstr(tmp, MIN((size > 0 ? size: 0), (LEN / 2) - 1));
+	my_innstr(tmp, MIN((size > 0 ? size : 0), (LEN / 2) - 1));
 
 #		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	tmp[MIN((size > 0 ? size : 0), LEN - 1)] = '\0';
@@ -467,10 +467,8 @@ word_highlight_string(
 #		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	/* safegurad against bogus regexps */
-	if (size > 1 && ((tmp[0] == '*' && tmp[size - 1] == '*') ||
-		 (tmp[0] == '/' && tmp[size - 1] == '/') ||
-		 (tmp[0] == '_' && tmp[size - 1] == '_') ||
-		 (tmp[0] == '-' && tmp[size - 1] == '-'))) {
+	if (size > 1 && tmp[size - 1] == tmp[0] &&
+		(tmp[0] == '*' || tmp[0] == '/' || tmp[0] == '_' || tmp[0] == '-')) {
 
 		switch (tinrc.word_h_display_marks) {
 			case 0:
@@ -867,10 +865,10 @@ my_innstr(
 	if (n < 0)
 		return 0;
 
-	buffer = my_malloc(sizeof(wchar_t) * (size_t) (n + 1));
+	buffer = my_malloc(sizeof(wchar_t) * (((size_t) n) + 1));
 
 	if (innwstr(buffer, n) != ERR) {
-		if ((len = wcstombs(str, buffer, (size_t) (2 * n))) == (size_t) (-1))
+		if ((len = wcstombs(str, buffer, 2 * ((size_t) n))) == (size_t) (-1))
 			len = 0;
 		str[len] = '\0';
 	}
