@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2024-12-23
+ *  Updated   : 2026-02-11
  *  Notes     :
  *
- * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,9 +57,9 @@ t_bool show_subject;
  * Local prototypes
  */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static wchar_t get_art_mark(struct t_article *art);
+	static wchar_t get_art_mark(const struct t_article *art);
 #else
-	static char get_art_mark(struct t_article *art);
+	static char get_art_mark(const struct t_article *art);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 static int enter_pager(int art, t_bool ignore_unavail, int level);
 static int thread_catchup(t_function func, const struct t_group *group);
@@ -95,7 +95,7 @@ static int ret_code = 0;		/* Set to < 0 when it is time to leave this menu */
 	static char
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 get_art_mark(
-	struct t_article *art)
+	const struct t_article *art)
 {
 	if (art->inrange) {
 		return tinrc.art_marked_inrange;
@@ -129,8 +129,8 @@ build_tline(
 	int l,
 	struct t_article *art)
 {
-	int w;
-	size_t gap, fill, i, len, len_start, len_end;
+	long i, fill;
+	size_t w, len, len_start, len_end, gap;
 	struct t_msgid *ptr;
 	char *buffer, *buf;
 	char *fmt = thrd_fmt.str;
@@ -194,7 +194,7 @@ build_tline(
 				free(buf);
 				break;
 
-			case 'F':	/* from */
+			case 'F':	/* from */ /* TODO: BiDi */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				get_author(TRUE, art, tmp, sizeof(tmp) - 1);
 
@@ -211,23 +211,24 @@ build_tline(
 				if (curr_group->attribute->show_author != SHOW_FROM_NONE) {
 					len_start = strwidth(buffer);
 					get_author(TRUE, art, buffer + strlen(buffer), thrd_fmt.len_from);
-					fill = thrd_fmt.len_from - (strwidth(buffer) - len_start);
-					gap = strlen(buffer);
-					for (i = 0; i < fill; i++)
-						buffer[gap + i] = ' ';
-					buffer[gap + fill] = '\0';
+					if ((fill = thrd_fmt.len_from - (strwidth(buffer) - len_start)) > 0) {
+						gap = strlen(buffer);
+						for (i = 0; i < fill; i++)
+							buffer[gap + i] = ' ';
+						buffer[gap + fill] = '\0';
+					}
 				}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				break;
 
-			case 'I':	/* initials */
+			case 'I':	/* initials */ /* TODO: BiDi? */
 				len = MIN(thrd_fmt.len_initials, sizeof(tmp) - 1);
 				get_initials(art, tmp, (int) len);
 				strcat(buffer, tmp);
 				w = strwidth(tmp);
-				if (w > 0 && len > (size_t) w) {
+				if (w > 0 && len > w) {
 					buf = buffer + strlen(buffer);
-					for (i = len - (size_t) w; i > 0; --i)
+					for (i = len - w; i > 0; --i)
 						*buf++ = ' ';
 					*buf = '\0';
 				}
@@ -277,9 +278,9 @@ build_tline(
 				tmp[len] = '\0';
 				strcat(buffer, tmp);
 				w = strwidth(tmp);
-				if (w > 0 && len > (size_t) w) {
+				if (w > 0 && len > w) {
 					buf = buffer + strlen(buffer);
-					for (i = len - (size_t) w; i > 0; --i)
+					for (i = len - w; i > 0; --i)
 						*buf++ = ' ';
 					*buf = '\0';
 				}
@@ -293,7 +294,7 @@ build_tline(
 				strcat(buffer, tin_ltoa(art->score, thrd_fmt.len_score));
 				break;
 
-			case 'T':	/* thread/subject */
+			case 'T':	/* thread/subject */ /* TODO: BiDi */
 				len = curr_group->attribute->show_author != SHOW_FROM_NONE ? thrd_fmt.len_subj : thrd_fmt.len_subj + thrd_fmt.len_from;
 				len_start = (size_t) strwidth(buffer);
 
@@ -313,13 +314,11 @@ build_tline(
 							/*
 							 * Copy in the subject up to where the author (if any) starts
 							 */
-							gap = (len - (len_end - len_start));
-
-							/*
-							 * Mutt-like thread tree. by sjpark@sparcs.kaist.ac.kr
-							 * Hide subject if same as parent's.
-							 */
-							if (gap > 0) {
+							if ((gap = (len - (len_end - len_start))) > 0) {
+								/*
+								 * Mutt-like thread tree. by sjpark@sparcs.kaist.ac.kr
+								 * Hide subject if same as parent's.
+								 */
 								for (ptr = art->refptr->parent; ptr && IS_EXPIRED(ptr); ptr = ptr->parent)
 									;
 								if (!(ptr && arts[ptr->article].subject == art->subject))
@@ -337,8 +336,8 @@ build_tline(
 #else
 								{
 									strncat(buffer, art->subject, gap);
+									buffer[len_end + gap] = '\0';
 								}
-								buffer[len_end + gap] = '\0';	/* Just in case */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 							}
 						}
@@ -349,8 +348,7 @@ build_tline(
 					case THREAD_MULTI:
 					case THREAD_PERC:
 						len_end = (size_t) strwidth(buffer);
-						gap = (len - (len_end - len_start));
-						if (gap > 0) {
+						if ((gap = (len - (len_end - len_start))) > 0)
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 							{
 								if ((wtmp = char2wchar_t(art->subject)) != NULL) {
@@ -358,17 +356,15 @@ build_tline(
 									free(wtmp);
 									if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
 										strcat(buffer, tmp);
-
 									free(wtmp2);
 								}
 							}
 #else
 							{
 								strncat(buffer, art->subject, gap);
+								buffer[len_end + gap] = '\0';
 							}
-							buffer[len_end + gap] = '\0';	/* Just in case */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-						}
 						break;
 
 					default:
@@ -376,11 +372,12 @@ build_tline(
 				}
 
 				/* pad out */
-				fill = (len - ((size_t) strwidth(buffer) - len_start));
-				gap = strlen(buffer);
-				for (i = 0; i < fill; i++)
-					buffer[gap + i] = ' ';
-				buffer[gap + fill] = '\0';
+				if ((fill = len + len_start - strwidth(buffer)) > 0) {
+					gap = strlen(buffer);
+					for (i = 0; i < fill; i++)
+						buffer[gap + i] = ' ';
+					buffer[gap + fill] = '\0';
+				}
 				break;
 
 			default:
@@ -637,8 +634,8 @@ thread_page(
 					if (invoke_editor(filter_file, filter_file_offset, NULL)) {
 						old_artnum = arts[find_response(thread_basenote, thdmenu.curr)].artnum;
 						unfilter_articles(group);
-						(void) read_filter_file(filter_file);
-						filter_articles(group);
+						if (read_filter_file(filter_file))
+							filter_articles(group);
 						make_threads(group, FALSE);
 						if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) { /* We have lost the thread */
 							ret_code = GRP_KILLED;
@@ -785,14 +782,17 @@ thread_page(
 #ifdef NNTP_ABLE
 				else {
 					const char *eyde = input_history[HIST_MESSAGE_ID][(hist_pos[HIST_MESSAGE_ID] - 1 + HIST_SIZE) % HIST_SIZE];
+					int old_thread_respnum = thread_respnum;
 
-					if (*eyde) {
+					if (eyde && *eyde) {
 						switch (show_article_by_msgid(eyde)) {
 							case LOOKUP_OK:
-								if (!index_group(curr_group))
+								if (!index_group(curr_group, TRUE))
 									ret_code = GRP_RETSELECT;
-								else
+								else {
+									fixup_thread(old_thread_respnum, FALSE);
 									show_thread_page();
+								}
 								break;
 
 							case LOOKUP_UNAVAIL:
@@ -899,7 +899,7 @@ thread_page(
 					else
 						draw_thread_arrow();
 
-					info_message(tagged ? _(txt_prefix_tagged) : _(txt_prefix_untagged), txt_article_sp[0]);
+					info_message(tagged ? _(txt_tagged_article_sp[0]) : _(txt_untagged_article));
 				}
 				break;
 
@@ -919,7 +919,7 @@ thread_page(
 			case MARK_THREAD_UNREAD:		/* mark thread as unread */
 				thd_mark_unread(group, base[thread_basenote]);
 				update_thread_page();
-				info_message(_(txt_marked_as_unread), _(txt_thread_upper));
+				info_message(_(txt_marked_thread_as_unread));
 				break;
 
 			case THREAD_SELECT_ARTICLE:		/* mark article as selected */
@@ -970,7 +970,8 @@ thread_page(
 
 			case GLOBAL_TOGGLE_INFO_LAST_LINE:		/* display subject in last line */
 				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
-				show_thread_page();
+				clear_message();
+				draw_thread_arrow();
 				break;
 
 			default:
@@ -1550,6 +1551,7 @@ make_prefix(
 	int prefix_ptr;
 	int depth = 0;
 	int depth_level = 0;
+	int odd = (maxlen % 2) ? 0 : 1;
 	struct t_msgid *ptr;
 
 	for (ptr = art->parent; ptr; ptr = ptr->parent)
@@ -1563,9 +1565,7 @@ make_prefix(
 
 	prefix_ptr = depth * 2 - 1;
 
-	if (prefix_ptr > maxlen - 1 - !(maxlen % 2)) {
-		int odd = ((maxlen % 2) ? 0 : 1);
-
+	if (prefix_ptr > maxlen - 1 - odd) {
 		prefix_ptr -= maxlen - ++depth_level - 2 - odd;
 
 		while (prefix_ptr > maxlen - 2 - odd) {
@@ -1590,6 +1590,7 @@ make_prefix(
 	for (ptr = art->parent; ptr && prefix_ptr > 1; ptr = ptr->parent) {
 		if (IS_EXPIRED(ptr))
 			continue;
+
 		buf[--prefix_ptr] = TREE_BLANK;
 		buf[--prefix_ptr] = (has_sibling(ptr) ? TREE_VERT : TREE_BLANK);
 	}
@@ -1728,6 +1729,8 @@ again:
 			return 0;
 
 		case GRP_GOTOTHREAD:		/* 'l' from pager */
+			if (which_thread(this_resp) == -1)
+				break;
 			show_thread_page();
 			move_to_item(which_response(this_resp));
 			return 0;
